@@ -1,32 +1,18 @@
-'''
-Classes related to producing and searching logs
-'''
+""" Test environment classes for Pacemaker's Cluster Test Suite (CTS)
+"""
 
-__copyright__='''
-Copyright (C) 2014 Andrew Beekhof <andrew@beekhof.net>
-Licensed under the GNU GPL.
-'''
+# Pacemaker targets compatibility with Python 2.7 and 3.2+
+from __future__ import print_function, unicode_literals, absolute_import, division
 
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+__copyright__ = "Copyright 2014-2018 Andrew Beekhof <andrew@beekhof.net>"
+__license__ = "GNU General Public License version 2 or later (GPLv2+) WITHOUT ANY WARRANTY"
 
 import sys, time, os, socket, random
 
 from cts.remote import *
+from cts.CTSvars import *
 
-class Environment:
+class Environment(object):
 
     def __init__(self, args):
         self.data = {}
@@ -45,7 +31,6 @@ class Environment:
         self["CIBfilename"] = None
         self["CIBResource"] = 0
         self["DoBSC"]    = 0
-        self["use_logd"] = 0
         self["oprofile"] = []
         self["warn-inactive"] = 0
         self["ListTests"] = 0
@@ -53,17 +38,14 @@ class Environment:
         self["LogWatcher"] = "any"
         self["SyslogFacility"] = "daemon"
         self["LogFileName"] = "/var/log/messages"
-        self["Schema"] = "pacemaker-2.0"
+        self["Schema"] = "pacemaker-3.0"
         self["Stack"] = "corosync"
         self["stonith-type"] = "external/ssh"
         self["stonith-params"] = "hostlist=all,livedangerously=yes"
         self["notification-agent"] = "/var/lib/pacemaker/notify.sh"
         self["notification-recipient"] = "/var/lib/pacemaker/notify.log"
         self["loop-minutes"] = 60
-        self["valgrind-prefix"] = None
-        self["valgrind-procs"] = "attrd cib crmd lrmd pengine stonith-ng"
-        self["valgrind-opts"] = """--leak-check=full --show-reachable=yes --trace-children=no --num-callers=25 --gen-suppressions=all --suppressions="""+CTSvars.CTS_home+"""/cts.supp"""
-
+        self["valgrind-procs"] = "pacemaker-attrd pacemaker-based pacemaker-controld pacemaker-execd pacemaker-fenced pacemaker-schedulerd"
         self["experimental-tests"] = 0
         self["container-tests"] = 0
         self["valgrind-tests"] = 0
@@ -83,8 +65,9 @@ class Environment:
         self.target = "localhost"
 
         self.parse_args(args)
-        self.discover()
-        self.validate()
+        if self["ListTests"] == 0:
+            self.validate()
+            self.discover()
 
     def SeedRandom(self, seed=None):
         if not seed:
@@ -103,7 +86,7 @@ class Environment:
             self.logger.debug("Environment["+key+"]:\t"+str(self[key]))
 
     def keys(self):
-        return self.data.keys()
+        return list(self.data.keys())
 
     def has_key(self, key):
         if key == "nodes":
@@ -162,50 +145,22 @@ class Environment:
 
     def set_stack(self, name):
         # Normalize stack names
-        if name == "heartbeat" or name == "lha":
-            self.data["Stack"] = "heartbeat"
-
-        elif name == "openais" or name == "ais"  or name == "whitetank":
-            self.data["Stack"] = "corosync (plugin v0)"
-
-        elif name == "corosync" or name == "cs" or name == "mcp":
-            self.data["Stack"] = "corosync 2.x"
-
-        elif name == "cman":
-            self.data["Stack"] = "corosync (cman)"
-
-        elif name == "v1":
-            self.data["Stack"] = "corosync (plugin v1)"
-
-        elif name == "v0":
-            self.data["Stack"] = "corosync (plugin v0)"
+        if name == "corosync" or name == "cs" or name == "mcp":
+            self.data["Stack"] = "corosync 2+"
 
         else:
             raise ValueError("Unknown stack: "+name)
-            sys.exit(1)
 
     def get_stack_short(self):
         # Create the Cluster Manager object
         if not "Stack" in self.data:
             return "unknown"
 
-        elif self.data["Stack"] == "heartbeat":
-            return "crm-lha"
-
-        elif self.data["Stack"] == "corosync 2.x":
+        elif self.data["Stack"] == "corosync 2+":
             if self["docker"]:
-                return "crm-mcp-docker"
+                return "crm-corosync-docker"
             else:
-                return "crm-mcp"
-
-        elif self.data["Stack"] == "corosync (cman)":
-            return "crm-cman"
-        
-        elif self.data["Stack"] == "corosync (plugin v1)":
-            return "crm-plugin-v1"
-        
-        elif self.data["Stack"] == "corosync (plugin v0)":
-            return "crm-plugin-v0"
+                return "crm-corosync"
 
         else:
             LogFactory().log("Unknown stack: "+self["stack"])
@@ -225,24 +180,46 @@ class Environment:
                 # default
                 self["syslogd"] = "rsyslog"
 
+    def disable_service(self, node, service):
+        if self["have_systemd"]:
+            # Systemd
+            return self.rsh(node, "systemctl disable %s" % service)
+
+        else:
+            # SYS-V
+            return self.rsh(node, "chkconfig %s off" % service)
+
+    def enable_service(self, node, service):
+        if self["have_systemd"]:
+            # Systemd
+            return self.rsh(node, "systemctl enable %s" % service)
+
+        else:
+            # SYS-V
+            return self.rsh(node, "chkconfig %s on" % service)
+
+    def service_is_enabled(self, node, service):
+        if self["have_systemd"]:
+            # Systemd
+
+            # With "systemctl is-enabled", we should check if the service is
+            # explicitly "enabled" instead of the return code. For example it returns
+            # 0 if the service is "static" or "indirect", but they don't really count
+            # as "enabled".
+            return not self.rsh(node, "systemctl is-enabled %s | grep enabled" % service)
+
+        else:
+            # SYS-V
+            return not self.rsh(node, "chkconfig --list | grep -e %s.*on" % service)
+
     def detect_at_boot(self):
         # Detect if the cluster starts at boot
         if not "at-boot" in self.data:
-            atboot = 0
-
-            if self["have_systemd"]:
-            # Systemd
-                atboot = atboot or not self.rsh(self.target, "systemctl is-enabled heartbeat.service")
-                atboot = atboot or not self.rsh(self.target, "systemctl is-enabled corosync.service")
-                atboot = atboot or not self.rsh(self.target, "systemctl is-enabled pacemaker.service")
-            else:
-                # SYS-V
-                atboot = atboot or not self.rsh(self.target, "chkconfig --list | grep -e corosync.*on -e heartbeat.*on -e pacemaker.*on")
-
-            self["at-boot"] = atboot
+            self["at-boot"] = self.service_is_enabled(self.target, "corosync") \
+                              or self.service_is_enabled(self.target, "pacemaker")
 
     def detect_ip_offset(self):
-        # Try to determin an offset for IPaddr resources
+        # Try to determine an offset for IPaddr resources
         if self["CIBResource"] and not "IPBase" in self.data:
             network=self.rsh(self.target, "ip addr | grep inet | grep -v -e link -e inet6 -e '/32' -e ' lo' | awk '{print $2}'", stdout=1).strip()
             self["IPBase"] = self.rsh(self.target, "nmap -sn -n %s | grep 'scan report' | awk '{print $NF}' | sed 's:(::' | sed 's:)::' | sort -V | tail -n 1" % network, stdout=1).strip()
@@ -258,7 +235,7 @@ class Environment:
                 self.logger.log("Defaulting to '%s', use --test-ip-base to override" % self["IPBase"])
 
     def filter_nodes(self):
-        if self["node-limit"] > 0:
+        if self['node-limit'] is not None and self["node-limit"] > 0:
             if len(self["nodes"]) > self["node-limit"]:
                 self.logger.log("Limiting the number of nodes configured=%d (max=%d)"
                                 %(len(self["nodes"]), self["node-limit"]))
@@ -283,14 +260,13 @@ class Environment:
         self["cts-master"] = master
 
         if not "have_systemd" in self.data:
-            self["have_systemd"] = not self.rsh(self.target, "systemctl list-units")
-        
+            self["have_systemd"] = not self.rsh(self.target,
+                                                "systemctl list-units",
+                                                silent=True)
         self.detect_syslog()
         self.detect_at_boot()
         self.detect_ip_offset()
 
-        self.validate()
-
     def parse_args(self, args):
         skipthis=None
 
@@ -477,14 +453,7 @@ class Environment:
                 dsh_file = "%s/.dsh/group/%s" % (os.environ['HOME'], args[i+1])
 
                 # Hacks to make my life easier
-                if args[i+1] == "r6":
-                    self["Stack"] = "cman"
-                    self["DoStonith"]=1
-                    self["stonith-type"] = "fence_xvm"
-                    self["stonith-params"] = "delay=0"
-                    self["IPBase"] = " fe80::1234:56:7890:4000"
-
-                elif args[i+1] == "virt1":
+                if args[i+1] == "virt1":
                     self["Stack"] = "corosync"
                     self["DoStonith"]=1
                     self["stonith-type"] = "fence_xvm"
@@ -543,9 +512,6 @@ class Environment:
                 skipthis=1
                 self["Schema"] = args[i+1]
 
-            elif args[i] == "--ais":
-                self["Stack"] = "openais"
-
             elif args[i] == "--at-boot" or args[i] == "--cluster-starts-at-boot":
                 skipthis=1
                 if args[i+1] == "1" or args[i+1] == "yes":
@@ -555,18 +521,9 @@ class Environment:
                 else:
                     self.usage(args[i+1])
 
-            elif args[i] == "--heartbeat" or args[i] == "--lha":
-                self["Stack"] = "heartbeat"
-
-            elif args[i] == "--hae":
-                self["Stack"] = "openais"
-                self["Schema"] = "hae"
-
             elif args[i] == "--stack":
                 if args[i+1] == "fedora" or args[i+1] == "fedora-17" or args[i+1] == "fedora-18":
                     self["Stack"] = "corosync"
-                elif args[i+1] == "rhel-6":
-                    self["Stack"] = "cman"
                 elif args[i+1] == "rhel-7":
                     self["Stack"] = "corosync"
                 else:
@@ -640,7 +597,7 @@ class Environment:
         print("\t [--nodes 'node list']        list of cluster nodes separated by whitespace")
         print("\t [--group | -g 'name']        use the nodes listed in the named DSH group (~/.dsh/groups/$name)")
         print("\t [--limit-nodes max]          only use the first 'max' cluster nodes supplied with --nodes")
-        print("\t [--stack (v0|v1|cman|corosync|heartbeat|openais)]    which cluster stack is installed")
+        print("\t [--stack corosync]           which cluster stack is installed")
         print("\t [--list-tests]               list the valid tests")
         print("\t [--benchmark]                add the timing information")
         print("\t ")
@@ -682,11 +639,12 @@ class Environment:
         print("\t [--yes | -y]                 continue to run cts when there is an interaction whether to continue running pacemaker-cts")
         print("\t ")
         print("\t Example: ")
-        print("\t    python sys.argv[0] -g virt1 --stack cs -r --stonith ssh --schema pacemaker-1.0 500")
+        # @PYTHON@ would be better here but not worth making file this a .in
+        print("\t    python sys.argv[0] -g virt1 -r --stonith ssh --schema pacemaker-2.0 500")
 
         sys.exit(status)
 
-class EnvFactory:
+class EnvFactory(object):
     instance = None
     def __init__(self):
         pass
