@@ -6,7 +6,20 @@
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
+#ifndef CIB_SECRETS__H
+#define CIB_SECRETS__H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * load parameters from an ini file (cib_secrets.c)
  */
 int replace_secret_params(const char *rsc_id, GHashTable *params);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif
