@@ -8,6 +8,10 @@
 #ifndef CRM_COMMON_MAINLOOP__H
 #  define CRM_COMMON_MAINLOOP__H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
  * \brief Wrappers for and extensions to glib mainloop
@@ -67,30 +71,6 @@ struct ipc_client_callbacks {
 qb_ipcs_service_t *mainloop_add_ipc_server(const char *name, enum qb_ipc_type type,
                                            struct qb_ipcs_service_handlers *callbacks);
 
-/*!
- * \brief Start server-side API end-point, hooked into the internal event loop
- *
- * \param[in] name    name of the IPC end-point ("address" for the client)
- * \param[in] type    selects libqb's IPC back-end (or use #QB_IPC_NATIVE)
- * \param[in] callbacks  defines libqb's IPC service-level handlers
- * \param[in] priority  priority relative to other events handled in the
- *                      abstract handling loop, use #QB_LOOP_MED when unsure
- *
- * \return libqb's opaque handle to the created service abstraction
- *
- * \note For portability concerns, do not use this function if you keep
- *       \p priority as #QB_LOOP_MED, stick with #mainloop_add_ipc_server
- *       (with exactly such semantics) instead (once you link with this new
- *       symbol employed, you can't downgrade the library freely anymore).
- *
- * \note The intended effect will only get fully reflected when run-time
- *       linked to patched libqb: https://github.com/ClusterLabs/libqb/pull/352
- */
-qb_ipcs_service_t *mainloop_add_ipc_server_with_prio(const char *name,
-                                                    enum qb_ipc_type type,
-                                                    struct qb_ipcs_service_handlers *callbacks,
-                                                    enum qb_loop_priority prio);
-
 void mainloop_del_ipc_server(qb_ipcs_service_t * server);
 
 mainloop_io_t *mainloop_add_ipc_client(const char *name, int priority, size_t max_size,
@@ -140,4 +120,8 @@ void pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms,
 
 #  define G_PRIORITY_MEDIUM (G_PRIORITY_HIGH/2)
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
