@@ -1,23 +1,17 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #ifndef CRM_COMMON_XML__H
 #  define CRM_COMMON_XML__H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
  * \brief Wrappers for and extensions to libxml2
@@ -32,11 +26,12 @@
 #  include <errno.h>
 #  include <fcntl.h>
 
-#  include <crm/crm.h>
-
 #  include <libxml/tree.h>
 #  include <libxml/xpath.h>
 
+#  include <crm/crm.h>
+#  include <crm/common/nvpair.h>
+
 /* Define compression parameters for IPC messages
  *
  * Compression costs a LOT, so we don't want to do it unless we're hitting
@@ -53,15 +48,6 @@
 
 gboolean add_message_xml(xmlNode * msg, const char *field, xmlNode * xml);
 xmlNode *get_message_xml(xmlNode * msg, const char *field);
-GHashTable *xml2list(xmlNode * parent);
-
-xmlNode *crm_create_nvpair_xml(xmlNode *parent, const char *id,
-                               const char *name, const char *value);
-
-void hash2nvpair(gpointer key, gpointer value, gpointer user_data);
-void hash2field(gpointer key, gpointer value, gpointer user_data);
-void hash2metafield(gpointer key, gpointer value, gpointer user_data);
-void hash2smartfield(gpointer key, gpointer value, gpointer user_data);
 
 xmlDoc *getDocPtr(xmlNode * node);
 
@@ -87,47 +73,6 @@ void fix_plus_plus_recursive(xmlNode * target);
  */
 xmlNode *create_xml_node(xmlNode * parent, const char *name);
 
-/*
- * Make a copy of name and value and use the copied memory to create
- * an attribute for node.
- *
- * If node, name or value are NULL, nothing is done.
- *
- * If name or value are an empty string, nothing is done.
- *
- * Returns FALSE on failure and TRUE on success.
- *
- */
-const char *crm_xml_add(xmlNode * node, const char *name, const char *value);
-
-const char *crm_xml_replace(xmlNode * node, const char *name, const char *value);
-
-const char *crm_xml_add_int(xmlNode * node, const char *name, int value);
-
-/*!
- * \brief Add a boolean attribute to an XML object
- *
- * Add an attribute with the value XML_BOOLEAN_TRUE or XML_BOOLEAN_FALSE
- * as appropriate to an XML object.
- *
- * \param[in,out] node   XML object to add attribute to
- * \param[in]     name   Name of attribute to add
- * \param[in]     value  Boolean whose value will be tested
- *
- * \return Pointer to newly created XML attribute's content, or NULL on error
- */
-static inline const char *
-crm_xml_add_boolean(xmlNode *node, const char *name, gboolean value)
-{
-    return crm_xml_add(node, name, (value? "true" : "false"));
-}
-
-/*
- * Unlink the node and set its doc pointer to NULL so free_xml()
- * will act appropriately
- */
-void unlink_xml_node(xmlNode * node);
-
 /*
  *
  */
@@ -176,9 +121,7 @@ xmlNode *subtract_xml_object(xmlNode * parent, xmlNode * left, xmlNode * right,
 
 gboolean can_prune_leaf(xmlNode * xml_node);
 
-void print_xml_diff(FILE * where, xmlNode * diff);
-
-gboolean apply_xml_diff(xmlNode * old, xmlNode * diff, xmlNode ** new);
+gboolean apply_xml_diff(xmlNode *old_xml, xmlNode *diff, xmlNode **new_xml);
 
 /*
  * Searching & Modifying
@@ -198,41 +141,15 @@ int find_xml_children(xmlNode ** children, xmlNode * root,
                       const char *tag, const char *field, const char *value,
                       gboolean search_matches);
 
-int crm_element_value_int(xmlNode * data, const char *name, int *dest);
-char *crm_element_value_copy(xmlNode * data, const char *name);
-int crm_element_value_const_int(const xmlNode * data, const char *name, int *dest);
-const char *crm_element_value_const(const xmlNode * data, const char *name);
 xmlNode *get_xpath_object(const char *xpath, xmlNode * xml_obj, int error_level);
 xmlNode *get_xpath_object_relative(const char *xpath, xmlNode * xml_obj, int error_level);
 
 static inline const char *
-crm_element_name(xmlNode *xml)
+crm_element_name(const xmlNode *xml)
 {
     return xml? (const char *)(xml->name) : NULL;
 }
 
-const char *crm_element_value(xmlNode * data, const char *name);
-
-/*!
- * \brief Copy an element from one XML object to another
- *
- * \param[in]     obj1     Source XML
- * \param[in,out] obj2     Destination XML
- * \param[in]     element  Name of element to copy
- *
- * \return Pointer to copied value (from source)
- */
-static inline const char *
-crm_copy_xml_element(xmlNode *obj1, xmlNode *obj2, const char *element)
-{
-    const char *value = crm_element_value(obj1, element);
-
-    crm_xml_add(obj2, element, value);
-    return value;
-}
-
-void xml_validate(const xmlNode * root);
-
 gboolean xml_has_children(const xmlNode * root);
 
 char *calculate_on_disk_digest(xmlNode * local_cib);
@@ -263,20 +180,20 @@ gboolean validate_xml_verbose(xmlNode * xml_blob);
  *                           the validity of the result; if no validation
  *                           success was observed against possibly multiple
  *                           schemas, the value is less or equal the result
- *                           of <tt>get_schema_version</tt> applied on the
- *                           input \p xml_blob value (unless that function
- *                           maps it to -1, then 0 would be used instead)
+ *                           of \c get_schema_version applied on the input
+ *                           \p xml_blob value (unless that function maps it
+ *                           to -1, then 0 would be used instead)
  * \param[in]     max        When \p transform is positive, this allows to
  *                           set upper boundary schema (per its index in the
- *                           global schemas table) beyond which its forbidden
+ *                           global schemas table) beyond which it's forbidden
  *                           to update by the means of XSLT transformation
  * \param[in]     transform  Whether to employ XSLT-based transformation so
- *                           as allow overcoming possible incompatibilities
+ *                           as to allow overcoming possible incompatibilities
  *                           between major schema versions (see above)
  * \param[in]     to_logs    If true, output notable progress info to
  *                           internal log streams; if false, to stderr
  *
- * \return <tt>pcmk_ok</tt> if no non-recoverable error encountered (up to
+ * \return \c pcmk_ok if no non-recoverable error encountered (up to
  *         caller to evaluate if the update satisfies the requirements
  *         per returned \p best value), negative value carrying the reason
  *         otherwise
@@ -293,65 +210,53 @@ void crm_xml_init(void);
 void crm_xml_cleanup(void);
 
 static inline xmlNode *
-__xml_first_child(xmlNode * parent)
+__xml_first_child(const xmlNode *parent)
 {
-    xmlNode *child = NULL;
+    xmlNode *child = parent? parent->children : NULL;
 
-    if (parent) {
-        child = parent->children;
-        while (child && child->type == XML_TEXT_NODE) {
-            child = child->next;
-        }
+    while (child && (child->type == XML_TEXT_NODE)) {
+        child = child->next;
     }
     return child;
 }
 
 static inline xmlNode *
-__xml_next(xmlNode * child)
+__xml_next(const xmlNode *child)
 {
-    if (child) {
+    xmlNode *next = child? child->next : NULL;
+
+    while (next && (next->type == XML_TEXT_NODE)) {
+        next = next->next;
+    }
+    return next;
+}
+
+static inline xmlNode *
+__xml_first_child_element(const xmlNode *parent)
+{
+    xmlNode *child = parent? parent->children : NULL;
+
+    while (child && (child->type != XML_ELEMENT_NODE)) {
         child = child->next;
-        while (child && child->type == XML_TEXT_NODE) {
-            child = child->next;
-        }
     }
     return child;
 }
 
 static inline xmlNode *
-__xml_first_child_element(xmlNode * parent)
+__xml_next_element(const xmlNode *child)
 {
-    xmlNode *child = NULL;
+    xmlNode *next = child? child->next : NULL;
 
-    if (parent) {
-        child = parent->children;
+    while (next && (next->type != XML_ELEMENT_NODE)) {
+        next = next->next;
     }
-
-    while (child) {
-        if(child->type == XML_ELEMENT_NODE) {
-            return child;
-        }
-        child = child->next;
-    }
-    return NULL;
-}
-
-static inline xmlNode *
-__xml_next_element(xmlNode * child)
-{
-    while (child) {
-        child = child->next;
-        if(child && child->type == XML_ELEMENT_NODE) {
-            return child;
-        }
-    }
-    return NULL;
+    return next;
 }
 
 void free_xml(xmlNode * child);
 
-xmlNode *first_named_child(xmlNode * parent, const char *name);
-xmlNode *crm_next_same_xml(xmlNode *sibling);
+xmlNode *first_named_child(const xmlNode *parent, const char *name);
+xmlNode *crm_next_same_xml(const xmlNode *sibling);
 
 xmlNode *sorted_xml(xmlNode * input, xmlNode * parent, gboolean recursive);
 xmlXPathObjectPtr xpath_search(xmlNode * xml_top, const char *path);
@@ -379,7 +284,8 @@ bool xml_acl_filtered_copy(const char *user, xmlNode* acl_source, xmlNode *xml,
 bool xml_tracking_changes(xmlNode * xml);
 bool xml_document_dirty(xmlNode *xml);
 void xml_track_changes(xmlNode * xml, const char *user, xmlNode *acl_source, bool enforce_acls);
-void xml_calculate_changes(xmlNode * old, xmlNode * new); /* For comparing two documents after the fact */
+void xml_calculate_changes(xmlNode *old_xml, xmlNode *new_xml);
+void xml_calculate_significant_changes(xmlNode *old_xml, xmlNode *new_xml);
 void xml_accept_changes(xmlNode * xml);
 void xml_log_changes(uint8_t level, const char *function, xmlNode *xml);
 void xml_log_patchset(uint8_t level, const char *function, xmlNode *xml);
@@ -404,4 +310,8 @@ void crm_xml_set_id(xmlNode *xml, const char *format, ...)
  */
 void crm_destroy_xml(gpointer data);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
