@@ -8,6 +8,10 @@
 #ifndef CRM__H
 #  define CRM__H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
  * \brief A dumping ground
@@ -23,7 +27,7 @@
 
 #  include <libxml/tree.h>
 
-#  define CRM_FEATURE_SET		"3.0.14"
+#  define CRM_FEATURE_SET		"3.1.0"
 
 #  define EOS		'\0'
 #  define DIMOF(a)	((int) (sizeof(a)/sizeof(a[0])) )
@@ -46,30 +50,24 @@ extern char *crm_system_name;
 
 /* *INDENT-OFF* */
 
-/* Clean these up at some point, some probably should be runtime options */
-#  define SOCKET_LEN	1024
-#  define APPNAME_LEN	256
-#  define MAX_IPC_FAIL	5
+// Used for some internal IPC timeouts (maybe should be configurable option)
 #  define MAX_IPC_DELAY   120
 
-#  define DAEMON_RESPAWN_STOP 100
+// How we represent "infinite" scores
+#  define CRM_SCORE_INFINITY    1000000
+#  define CRM_INFINITY_S        "INFINITY"
+#  define CRM_PLUS_INFINITY_S   "+" CRM_INFINITY_S
+#  define CRM_MINUS_INFINITY_S  "-" CRM_INFINITY_S
 
-#  define MSG_LOG			1
-#  define DOT_FSA_ACTIONS		1
-#  define DOT_ALL_FSA_INPUTS	1
-/* #define FSA_TRACE		1 */
-
-/* This header defines INFINITY, but it might be defined elsewhere as well
- * (e.g. math.h), so undefine it first. This, of course, complicates any attempt
- * to use the other definition in any code that includes this header.
+/* @COMPAT API < 2.0.0 Deprecated "infinity" aliases
  *
- * @TODO: Rename our constant (which will break API backward compatibility).
+ * INFINITY might be defined elsewhere (e.g. math.h), so undefine it first.
+ * This, of course, complicates any attempt to use the other definition in any
+ * code that includes this header.
  */
 #  undef INFINITY
-
 #  define INFINITY_S        "INFINITY"
 #  define MINUS_INFINITY_S "-INFINITY"
-
 #  define INFINITY        1000000
 
 /* Sub-systems */
@@ -133,24 +131,6 @@ extern char *crm_system_name;
 #  define CRM_OP_RM_NODE_CACHE "rm_node_cache"
 #  define CRM_OP_MAINTENANCE_NODES "maintenance_nodes"
 
-/* @COMPAT: These symbols are deprecated and not used by Pacemaker,
- * but they are kept for public API backward compatibility.
- */
-#  define CRM_OP_DIE		    "die_no_respawn"
-#  define CRM_OP_RETRIVE_CIB	"retrieve_cib"
-#  define CRM_OP_HBEAT		    "dc_beat"
-#  define CRM_OP_ABORT		    "abort"
-#  define CRM_OP_EVENTCC		"event_cc"
-#  define CRM_OP_TEABORT		"te_abort"
-#  define CRM_OP_TEABORTED	    "te_abort_confirmed"
-#  define CRM_OP_TE_HALT		"te_halt"
-#  define CRM_OP_TECOMPLETE	    "te_complete"
-#  define CRM_OP_TETIMEOUT	    "te_timeout"
-#  define CRM_OP_TRANSITION	    "transition"
-#  define CRM_OP_DEBUG_UP		"debug_inc" /* unused since 1.1.8 */
-#  define CRM_OP_DEBUG_DOWN     "debug_dec" /* unused since 1.1.8 */
-#  define CRM_OP_NODES_PROBED	"probe_nodes_complete"
-
 /* Possible cluster membership states */
 #  define CRMD_JOINSTATE_DOWN           "down"
 #  define CRMD_JOINSTATE_PENDING        "pending"
@@ -211,13 +191,17 @@ typedef GList *GListPtr;
 
 #  include <crm/common/logging.h>
 #  include <crm/common/util.h>
-#  include <crm/error.h>
 
-static inline const char *crm_action_str(const char *task, int interval) {
-    if(safe_str_eq(task, RSC_STATUS) && !interval) {
+static inline const char *
+crm_action_str(const char *task, guint interval_ms) {
+    if(safe_str_eq(task, RSC_STATUS) && !interval_ms) {
         return "probe";
     }
     return task;
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
