@@ -11,15 +11,6 @@
 #  include <crm/common/ipc.h>
 #  include <crm/common/xml.h>
 
-enum st_device_flags
-{
-    st_device_supports_list   = 0x0001,
-    st_device_supports_status = 0x0002,
-    st_device_supports_reboot = 0x0004,
-    st_device_supports_parameter_plug = 0x0008,
-    st_device_supports_parameter_port = 0x0010,
-};
-
 struct stonith_action_s;
 typedef struct stonith_action_s stonith_action_t;
 
@@ -28,19 +19,16 @@ stonith_action_t *stonith_action_create(const char *agent,
                                         const char *victim,
                                         uint32_t victim_nodeid,
                                         int timeout,
-                                        GHashTable * device_args,
-                                        GHashTable * port_map,
-                                        const char * host_arg);
+                                        GHashTable * device_args, GHashTable * port_map);
 void stonith__destroy_action(stonith_action_t *action);
 void stonith__action_result(stonith_action_t *action, int *rc, char **output,
                             char **error_output);
 
-int
+GPid
 stonith_action_execute_async(stonith_action_t * action,
                              void *userdata,
                              void (*done) (GPid pid, int rc, const char *output,
-                                           gpointer user_data),
-                             void (*fork_cb) (GPid pid, gpointer user_data));
+                                           gpointer user_data));
 
 int stonith__execute(stonith_action_t *action);
 
@@ -55,8 +43,6 @@ xmlNode *create_device_registration_xml(const char *id,
                                         stonith_key_value_t *params,
                                         const char *rsc_provides);
 
-long long stonith__device_parameter_flags(xmlNode *metadata);
-
 #  define ST_LEVEL_MAX 10
 
 #  define F_STONITH_CLIENTID      "st_clientid"
@@ -66,12 +52,10 @@ long long stonith__device_parameter_flags(xmlNode *metadata);
 #  define F_STONITH_OPERATION     "st_op"
 #  define F_STONITH_TARGET        "st_target"
 #  define F_STONITH_REMOTE_OP_ID  "st_remote_op"
-#  define F_STONITH_REMOTE_OP_ID_RELAY  "st_remote_op_relay"
 #  define F_STONITH_RC            "st_rc"
 /*! Timeout period per a device execution */
 #  define F_STONITH_TIMEOUT       "st_timeout"
 #  define F_STONITH_TOLERANCE     "st_tolerance"
-#  define F_STONITH_DELAY         "st_delay"
 /*! Action specific timeout period returned in query of fencing devices. */
 #  define F_STONITH_ACTION_TIMEOUT       "st_action_timeout"
 /*! Host in query result is not allowed to run this action */
@@ -90,7 +74,6 @@ long long stonith__device_parameter_flags(xmlNode *metadata);
 #  define F_STONITH_CALLBACK_TOKEN    "st_async_id"
 #  define F_STONITH_CLIENTNAME        "st_clientname"
 #  define F_STONITH_CLIENTNODE        "st_clientnode"
-#  define F_STONITH_NOTIFY_TYPE       "st_notify_type"
 #  define F_STONITH_NOTIFY_ACTIVATE   "st_notify_activate"
 #  define F_STONITH_NOTIFY_DEACTIVATE "st_notify_deactivate"
 #  define F_STONITH_DELEGATE      "st_delegate"
@@ -117,7 +100,6 @@ long long stonith__device_parameter_flags(xmlNode *metadata);
 #  define T_STONITH_TIMEOUT_VALUE "st-async-timeout-value"
 #  define T_STONITH_NOTIFY    "st_notify"
 
-#  define STONITH_ATTR_ARGMAP    "pcmk_arg_map"
 #  define STONITH_ATTR_HOSTARG   "pcmk_host_argument"
 #  define STONITH_ATTR_HOSTMAP   "pcmk_host_map"
 #  define STONITH_ATTR_HOSTLIST  "pcmk_host_list"
@@ -133,17 +115,12 @@ long long stonith__device_parameter_flags(xmlNode *metadata);
 #  define STONITH_OP_QUERY       "st_query"
 #  define STONITH_OP_FENCE       "st_fence"
 #  define STONITH_OP_RELAY       "st_relay"
-#  define STONITH_OP_CONFIRM     "st_confirm"
 #  define STONITH_OP_DEVICE_ADD      "st_device_register"
 #  define STONITH_OP_DEVICE_DEL      "st_device_remove"
-#  define STONITH_OP_DEVICE_METADATA "st_device_metadata"
 #  define STONITH_OP_FENCE_HISTORY   "st_fence_history"
 #  define STONITH_OP_LEVEL_ADD       "st_level_add"
 #  define STONITH_OP_LEVEL_DEL       "st_level_remove"
 
-#  define stonith_channel            "st_command"
-#  define stonith_channel_callback   "st_callback"
-
 #  define STONITH_WATCHDOG_AGENT  "#watchdog"
 
 #  ifdef HAVE_STONITH_STONITH_H
@@ -161,7 +138,7 @@ int stonith__list_rhcs_agents(stonith_key_value_t **devices);
 int stonith__rhcs_metadata(const char *agent, int timeout, char **output);
 bool stonith__agent_is_rhcs(const char *agent);
 int stonith__rhcs_validate(stonith_t *st, int call_options, const char *target,
-                           const char *agent, GHashTable *params, const char *host_arg,
+                           const char *agent, GHashTable *params,
                            int timeout, char **output, char **error_output);
 
 #endif
