@@ -1,31 +1,24 @@
 /*
- * Copyright (c) 2012 David Vossel <davidvossel@gmail.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * Copyright 2012-2018 David Vossel <davidvossel@gmail.com>
  *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #ifndef LRMD__H
 #  define LRMD__H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
- * \brief Local Resource Manager
+ * \brief Resource agent executor
  * \ingroup lrmd
  */
 #include <stdbool.h>
+#include <crm_config.h>
 #include <crm/services.h>
 
 typedef struct lrmd_s lrmd_t;
@@ -49,7 +42,7 @@ typedef struct lrmd_key_value_s {
 #define LRMD_MIN_PROTOCOL_VERSION "1.0"
 
 /* *INDENT-OFF* */
-#define DEFAULT_REMOTE_KEY_LOCATION "/etc/pacemaker/authkey"
+#define DEFAULT_REMOTE_KEY_LOCATION PACEMAKER_CONFIG_DIR "/authkey"
 #define ALT_REMOTE_KEY_LOCATION "/etc/corosync/authkey"
 #define DEFAULT_REMOTE_PORT 3121
 #define DEFAULT_REMOTE_USERNAME "lrmd"
@@ -63,7 +56,6 @@ typedef struct lrmd_key_value_s {
 #define F_LRMD_REMOTE_MSG_ID    "lrmd_remote_msg_id"
 #define F_LRMD_CALLBACK_TOKEN   "lrmd_async_id"
 #define F_LRMD_CALLID           "lrmd_callid"
-#define F_LRMD_CANCEL_CALLID    "lrmd_cancel_callid"
 #define F_LRMD_CALLOPTS         "lrmd_callopt"
 #define F_LRMD_CALLDATA         "lrmd_calldata"
 #define F_LRMD_RC               "lrmd_rc"
@@ -88,7 +80,6 @@ typedef struct lrmd_key_value_s {
 #define F_LRMD_RSC_EXIT_REASON  "lrmd_rsc_exit_reason"
 #define F_LRMD_RSC_START_DELAY  "lrmd_rsc_start_delay"
 #define F_LRMD_RSC_INTERVAL     "lrmd_rsc_interval"
-#define F_LRMD_RSC_METADATA     "lrmd_rsc_metadata_res"
 #define F_LRMD_RSC_DELETED      "lrmd_rsc_deleted"
 #define F_LRMD_RSC              "lrmd_rsc"
 
@@ -96,7 +87,6 @@ typedef struct lrmd_key_value_s {
 #define F_LRMD_ALERT_PATH         "lrmd_alert_path"
 #define F_LRMD_ALERT              "lrmd_alert"
 
-#define LRMD_OP_RSC_CHK_REG       "lrmd_rsc_check_register"
 #define LRMD_OP_RSC_REG           "lrmd_rsc_register"
 #define LRMD_OP_RSC_EXEC          "lrmd_rsc_exec"
 #define LRMD_OP_RSC_CANCEL        "lrmd_rsc_cancel"
@@ -107,6 +97,7 @@ typedef struct lrmd_key_value_s {
 #define LRMD_OP_NEW_CLIENT        "lrmd_rsc_new_client"
 #define LRMD_OP_CHECK             "lrmd_check"
 #define LRMD_OP_ALERT_EXEC        "lrmd_alert_exec"
+#define LRMD_OP_GET_RECURRING     "lrmd_get_recurring"
 
 #define LRMD_IPC_OP_NEW           "new"
 #define LRMD_IPC_OP_DESTROY       "destroy"
@@ -121,7 +112,6 @@ typedef struct lrmd_key_value_s {
 #define F_LRMD_IPC_IPC_SERVER   "lrmd_ipc_server"
 #define F_LRMD_IPC_SESSION      "lrmd_ipc_session"
 #define F_LRMD_IPC_CLIENT       "lrmd_ipc_client"
-#define F_LRMD_IPC_PROXY_NODE   "lrmd_ipc_proxy_node"
 #define F_LRMD_IPC_USER         "lrmd_ipc_user"
 #define F_LRMD_IPC_MSG          "lrmd_ipc_msg"
 #define F_LRMD_IPC_MSG_ID       "lrmd_ipc_msg_id"
@@ -131,15 +121,16 @@ typedef struct lrmd_key_value_s {
 #define T_LRMD_REPLY     "lrmd_reply"
 #define T_LRMD_NOTIFY    "lrmd_notify"
 #define T_LRMD_IPC_PROXY "lrmd_ipc_proxy"
+#define T_LRMD_RSC_OP    "lrmd_rsc_op"
 /* *INDENT-ON* */
 
 /*!
- * \brief Create a new local lrmd connection
+ * \brief Create a new connection to the local executor
  */
 lrmd_t *lrmd_api_new(void);
 
 /*!
- * \brief Create a new remote lrmd connection using tls backend
+ * \brief Create a new TLS connection to a remote executor
  *
  * \param nodename  name of remote node identified with this connection
  * \param server    name of server to connect to
@@ -152,7 +143,7 @@ lrmd_t *lrmd_remote_api_new(const char *nodename, const char *server, int port);
 /*!
  * \brief Use after lrmd_poll returns 1 to read and dispatch a message
  *
- * \param[in,out] lrmd  lrmd connection object
+ * \param[in,out] lrmd  Executor connection object
  *
  * \return TRUE if connection is still up, FALSE if disconnected
  */
@@ -168,7 +159,7 @@ bool lrmd_dispatch(lrmd_t * lrmd);
 int lrmd_poll(lrmd_t * lrmd, int timeout);
 
 /*!
- * \brief Destroy lrmd object
+ * \brief Destroy executor connection object
  */
 void lrmd_api_delete(lrmd_t * lrmd);
 lrmd_key_value_t *lrmd_key_value_add(lrmd_key_value_t * kvp, const char *key, const char *value);
@@ -217,7 +208,7 @@ typedef struct lrmd_event_data_s {
     /*! The operation's timeout period in ms. */
     int timeout;
     /*! The operation's recurring interval in ms. */
-    int interval;
+    guint interval_ms;
     /*! The operation's start delay value in ms. */
     int start_delay;
     /*! This operation that just completed is on a deleted rsc. */
@@ -225,7 +216,7 @@ typedef struct lrmd_event_data_s {
 
     /*! The executed ra return code mapped to OCF */
     enum ocf_exitcode rc;
-    /*! The lrmd status returned for exec_complete events */
+    /*! The executor status returned for exec_complete events */
     int op_status;
     /*! stdout from resource agent operation */
     const char *output;
@@ -254,22 +245,28 @@ typedef struct lrmd_event_data_s {
     const char *exit_reason;
 } lrmd_event_data_t;
 
-lrmd_event_data_t *lrmd_new_event(const char *rsc_id, const char *task,
-                                  int interval_ms);
 lrmd_event_data_t *lrmd_copy_event(lrmd_event_data_t * event);
 void lrmd_free_event(lrmd_event_data_t * event);
 
 typedef struct lrmd_rsc_info_s {
     char *id;
     char *type;
-    char *class;
+    char *standard;
     char *provider;
 } lrmd_rsc_info_t;
 
+typedef struct lrmd_op_info_s {
+    char *rsc_id;
+    char *action;
+    char *interval_ms_s;
+    char *timeout_ms_s;
+} lrmd_op_info_t;
+
 lrmd_rsc_info_t *lrmd_new_rsc_info(const char *rsc_id, const char *standard,
                                    const char *provider, const char *type);
 lrmd_rsc_info_t *lrmd_copy_rsc_info(lrmd_rsc_info_t * rsc_info);
 void lrmd_free_rsc_info(lrmd_rsc_info_t * rsc_info);
+void lrmd_free_op_info(lrmd_op_info_t *op_info);
 
 typedef void (*lrmd_event_callback) (lrmd_event_data_t * event);
 
@@ -283,7 +280,7 @@ void lrmd_key_value_freeall(lrmd_key_value_t * head);
 
 typedef struct lrmd_api_operations_s {
     /*!
-     * \brief Connect from the lrmd.
+     * \brief Connect to an executor
      *
      * \retval 0, success
      * \retval negative error code on failure
@@ -291,7 +288,7 @@ typedef struct lrmd_api_operations_s {
     int (*connect) (lrmd_t * lrmd, const char *client_name, int *fd);
 
     /*!
-     * \brief Establish an connection to lrmd, don't block while connecting.
+     * \brief Initiate an executor connection without blocking
      * \note this function requires the use of mainloop.
      *
      * \note The is returned using the event callback.
@@ -311,7 +308,7 @@ typedef struct lrmd_api_operations_s {
     int (*is_connected) (lrmd_t * lrmd);
 
     /*!
-     * \brief Poke lrmd connection to verify it is still capable of serving requests
+     * \brief Poke executor connection to verify it is still capable of serving requests
      * \note The response comes in the form of a poke event to the callback. 
      *
      * \retval 0, wait for response in callback
@@ -320,7 +317,7 @@ typedef struct lrmd_api_operations_s {
     int (*poke_connection) (lrmd_t * lrmd);
 
     /*!
-     * \brief Disconnect from the lrmd.
+     * \brief Disconnect from the executor.
      *
      * \retval 0, success
      * \retval negative error code on failure
@@ -328,7 +325,7 @@ typedef struct lrmd_api_operations_s {
     int (*disconnect) (lrmd_t * lrmd);
 
     /*!
-     * \brief Register a resource with the lrmd.
+     * \brief Register a resource with the executor.
      *
      * \note Synchronous, guaranteed to occur in daemon before function returns.
      *
@@ -337,7 +334,7 @@ typedef struct lrmd_api_operations_s {
      */
     int (*register_rsc) (lrmd_t * lrmd,
                          const char *rsc_id,
-                         const char *class,
+                         const char *standard,
                          const char *provider, const char *agent, enum lrmd_call_options options);
 
     /*!
@@ -350,7 +347,15 @@ typedef struct lrmd_api_operations_s {
                                       const char *rsc_id, enum lrmd_call_options options);
 
     /*!
-     * \brief Unregister a resource from the lrmd.
+     * \brief Retrieve registered recurring operations
+     *
+     * \return pcmk_ok on success, -errno otherwise
+     */
+    int (*get_recurring_ops) (lrmd_t *lrmd, const char *rsc_id, int timeout_ms,
+                              enum lrmd_call_options options, GList **output);
+
+    /*!
+     * \brief Unregister a resource from the executor.
      *
      * \note All pending and recurring operations will be cancelled
      *       automatically.
@@ -366,7 +371,7 @@ typedef struct lrmd_api_operations_s {
     int (*unregister_rsc) (lrmd_t * lrmd, const char *rsc_id, enum lrmd_call_options options);
 
     /*!
-     * \brief Sets the callback to receive lrmd events on.
+     * \brief Set a callback for executor events
      */
     void (*set_callback) (lrmd_t * lrmd, lrmd_event_callback callback);
 
@@ -386,7 +391,7 @@ typedef struct lrmd_api_operations_s {
      * \retval negative error code on failure
      */
     int (*exec) (lrmd_t * lrmd, const char *rsc_id, const char *action, const char *userdata,   /* userdata string given back in event notification */
-                 int interval,  /* ms */
+                 guint interval_ms,
                  int timeout,   /* ms */
                  int start_delay,       /* ms */
                  enum lrmd_call_options options, lrmd_key_value_t * params);    /* ownership of params is given up to api here */
@@ -404,7 +409,7 @@ typedef struct lrmd_api_operations_s {
      * \note For each resource, cancel operations and exec operations
      *       are processed in the order they are received.
      *       It is safe to assume that for a single resource, a cancel
-     *       will occur in the lrmd before an exec if the client's cancel
+     *       will occur in the executor before an exec if the client's cancel
      *       api call occurs before the exec api call.
      *
      *       It is not however safe to assume any operation on one resource will
@@ -414,47 +419,48 @@ typedef struct lrmd_api_operations_s {
      * \retval 0, cancel command sent.
      * \retval negative error code on failure
      */
-    int (*cancel) (lrmd_t * lrmd, const char *rsc_id, const char *action, int interval);
+    int (*cancel) (lrmd_t *lrmd, const char *rsc_id, const char *action,
+                   guint interval_ms);
 
     /*!
      * \brief Get resource metadata for a specified resource agent
      *
-     * \param[in]  lrmd      LRMD connection (unused)
-     * \param[in]  class     Resource agent class
+     * \param[in]  lrmd      Executor connection (unused)
+     * \param[in]  standard  Resource agent class
      * \param[in]  provider  Resource agent provider
      * \param[in]  agent     Resource agent type
      * \param[out] output    Metadata will be stored here (must not be NULL)
-     * \param[in]  options   Options to use with any LRMD API calls (unused)
+     * \param[in]  options   Options to use with any executor API calls (unused)
      *
      * \note Caller is responsible for freeing output. This call is currently
      *       always synchronous (blocking), and always done directly by the
-     *       library (not via the LRMD connection). This means that it is based
-     *       on the local host environment, even if the lrmd connection is to a
+     *       library (not via the executor connection). This means that it is based
+     *       on the local host environment, even if the executor connection is to a
      *       remote node, so (for most resource agent classes) this will fail if
      *       the agent is not installed locally. This also means that, if an
      *       external agent must be executed, it will be executed by the
-     *       caller's user, not the lrmd's.
-     * \todo Add a metadata call to the LRMD API and let the server handle this.
+     *       caller's user, not the executor's.
+     * \todo Add a metadata call to the executor API and let the server handle this.
      *
      * \retval lrmd_ok success
      * \retval negative error code on failure
      */
     int (*get_metadata) (lrmd_t * lrmd,
-                         const char *class,
+                         const char *standard,
                          const char *provider,
                          const char *agent, char **output, enum lrmd_call_options options);
 
     /*!
      * \brief Retrieve a list of installed resource agents.
      *
-     * \note if class is not provided, all known agents will be returned
+     * \note if standard is not provided, all known agents will be returned
      * \note list must be freed using lrmd_list_freeall()
      *
      * \retval num items in list on success
      * \retval negative error code on failure
      */
-    int (*list_agents) (lrmd_t * lrmd, lrmd_list_t ** agents, const char *class,
-                        const char *provider);
+    int (*list_agents) (lrmd_t * lrmd, lrmd_list_t ** agents,
+                        const char *standard, const char *provider);
 
     /*!
      * \brief Retrieve a list of resource agent providers
@@ -523,7 +529,7 @@ typedef struct lrmd_api_operations_s {
 
 struct lrmd_s {
     lrmd_api_operations_t *cmds;
-    void *private;
+    void *lrmd_private;
 };
 
 static inline const char *
@@ -548,4 +554,8 @@ lrmd_event_type2str(enum lrmd_callback_event type)
     return "unknown";
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
