@@ -71,7 +71,6 @@ bool pe_can_fence(pe_working_set_t *data_set, node_t *node);
 
 int merge_weights(int w1, int w2);
 void add_hash_param(GHashTable * hash, const char *name, const char *value);
-void append_hashtable(gpointer key, gpointer value, gpointer user_data);
 
 char *native_parameter(resource_t * rsc, node_t * node, gboolean create, const char *name,
                        pe_working_set_t * data_set);
@@ -86,44 +85,35 @@ void native_add_running(resource_t * rsc, node_t * node, pe_working_set_t * data
 gboolean native_unpack(resource_t * rsc, pe_working_set_t * data_set);
 gboolean group_unpack(resource_t * rsc, pe_working_set_t * data_set);
 gboolean clone_unpack(resource_t * rsc, pe_working_set_t * data_set);
-gboolean master_unpack(resource_t * rsc, pe_working_set_t * data_set);
 gboolean container_unpack(resource_t * rsc, pe_working_set_t * data_set);
 
-resource_t *native_find_rsc(resource_t * rsc, const char *id, node_t * node, int flags);
+resource_t *native_find_rsc(resource_t *rsc, const char *id, const node_t *node,
+                            int flags);
 
 gboolean native_active(resource_t * rsc, gboolean all);
 gboolean group_active(resource_t * rsc, gboolean all);
 gboolean clone_active(resource_t * rsc, gboolean all);
-gboolean master_active(resource_t * rsc, gboolean all);
 gboolean container_active(resource_t * rsc, gboolean all);
 
 void native_print(resource_t * rsc, const char *pre_text, long options, void *print_data);
 void group_print(resource_t * rsc, const char *pre_text, long options, void *print_data);
 void clone_print(resource_t * rsc, const char *pre_text, long options, void *print_data);
-void master_print(resource_t * rsc, const char *pre_text, long options, void *print_data);
 void container_print(resource_t * rsc, const char *pre_text, long options, void *print_data);
 
 void native_free(resource_t * rsc);
 void group_free(resource_t * rsc);
 void clone_free(resource_t * rsc);
-void master_free(resource_t * rsc);
 void container_free(resource_t * rsc);
 
 enum rsc_role_e native_resource_state(const resource_t * rsc, gboolean current);
 enum rsc_role_e group_resource_state(const resource_t * rsc, gboolean current);
 enum rsc_role_e clone_resource_state(const resource_t * rsc, gboolean current);
-enum rsc_role_e master_resource_state(const resource_t * rsc, gboolean current);
 enum rsc_role_e container_resource_state(const resource_t * rsc, gboolean current);
 
-void pe__count_common(pe_resource_t *rsc);
-void pe__count_bundle(pe_resource_t *rsc);
-
 gboolean common_unpack(xmlNode * xml_obj, resource_t ** rsc, resource_t * parent,
                        pe_working_set_t * data_set);
 void common_free(resource_t * rsc);
 
-extern pe_working_set_t *pe_dataset;
-
 extern node_t *node_copy(const node_t *this_node);
 extern time_t get_effective_time(pe_working_set_t * data_set);
 
@@ -146,14 +136,14 @@ pe_action_t *pe__clear_failcount(pe_resource_t *rsc, pe_node_t *node,
 
 /* Functions for finding/counting a resource's active nodes */
 
-pe_node_t *pe__find_active_on(const resource_t *rsc,
+pe_node_t *pe__find_active_on(const pe_resource_t *rsc,
                               unsigned int *count_all,
                               unsigned int *count_clean);
-pe_node_t *pe__find_active_requires(const resource_t *rsc,
+pe_node_t *pe__find_active_requires(const pe_resource_t *rsc,
                                     unsigned int *count);
 
 static inline pe_node_t *
-pe__current_node(const resource_t *rsc)
+pe__current_node(const pe_resource_t *rsc)
 {
     return pe__find_active_on(rsc, NULL, NULL);
 }
@@ -162,7 +152,6 @@ pe__current_node(const resource_t *rsc)
 /* Binary like operators for lists of nodes */
 extern void node_list_exclude(GHashTable * list, GListPtr list2, gboolean merge_scores);
 extern GListPtr node_list_dup(GListPtr list, gboolean reset, gboolean filter);
-extern GListPtr node_list_from_hash(GHashTable * hash, gboolean reset, gboolean filter);
 
 extern GHashTable *node_hash_from_list(GListPtr list);
 static inline gpointer
@@ -178,28 +167,21 @@ extern action_t *get_pseudo_op(const char *name, pe_working_set_t * data_set);
 extern gboolean order_actions(action_t * lh_action, action_t * rh_action, enum pe_ordering order);
 
 GHashTable *node_hash_dup(GHashTable * hash);
-extern GListPtr node_list_and(GListPtr list1, GListPtr list2, gboolean filter);
-
-extern GListPtr node_list_xor(GListPtr list1, GListPtr list2, gboolean filter);
-
-extern GListPtr node_list_minus(GListPtr list1, GListPtr list2, gboolean filter);
-
-extern void pe_free_shallow(GListPtr alist);
-extern void pe_free_shallow_adv(GListPtr alist, gboolean with_data);
 
 /* Printing functions for debug */
 extern void print_node(const char *pre_text, node_t * node, gboolean details);
 
+extern void print_resource(int log_level, const char *pre_text, resource_t * rsc, gboolean details);
+
+extern void dump_node_scores_worker(int level, const char *file, const char *function, int line,
+                                    resource_t * rsc, const char *comment, GHashTable * nodes);
+
 extern void dump_node_capacity(int level, const char *comment, node_t * node);
 extern void dump_rsc_utilization(int level, const char *comment, resource_t * rsc, node_t * node);
 
-void pe__show_node_weights_as(const char *file, const char *function,
-                              int line, bool to_log, pe_resource_t *rsc,
-                              const char *comment, GHashTable *nodes);
-
-#define pe__show_node_weights(level, rsc, text, nodes)              \
-        pe__show_node_weights_as(__FILE__, __FUNCTION__, __LINE__,  \
-                                 (level), (rsc), (text), (nodes))
+#  define dump_node_scores(level, rsc, text, nodes) do {		\
+        dump_node_scores_worker(level, __FILE__, __FUNCTION__, __LINE__, rsc, text, nodes); \
+    } while(0)
 
 /* Sorting functions */
 extern gint sort_rsc_priority(gconstpointer a, gconstpointer b);
@@ -329,7 +311,7 @@ typedef struct op_digest_cache_s {
 op_digest_cache_t *rsc_action_digest_cmp(resource_t * rsc, xmlNode * xml_op, node_t * node,
                                          pe_working_set_t * data_set);
 
-pe_action_t *pe_fence_op(pe_node_t * node, const char *op, bool optional, const char *reason, bool priority_delay, pe_working_set_t * data_set);
+action_t *pe_fence_op(node_t * node, const char *op, bool optional, const char *reason, pe_working_set_t * data_set);
 void trigger_unfencing(
     resource_t * rsc, node_t *node, const char *reason, action_t *dependency, pe_working_set_t * data_set);
 
@@ -346,7 +328,7 @@ gboolean add_tag_ref(GHashTable * tags, const char * tag_name,  const char * obj
 
 void print_rscs_brief(GListPtr rsc_list, const char * pre_text, long options,
                       void * print_data, gboolean print_all);
-void pe_fence_node(pe_working_set_t * data_set, pe_node_t * node, const char *reason, bool priority_delay);
+void pe_fence_node(pe_working_set_t * data_set, node_t * node, const char *reason);
 
 node_t *pe_create_node(const char *id, const char *uname, const char *type,
                        const char *score, pe_working_set_t * data_set);
@@ -355,8 +337,12 @@ void common_print(resource_t * rsc, const char *pre_text, const char *name, node
 resource_t *find_container_child(const resource_t *bundle, const node_t *node);
 bool container_fix_remote_addr(resource_t *rsc);
 const char *container_fix_remote_addr_in(resource_t *rsc, xmlNode *xml, const char *field);
-const char *pe_node_attribute_calculated(pe_node_t *node, const char *name, resource_t *rsc);
+const char *pe_node_attribute_calculated(const pe_node_t *node,
+                                         const char *name,
+                                         const resource_t *rsc);
 const char *pe_node_attribute_raw(pe_node_t *node, const char *name);
+bool pe__is_universal_clone(pe_resource_t *rsc,
+                            pe_working_set_t *data_set);
 void pe__add_param_check(xmlNode *rsc_op, pe_resource_t *rsc, pe_node_t *node,
                          enum pe_check_parameters, pe_working_set_t *data_set);
 void pe__foreach_param_check(pe_working_set_t *data_set,
@@ -364,8 +350,4 @@ void pe__foreach_param_check(pe_working_set_t *data_set,
                                         enum pe_check_parameters,
                                         pe_working_set_t*));
 void pe__free_param_checks(pe_working_set_t *data_set);
-
-bool pe__shutdown_requested(pe_node_t *node);
-bool pe__resource_is_disabled(pe_resource_t *rsc);
-
 #endif
