@@ -1,23 +1,17 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #ifndef PENGINE_STATUS__H
 #  define PENGINE_STATUS__H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*!
  * \file
  * \brief Cluster status and scheduling
@@ -29,22 +23,19 @@
 #  include <crm/common/iso8601.h>
 #  include <crm/pengine/common.h>
 
-typedef struct node_s pe_node_t;
-typedef struct node_s node_t;
-typedef struct pe_action_s action_t;
+typedef struct pe_node_s pe_node_t;
 typedef struct pe_action_s pe_action_t;
-typedef struct resource_s resource_t;
-typedef struct ticket_s ticket_t;
+typedef struct pe_resource_s pe_resource_t;
+typedef struct pe_working_set_s pe_working_set_t;
 
-// forward-compatible with Pacemaker 2.0.0
-typedef struct resource_s pe_resource_t;
+#  include <crm/pengine/complex.h>
 
-typedef enum no_quorum_policy_e {
+enum pe_quorum_policy {
     no_quorum_freeze,
     no_quorum_stop,
     no_quorum_ignore,
     no_quorum_suicide
-} no_quorum_policy_t;
+};
 
 enum node_type {
     node_ping,
@@ -52,23 +43,24 @@ enum node_type {
     node_remote
 };
 
+//! \deprecated will be removed in a future release
 enum pe_restart {
     pe_restart_restart,
     pe_restart_ignore
 };
 
+//! Determine behavior of pe_find_resource_with_flags()
 enum pe_find {
-    pe_find_renamed  = 0x001, // match resource ID or LRM history ID
-    pe_find_anon     = 0x002, // match base name of anonymous clone instances
-    pe_find_clone    = 0x004, // match only clone instances
-    pe_find_current  = 0x008, // match resource active on specified node
-    pe_find_inactive = 0x010, // match resource not running anywhere
-    pe_find_any      = 0x020, // match base name of any clone instance
+    pe_find_renamed  = 0x001, //!< match resource ID or LRM history ID
+    pe_find_anon     = 0x002, //!< match base name of anonymous clone instances
+    pe_find_clone    = 0x004, //!< match only clone instances
+    pe_find_current  = 0x008, //!< match resource active on specified node
+    pe_find_inactive = 0x010, //!< match resource not running anywhere
+    pe_find_any      = 0x020, //!< match base name of any clone instance
 };
 
 #  define pe_flag_have_quorum           0x00000001ULL
 #  define pe_flag_symmetric_cluster     0x00000002ULL
-#  define pe_flag_is_managed_default    0x00000004ULL
 #  define pe_flag_maintenance_mode      0x00000008ULL
 
 #  define pe_flag_stonith_enabled       0x00000010ULL
@@ -92,29 +84,20 @@ enum pe_find {
 #  define pe_flag_sanitized             0x00200000ULL
 #  define pe_flag_stdout                0x00400000ULL
 
-//! Don't count total, disabled and blocked resource instances
-#  define pe_flag_no_counts             0x00800000ULL
-
-/*! Skip deprecated code that is kept solely for backward API compatibility.
- * (Internal code should always set this.)
- */
-#  define pe_flag_no_compat             0x01000000ULL
-
-typedef struct pe_working_set_s {
+struct pe_working_set_s {
     xmlNode *input;
     crm_time_t *now;
 
     /* options extracted from the input */
     char *dc_uuid;
-    node_t *dc_node;
+    pe_node_t *dc_node;
     const char *stonith_action;
     const char *placement_strategy;
 
     unsigned long long flags;
 
     int stonith_timeout;
-    int default_resource_stickiness;
-    no_quorum_policy_t no_quorum_policy;
+    enum pe_quorum_policy no_quorum_policy;
 
     GHashTable *config_hash;
     GHashTable *tickets;
@@ -152,9 +135,7 @@ typedef struct pe_working_set_s {
 
     GList *param_check; // History entries that need to be checked
     GList *stop_needed; // Containers that need stop actions
-    int ninstances;     // Total number of resource instances
-    int priority_fencing_delay; // Priority fencing delay
-} pe_working_set_t;
+};
 
 enum pe_check_parameters {
     /* Clear fail count if parameters changed for un-expired start or monitor
@@ -168,11 +149,12 @@ enum pe_check_parameters {
     pe_check_active,
 };
 
-struct node_shared_s {
+struct pe_node_shared_s {
     const char *id;
     const char *uname;
+    enum node_type type;
 
-    /* @TODO convert these flags (and the ones at the end) into a bitfield */
+    /* @TODO convert these flags into a bitfield */
     gboolean online;
     gboolean standby;
     gboolean standby_onfail;
@@ -182,40 +164,31 @@ struct node_shared_s {
     gboolean shutdown;
     gboolean expected_up;
     gboolean is_dc;
-
-    int num_resources;
-    GListPtr running_rsc;       /* resource_t* */
-    GListPtr allocated_rsc;     /* resource_t* */
-
-    resource_t *remote_rsc;
-
-    GHashTable *attrs;          /* char* => char* */
-    enum node_type type;
-
-    GHashTable *utilization;
-
-    /*! cache of calculated rsc digests for this node. */
-    GHashTable *digest_cache;
-
     gboolean maintenance;
     gboolean rsc_discovery_enabled;
     gboolean remote_requires_reset;
     gboolean remote_was_fenced;
     gboolean remote_maintenance; /* what the remote-rsc is thinking */
     gboolean unpacked;
-    int priority; // calculated based on the priority of resources running on the node
+
+    int num_resources;
+    pe_resource_t *remote_rsc;
+    GListPtr running_rsc;       /* pe_resource_t* */
+    GListPtr allocated_rsc;     /* pe_resource_t* */
+
+    GHashTable *attrs;          /* char* => char* */
+    GHashTable *utilization;
+    GHashTable *digest_cache;   //!< cache of calculated resource digests
 };
 
-struct node_s {
+struct pe_node_s {
     int weight;
     gboolean fixed;
     int count;
-    struct node_shared_s *details;
+    struct pe_node_shared_s *details;
     int rsc_discover_mode;
 };
 
-#  include <crm/pengine/complex.h>
-
 #  define pe_rsc_orphan                     0x00000001ULL
 #  define pe_rsc_managed                    0x00000002ULL
 #  define pe_rsc_block                      0x00000004ULL
@@ -224,35 +197,30 @@ struct node_s {
 #  define pe_rsc_notify                     0x00000010ULL
 #  define pe_rsc_unique                     0x00000020ULL
 #  define pe_rsc_fence_device               0x00000040ULL
+#  define pe_rsc_promotable                 0x00000080ULL
 
 #  define pe_rsc_provisional                0x00000100ULL
 #  define pe_rsc_allocating                 0x00000200ULL
 #  define pe_rsc_merging                    0x00000400ULL
-#  define pe_rsc_munging                    0x00000800ULL
 
-#  define pe_rsc_try_reload                 0x00001000ULL
 #  define pe_rsc_reload                     0x00002000ULL
 #  define pe_rsc_allow_remote_remotes       0x00004000ULL
 
 #  define pe_rsc_failed                     0x00010000ULL
-#  define pe_rsc_shutdown                   0x00020000ULL
 #  define pe_rsc_runnable                   0x00040000ULL
 #  define pe_rsc_start_pending              0x00080000ULL
 
 #  define pe_rsc_starting                   0x00100000ULL
 #  define pe_rsc_stopping                   0x00200000ULL
-#  define pe_rsc_migrating                  0x00400000ULL
 #  define pe_rsc_allow_migrate              0x00800000ULL
 
 #  define pe_rsc_failure_ignored            0x01000000ULL
-#  define pe_rsc_unexpectedly_running       0x02000000ULL
 #  define pe_rsc_maintenance                0x04000000ULL
 #  define pe_rsc_is_container               0x08000000ULL
 
 #  define pe_rsc_needs_quorum               0x10000000ULL
 #  define pe_rsc_needs_fencing              0x20000000ULL
 #  define pe_rsc_needs_unfencing            0x40000000ULL
-#  define pe_rsc_have_unfencing             0x80000000ULL // obsolete (not set or used by cluster)
 
 enum pe_graph_flags {
     pe_graph_none = 0x00000,
@@ -269,7 +237,6 @@ enum pe_action_flags {
     pe_action_print_always = 0x00008,
 
     pe_action_have_node_attrs = 0x00010,
-    pe_action_failure_is_fatal = 0x00020, /* no longer used, here for API compatibility */
     pe_action_implied_by_stonith = 0x00040,
     pe_action_migrate_runnable =   0x00080,
 
@@ -288,44 +255,55 @@ enum pe_action_flags {
 };
 /* *INDENT-ON* */
 
-struct resource_s {
+struct pe_resource_s {
     char *id;
     char *clone_name;
     xmlNode *xml;
     xmlNode *orig_xml;
     xmlNode *ops_xml;
 
-    resource_t *parent;
-    void *variant_opaque;
+    pe_working_set_t *cluster;
+    pe_resource_t *parent;
+
     enum pe_obj_types variant;
+    void *variant_opaque;
     resource_object_functions_t *fns;
     resource_alloc_functions_t *cmds;
 
     enum rsc_recovery_type recovery_type;
-    enum pe_restart restart_type;
+
+    // @TODO only pe_restart_restart is of interest, so merge into flags
+    enum pe_restart restart_type; //!< \deprecated will be removed in future release
 
     int priority;
     int stickiness;
     int sort_index;
     int failure_timeout;
-    int effective_priority;
     int migration_threshold;
-
-    gboolean is_remote_node;
+    guint remote_reconnect_ms;
+    char *pending_task;
 
     unsigned long long flags;
 
-    // These fields should be treated as internal to Pacemaker
-    GListPtr rsc_cons_lhs;      /* rsc_colocation_t* */
-    GListPtr rsc_cons;          /* rsc_colocation_t* */
-    GListPtr rsc_location;      // List of pe__location_t*
-    GListPtr actions;           /* action_t*         */
-    GListPtr rsc_tickets;       /* rsc_ticket*       */
+    // @TODO merge these into flags
+    gboolean is_remote_node;
+    gboolean exclusive_discover;
 
-    node_t *allocated_to;
-    GListPtr running_on;        /* node_t*   */
-    GHashTable *known_on;       /* node_t*   */
-    GHashTable *allowed_nodes;  /* node_t*   */
+    //!@{
+    //! This field should be treated as internal to Pacemaker
+    GListPtr rsc_cons_lhs;      // List of rsc_colocation_t*
+    GListPtr rsc_cons;          // List of rsc_colocation_t*
+    GListPtr rsc_location;      // List of pe__location_t*
+    GListPtr actions;           // List of pe_action_t*
+    GListPtr rsc_tickets;       // List of rsc_ticket*
+    //!@}
+
+    pe_node_t *allocated_to;
+    pe_node_t *partial_migration_target;
+    pe_node_t *partial_migration_source;
+    GListPtr running_on;        /* pe_node_t*   */
+    GHashTable *known_on;       /* pe_node_t*   */
+    GHashTable *allowed_nodes;  /* pe_node_t*   */
 
     enum rsc_role_e role;
     enum rsc_role_e next_role;
@@ -334,23 +312,12 @@ struct resource_s {
     GHashTable *parameters;
     GHashTable *utilization;
 
-    GListPtr children;          /* resource_t*   */
-    GListPtr dangling_migrations;       /* node_t*       */
+    GListPtr children;          /* pe_resource_t*   */
+    GListPtr dangling_migrations;       /* pe_node_t*       */
 
-    node_t *partial_migration_target;
-    node_t *partial_migration_source;
-
-    resource_t *container;
+    pe_resource_t *container;
     GListPtr fillers;
 
-    char *pending_task;
-
-    const char *isolation_wrapper;
-    gboolean exclusive_discover;
-    int remote_reconnect_interval;
-
-    pe_working_set_t *cluster;
-
     pe_node_t *pending_node;    // Node on which pending_task is happening
 
 #if ENABLE_VERSIONED_ATTRS
@@ -370,26 +337,20 @@ struct pe_action_s {
     int id;
     int priority;
 
-    resource_t *rsc;
-    node_t *node;
+    pe_resource_t *rsc;
+    pe_node_t *node;
     xmlNode *op_entry;
 
     char *task;
     char *uuid;
     char *cancel_task;
+    char *reason;
 
     enum pe_action_flags flags;
     enum rsc_start_requirement needs;
     enum action_fail_response on_fail;
     enum rsc_role_e fail_role;
 
-    action_t *pre_notify;
-    action_t *pre_notified;
-    action_t *post_notify;
-    action_t *post_notified;
-
-    int seen_count;
-
     GHashTable *meta;
     GHashTable *extra;
 
@@ -413,29 +374,27 @@ struct pe_action_s {
      * to be considered runnable */ 
     int required_runnable_before;
 
-    GListPtr actions_before;    /* action_wrapper_t* */
-    GListPtr actions_after;     /* action_wrapper_t* */
+    GListPtr actions_before;    /* pe_action_wrapper_t* */
+    GListPtr actions_after;     /* pe_action_wrapper_t* */
 
     /* Some of the above fields could be moved to the details,
      * except for API backward compatibility.
      */
     void *action_details; // varies by type of action
-
-    char *reason;
 };
 
-struct ticket_s {
+typedef struct pe_ticket_s {
     char *id;
     gboolean granted;
     time_t last_granted;
     gboolean standby;
     GHashTable *state;
-};
+} pe_ticket_t;
 
-typedef struct tag_s {
+typedef struct pe_tag_s {
     char *id;
     GListPtr refs;
-} tag_t;
+} pe_tag_t;
 
 enum pe_link_state {
     pe_link_not_dumped,
@@ -493,28 +452,27 @@ enum pe_ordering {
 };
 /* *INDENT-ON* */
 
-typedef struct action_wrapper_s action_wrapper_t;
-struct action_wrapper_s {
+typedef struct pe_action_wrapper_s {
     enum pe_ordering type;
     enum pe_link_state state;
-    action_t *action;
-};
+    pe_action_t *action;
+} pe_action_wrapper_t;
 
-const char *rsc_printable_id(resource_t *rsc);
+const char *rsc_printable_id(pe_resource_t *rsc);
 gboolean cluster_status(pe_working_set_t * data_set);
 pe_working_set_t *pe_new_working_set(void);
 void pe_free_working_set(pe_working_set_t *data_set);
 void set_working_set_defaults(pe_working_set_t * data_set);
 void cleanup_calculations(pe_working_set_t * data_set);
 void pe_reset_working_set(pe_working_set_t *data_set);
-resource_t *pe_find_resource(GListPtr rsc_list, const char *id_rh);
-resource_t *pe_find_resource_with_flags(GListPtr rsc_list, const char *id, enum pe_find flags);
-node_t *pe_find_node(GListPtr node_list, const char *uname);
-node_t *pe_find_node_id(GListPtr node_list, const char *id);
-node_t *pe_find_node_any(GListPtr node_list, const char *id, const char *uname);
+pe_resource_t *pe_find_resource(GListPtr rsc_list, const char *id_rh);
+pe_resource_t *pe_find_resource_with_flags(GListPtr rsc_list, const char *id, enum pe_find flags);
+pe_node_t *pe_find_node(GListPtr node_list, const char *uname);
+pe_node_t *pe_find_node_id(GListPtr node_list, const char *id);
+pe_node_t *pe_find_node_any(GListPtr node_list, const char *id, const char *uname);
 GListPtr find_operations(const char *rsc, const char *node, gboolean active_filter,
                          pe_working_set_t * data_set);
-int pe_bundle_replicas(const resource_t *rsc);
+int pe_bundle_replicas(const pe_resource_t *rsc);
 #if ENABLE_VERSIONED_ATTRS
 pe_rsc_action_details_t *pe_rsc_action_details(pe_action_t *action);
 #endif
@@ -527,9 +485,9 @@ pe_rsc_action_details_t *pe_rsc_action_details(pe_action_t *action);
  * \return TRUE if resource is clone, FALSE otherwise
  */
 static inline bool
-pe_rsc_is_clone(resource_t *rsc)
+pe_rsc_is_clone(pe_resource_t *rsc)
 {
-    return rsc && ((rsc->variant == pe_clone) || (rsc->variant == pe_master));
+    return rsc && (rsc->variant == pe_clone);
 }
 
 /*!
@@ -540,7 +498,7 @@ pe_rsc_is_clone(resource_t *rsc)
  * \return TRUE if resource is unique clone, FALSE otherwise
  */
 static inline bool
-pe_rsc_is_unique_clone(resource_t *rsc)
+pe_rsc_is_unique_clone(pe_resource_t *rsc)
 {
     return pe_rsc_is_clone(rsc) && is_set(rsc->flags, pe_rsc_unique);
 }
@@ -553,15 +511,35 @@ pe_rsc_is_unique_clone(resource_t *rsc)
  * \return TRUE if resource is anonymous clone, FALSE otherwise
  */
 static inline bool
-pe_rsc_is_anon_clone(resource_t *rsc)
+pe_rsc_is_anon_clone(pe_resource_t *rsc)
 {
     return pe_rsc_is_clone(rsc) && is_not_set(rsc->flags, pe_rsc_unique);
 }
 
+/*!
+ * \brief Check whether a resource is part of a bundle
+ *
+ * \param[in] rsc  Resource to check
+ *
+ * \return TRUE if resource is part of a bundle, FALSE otherwise
+ */
 static inline bool
-pe_rsc_is_bundled(resource_t *rsc)
+pe_rsc_is_bundled(pe_resource_t *rsc)
 {
     return uber_parent(rsc)->parent != NULL;
 }
 
+// Deprecated type aliases
+typedef struct pe_action_s action_t;                 //!< \deprecated Use pe_action_t instead
+typedef struct pe_action_wrapper_s action_wrapper_t; //!< \deprecated Use pe_action_wrapper_t instead
+typedef struct pe_node_s node_t;                     //!< \deprecated Use pe_node_t instead
+typedef struct pe_resource_s resource_t;             //!< \deprecated Use pe_resource_t instead
+typedef struct pe_tag_s tag_t;                       //!< \deprecated Use pe_tag_t instead
+typedef struct pe_ticket_s ticket_t;                 //!< \deprecated Use pe_ticket_t instead
+typedef enum pe_quorum_policy no_quorum_policy_t;    //!< \deprecated Use enum pe_quorum_policy instead
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
