@@ -1,22 +1,8 @@
-/* crm_internal.h */
-
 /*
- * Copyright (C) 2006 - 2008
- *     Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2006-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #ifndef CRM_INTERNAL__H
@@ -39,7 +25,6 @@ void *find_library_function(void **handle, const char *lib, const char *fn, int
 
 /* For ACLs */
 char *uid2username(uid_t uid);
-void determine_request_user(const char *user, xmlNode * request, const char *field);
 const char *crm_acl_get_set_user(xmlNode * request, const char *field, const char *peer_user);
 
 #  if ENABLE_ACL
@@ -94,7 +79,7 @@ void crm_set_options(const char *short_options, const char *usage, struct crm_op
                      const char *app_desc);
 int crm_get_option(int argc, char **argv, int *index);
 int crm_get_option_long(int argc, char **argv, int *index, const char **longname);
-int crm_help(char cmd, int exit_code);
+crm_exit_t crm_help(char cmd, crm_exit_t exit_code);
 
 /* Cluster Option Processing */
 typedef struct pe_cluster_option_s {
@@ -130,50 +115,20 @@ gboolean check_quorum(const char *value);
 gboolean check_script(const char *value);
 gboolean check_utilization(const char *value);
 long crm_get_sbd_timeout(void);
+long crm_auto_watchdog_timeout(void);
 gboolean check_sbd_timeout(const char *value);
+void crm_args_fini(void);
 
 /* char2score */
 extern int node_score_red;
 extern int node_score_green;
 extern int node_score_yellow;
-extern int node_score_infinity;
 
 /* Assorted convenience functions */
-
-/*!
- * \internal
- * \brief Detect if process per PID and optionally exe path (component) exists
- *
- * \param[in] pid     PID of process assumed alive, disproving of which to try
- * \param[in] daemon  exe path (component) to possibly match with procfs entry
- *
- * \return -1 on invalid PID specification, -2 when the calling process has no
- *         (is refused an) ability to (dis)prove the predicate,
- *         0 if the negation of the predicate is confirmed (check-through-kill
- *         indicates so, or the subsequent check-through-procfs-match on
- *         \p daemon when provided and procfs available at the standard path),
- *         1 if it cannot be disproved (reliably [modulo race conditions]
- *         when \p daemon provided, procfs available at the standard path
- *         and the calling process has permissions to access the respective
- *         procfs location, less so otherwise, since mere check-through-kill
- *         is exercised without powers to exclude PID recycled in the interim).
- *
- * \note This function cannot be used to verify \e authenticity of the process.
- */
-int crm_pid_active(long pid, const char *daemon);
 void crm_make_daemon(const char *name, gboolean daemonize, const char *pidfile);
 
-/* from operations.c */
-char *generate_op_key(const char *rsc_id, const char *op_type, int interval);
-char *generate_notify_key(const char *rsc_id, const char *notify_type, const char *op_type);
-char *generate_transition_magic_v202(const char *transition_key, int op_status);
-char *generate_transition_magic(const char *transition_key, int op_status, int op_rc);
-char *generate_transition_key(int action, int transition_id, int target_rc, const char *node);
-void filter_action_parameters(xmlNode *param_set, const char *version);
-xmlNode *create_operation_update(xmlNode *parent, lrmd_event_data_t *event,
-                                 const char *caller_version, int target_rc,
-                                 const char *node, const char *origin,
-                                 int level);
+// printf-style format to create operation ID from resource, action, interval
+#define CRM_OP_FMT "%s_%s_%u"
 
 static inline long long
 crm_clear_bit(const char *function, int line, const char *target, long long word, long long bit)
@@ -217,15 +172,11 @@ void set_daemon_option(const char *option, const char *value);
 gboolean daemon_option_enabled(const char *daemon, const char *option);
 void strip_text_nodes(xmlNode * xml);
 void pcmk_panic(const char *origin);
-void sysrq_init(void);
 pid_t pcmk_locate_sbd(void);
-long crm_pidfile_inuse(const char *filename, long mypid, const char *daemon);
-long crm_read_pidfile(const char *filename);
 
 #  define crm_config_err(fmt...) { crm_config_error = TRUE; crm_err(fmt); }
 #  define crm_config_warn(fmt...) { crm_config_warning = TRUE; crm_warn(fmt); }
 
-#  define attrd_channel		T_ATTRD
 #  define F_ATTRD_KEY		"attr_key"
 #  define F_ATTRD_ATTRIBUTE	"attr_name"
 #  define F_ATTRD_REGEX 	"attr_regex"
@@ -236,7 +187,6 @@ long crm_read_pidfile(const char *filename);
 #  define F_ATTRD_IS_PRIVATE     "attr_is_private"
 #  define F_ATTRD_SECTION	"attr_section"
 #  define F_ATTRD_DAMPEN	"attr_dampening"
-#  define F_ATTRD_IGNORE_LOCALLY "attr_ignore_locally"
 #  define F_ATTRD_HOST		"attr_host"
 #  define F_ATTRD_HOST_ID	"attr_host_id"
 #  define F_ATTRD_USER		"attr_user"
@@ -263,18 +213,10 @@ long crm_read_pidfile(const char *filename);
 
 
 #  if SUPPORT_COROSYNC
-#    if CS_USES_LIBQB
-#      include <qb/qbipc_common.h>
-#      include <corosync/corotypes.h>
+#    include <qb/qbipc_common.h>
+#    include <corosync/corotypes.h>
 typedef struct qb_ipc_request_header cs_ipc_header_request_t;
 typedef struct qb_ipc_response_header cs_ipc_header_response_t;
-#    else
-#      include <corosync/corodefs.h>
-#      include <corosync/coroipcc.h>
-#      include <corosync/coroipc_types.h>
-typedef coroipc_request_header_t cs_ipc_header_request_t;
-typedef coroipc_response_header_t cs_ipc_header_response_t;
-#    endif
 #  else
 typedef struct {
     int size __attribute__ ((aligned(8)));
