@@ -1,21 +1,10 @@
 /*
- * Copyright (c) 2004 International Business Machines
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * Copyright 2004-2018 International Business Machines
  *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #include <crm_internal.h>
 #include <unistd.h>
 #include <limits.h>
@@ -35,8 +24,8 @@
 #include <crm/common/ipc.h>
 #include <crm/common/xml.h>
 
-#define cib_flag_dirty 0x00001
-#define cib_flag_live  0x00002
+#define CIB_FLAG_DIRTY 0x00001
+#define CIB_FLAG_LIVE  0x00002
 
 typedef struct cib_file_opaque_s {
     int flags;
@@ -186,36 +175,29 @@ cib_file_read_and_verify(const char *filename, const char *sigfile, xmlNode **ro
  *
  * \param[in] filename Name of file to check
  *
- * \return TRUE if file exists and has one of the possible live CIB filenames
+ * \return TRUE if file exists and its real path is same as live CIB's
  */
 static gboolean
 cib_file_is_live(const char *filename)
 {
+    gboolean same = FALSE;
+
     if (filename != NULL) {
-        /* Canonicalize all file names for true comparison */
+        // Canonicalize file names for true comparison
         char *real_filename = crm_compat_realpath(filename);
 
         if (real_filename != NULL) {
-            const char *livenames[] = {
-                CRM_CONFIG_DIR "/" CIB_LIVE_NAME,
-                CRM_LEGACY_CONFIG_DIR "/" CIB_LIVE_NAME
-            };
             char *real_livename;
-            int i;
 
-            /* Compare against each possible live CIB name */
-            for (i = 0; i < DIMOF(livenames); ++i) {
-                real_livename = crm_compat_realpath(livenames[i]);
-                if (real_livename && !strcmp(real_filename, real_livename)) {
-                    free(real_livename);
-                    return TRUE;
-                }
-                free(real_livename);
+            real_livename = crm_compat_realpath(CRM_CONFIG_DIR "/" CIB_LIVE_NAME);
+            if (real_livename && !strcmp(real_filename, real_livename)) {
+                same = TRUE;
             }
+            free(real_livename);
             free(real_filename);
         }
     }
-    return FALSE;
+    return same;
 }
 
 /* cib_file_backup() and cib_file_write_with_digest() need to chown the
@@ -490,7 +472,7 @@ cib_file_new(const char *cib_location)
     }
     private->flags = 0;
     if (cib_file_is_live(cib_location)) {
-        set_bit(private->flags, cib_flag_live);
+        set_bit(private->flags, CIB_FLAG_LIVE);
         crm_trace("File %s detected as live CIB", cib_location);
     }
     private->filename = strdup(cib_location);
@@ -512,7 +494,7 @@ static xmlNode *in_mem_cib = NULL;
 
 /*!
  * \internal
- * \brief Read CIB from disk and validate it against XML DTD
+ * \brief Read CIB from disk and validate it against XML schema
  *
  * \param[in] filename Name of file to read CIB from
  *
@@ -521,7 +503,7 @@ static xmlNode *in_mem_cib = NULL;
  *         -pcmk_err_schema_validation if XML doesn't parse or validate
  * \note If filename is the live CIB, this will *not* verify its digest,
  *       though that functionality would be trivial to add here.
- *       Also, this will *not* verify that the file is writeable,
+ *       Also, this will *not* verify that the file is writable,
  *       because some callers might not need to write.
  */
 static int
@@ -529,7 +511,6 @@ load_file_cib(const char *filename)
 {
     struct stat buf;
     xmlNode *root = NULL;
-    const char *ignore_dtd = NULL;
 
     /* Ensure file is readable */
     if (stat(filename, &buf) < 0) {
@@ -547,10 +528,11 @@ load_file_cib(const char *filename)
         create_xml_node(root, XML_CIB_TAG_STATUS);
     }
 
-    /* Validate XML against its specified DTD */
-    ignore_dtd = crm_element_value(root, XML_ATTR_VALIDATION);
+    /* Validate XML against its specified schema */
     if (validate_xml(root, NULL, TRUE) == FALSE) {
-        crm_err("CIB does not validate against %s", ignore_dtd);
+        const char *schema = crm_element_value(root, XML_ATTR_VALIDATION);
+
+        crm_err("CIB does not validate against %s", schema);
         free_xml(root);
         return -pcmk_err_schema_validation;
     }
@@ -672,7 +654,7 @@ cib_file_write_live(char *path)
  * \param[in] cib CIB object to sign off
  *
  * \return pcmk_ok on success, pcmk_err_generic on failure
- * \todo This method should refuse to write the live CIB if the CIB daemon is
+ * \todo This method should refuse to write the live CIB if the CIB manager is
  *       running.
  */
 int
@@ -681,15 +663,15 @@ cib_file_signoff(cib_t * cib)
     int rc = pcmk_ok;
     cib_file_opaque_t *private = cib->variant_opaque;
 
-    crm_debug("Signing out of the CIB Service");
+    crm_debug("Disconnecting from the CIB manager");
     cib->state = cib_disconnected;
     cib->type = cib_no_connection;
 
     /* If the in-memory CIB has been changed, write it to disk */
-    if (is_set(private->flags, cib_flag_dirty)) {
+    if (is_set(private->flags, CIB_FLAG_DIRTY)) {
 
         /* If this is the live CIB, write it out with a digest */
-        if (is_set(private->flags, cib_flag_live)) {
+        if (is_set(private->flags, CIB_FLAG_LIVE)) {
             if (cib_file_write_live(private->filename) < 0) {
                 rc = pcmk_err_generic;
             }
@@ -705,7 +687,7 @@ cib_file_signoff(cib_t * cib)
 
         if (rc == pcmk_ok) {
             crm_info("Wrote CIB to %s", private->filename);
-            clear_bit(private->flags, cib_flag_dirty);
+            clear_bit(private->flags, CIB_FLAG_DIRTY);
         } else {
             crm_err("Could not write CIB to %s", private->filename);
         }
@@ -844,7 +826,7 @@ cib_file_perform_op_delegate(cib_t * cib, const char *op, const char *host, cons
         xml_log_patchset(LOG_DEBUG, "cib:diff", cib_diff);
         free_xml(in_mem_cib);
         in_mem_cib = result_cib;
-        set_bit(private->flags, cib_flag_dirty);
+        set_bit(private->flags, CIB_FLAG_DIRTY);
     }
 
     free_xml(cib_diff);
