@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -119,7 +108,8 @@ cib_process_upgrade(const char *op, int options, const char *section, xmlNode *
         max_version = get_schema_version(max);
     }
 
-    rc = update_validation(result_cib, &new_version, max_version, TRUE, TRUE);
+    rc = update_validation(result_cib, &new_version, max_version, TRUE,
+                           !(options & cib_verbose));
     if (new_version > current_version) {
         cib_update_counter(*result_cib, XML_ATTR_GENERATION_ADMIN, FALSE);
         cib_update_counter(*result_cib, XML_ATTR_GENERATION, TRUE);
@@ -156,9 +146,8 @@ cib_update_counter(xmlNode * xml_obj, const char *field, gboolean reset)
         old_value = crm_element_value_copy(xml_obj, field);
     }
     if (old_value != NULL) {
-        new_value = calloc(1, 128);
         int_value = atoi(old_value);
-        sprintf(new_value, "%d", ++int_value);
+        new_value = crm_itoa(++int_value);
     } else {
         new_value = strdup("1");
     }
@@ -434,9 +423,7 @@ update_cib_object(xmlNode * parent, xmlNode * update)
                     goto incr;
                 }
 
-                replace_item = calloc(1, lpc - last + 1);
-                memcpy(replace_item, replace + last, lpc - last);
-
+                replace_item = strndup(replace + last, lpc - last);
                 remove = find_xml_node(target, replace_item, FALSE);
                 if (remove != NULL) {
                     crm_trace("Replacing node <%s> in <%s>",
@@ -513,7 +500,7 @@ add_cib_object(xmlNode * parent, xmlNode * new_obj)
         ;                       /* do nothing */
 
     } else if (equiv_node != NULL) {
-        result = -ENOTUNIQ;
+        result = -EEXIST;
 
     } else {
         result = update_cib_object(parent, new_obj);
@@ -571,8 +558,8 @@ cib_process_create(const char *op, int options, const char *section, xmlNode * r
         update_results(failed, input, op, result);
     }
 
-    if (xml_has_children(failed)) {
-        CRM_CHECK(result != pcmk_ok, result = -EINVAL);
+    if ((result == pcmk_ok) && xml_has_children(failed)) {
+        result = -EINVAL;
     }
 
     if (result != pcmk_ok) {
@@ -723,7 +710,7 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
         }
 
         path = xmlGetNodePath(match);
-        crm_debug("Processing %s op for %s (%s)", op, section, path);
+        crm_debug("Processing %s op for %s with %s", op, section, path);
         free(path);
 
         if (safe_str_eq(op, CIB_OP_DELETE)) {
@@ -763,38 +750,26 @@ cib_process_xpath(const char *op, int options, const char *section, xmlNode * re
                 }
 
             } else if (options & cib_xpath_address) {
-
-                int path_len = 0;
                 char *path = NULL;
                 xmlNode *parent = match;
 
                 while (parent && parent->type == XML_ELEMENT_NODE) {
-                    int extra = 1;
-                    char *new_path = NULL;
                     const char *id = crm_element_value(parent, XML_ATTR_ID);
+                    char *new_path = NULL;
 
-                    extra += strlen((const char *)parent->name);
                     if (id) {
-                        extra += 8;     /* [@id=""] */
-                        extra += strlen(id);
-                    }
-
-                    path_len += extra;
-                    new_path = malloc(path_len + 1);
-                    if(new_path == NULL) {
-                        break;
-
-                    } else if (id) {
-                        snprintf(new_path, path_len + 1, "/%s[@id='%s']%s", parent->name, id,
-                                 path ? path : "");
+                        new_path = crm_strdup_printf("/%s[@id='%s']%s",
+                                                     parent->name, id,
+                                                     (path? path : ""));
                     } else {
-                        snprintf(new_path, path_len + 1, "/%s%s", parent->name, path ? path : "");
+                        new_path = crm_strdup_printf("/%s%s", parent->name,
+                                                     (path? path : ""));
                     }
                     free(path);
                     path = new_path;
                     parent = parent->parent;
                 }
-                crm_trace("Got: %s\n", path);
+                crm_trace("Got: %s", path);
 
                 if (*answer == NULL) {
                     *answer = create_xml_node(NULL, "xpath-query");
