@@ -62,84 +62,6 @@ struct config_root_s known_paths[] = {
 };
 /* *INDENT-ON* */
 
-int
-cib_compare_generation(xmlNode * left, xmlNode * right)
-{
-    int lpc = 0;
-
-    const char *attributes[] = {
-        XML_ATTR_GENERATION_ADMIN,
-        XML_ATTR_GENERATION,
-        XML_ATTR_NUMUPDATES,
-    };
-
-    crm_log_xml_trace(left, "left");
-    crm_log_xml_trace(right, "right");
-
-    for (lpc = 0; lpc < DIMOF(attributes); lpc++) {
-        int int_elem_l = -1;
-        int int_elem_r = -1;
-        const char *elem_r = NULL;
-        const char *elem_l = crm_element_value(left, attributes[lpc]);
-
-        if (right != NULL) {
-            elem_r = crm_element_value(right, attributes[lpc]);
-        }
-
-        if (elem_l != NULL) {
-            int_elem_l = crm_parse_int(elem_l, NULL);
-        }
-        if (elem_r != NULL) {
-            int_elem_r = crm_parse_int(elem_r, NULL);
-        }
-
-        if (int_elem_l < int_elem_r) {
-            crm_trace("%s (%s < %s)", attributes[lpc], crm_str(elem_l), crm_str(elem_r));
-            return -1;
-
-        } else if (int_elem_l > int_elem_r) {
-            crm_trace("%s (%s > %s)", attributes[lpc], crm_str(elem_l), crm_str(elem_r));
-            return 1;
-        }
-    }
-
-    return 0;
-}
-
-/* Deprecated - doesn't expose -EACCES */
-xmlNode *
-get_cib_copy(cib_t * cib)
-{
-    xmlNode *xml_cib;
-    int options = cib_scope_local | cib_sync_call;
-    int rc = pcmk_ok;
-
-    if (cib->state == cib_disconnected) {
-        return NULL;
-    }
-
-    rc = cib->cmds->query(cib, NULL, &xml_cib, options);
-    if (rc == -EACCES) {
-        return NULL;
-
-    } else if (rc != pcmk_ok) {
-        crm_err("Couldn't retrieve the CIB");
-        free_xml(xml_cib);
-        return NULL;
-
-    } else if (xml_cib == NULL) {
-        crm_err("The CIB result was empty");
-        free_xml(xml_cib);
-        return NULL;
-    }
-
-    if (safe_str_eq(crm_element_name(xml_cib), XML_TAG_CIB)) {
-        return xml_cib;
-    }
-    free_xml(xml_cib);
-    return NULL;
-}
-
 xmlNode *
 cib_get_generation(cib_t * cib)
 {
@@ -235,7 +157,7 @@ get_object_root(const char *object_type, xmlNode * the_root)
         return the_root;        /* or return NULL? */
     }
 
-    return get_xpath_object(xpath, the_root, LOG_DEBUG_4);
+    return get_xpath_object(xpath, the_root, LOG_TRACE);
 }
 
 /*
@@ -274,7 +196,7 @@ cib_acl_enabled(xmlNode *xml, const char *user)
 #if ENABLE_ACL
     if(pcmk_acl_required(user)) {
         const char *value = NULL;
-        GHashTable *options = g_hash_table_new_full(crm_str_hash, g_str_equal, g_hash_destroy_str, g_hash_destroy_str);
+        GHashTable *options = crm_str_table_new();
 
         cib_read_config(options, xml);
         value = cib_pref(options, "enable-acl");
@@ -282,7 +204,7 @@ cib_acl_enabled(xmlNode *xml, const char *user)
         g_hash_table_destroy(options);
     }
 
-    crm_debug("CIB ACL is %s", rc ? "enabled" : "disabled");
+    crm_trace("CIB ACL is %s", rc ? "enabled" : "disabled");
 #endif
     return rc;
 }
@@ -294,7 +216,7 @@ cib_perform_op(const char *op, int call_options, cib_op_t * fn, gboolean is_quer
                xmlNode * current_cib, xmlNode ** result_cib, xmlNode ** diff, xmlNode ** output)
 {
     int rc = pcmk_ok;
-    gboolean check_dtd = TRUE;
+    gboolean check_schema = TRUE;
     xmlNode *top = NULL;
     xmlNode *scratch = NULL;
     xmlNode *local_diff = NULL;
@@ -502,9 +424,9 @@ cib_perform_op(const char *op, int call_options, cib_op_t * fn, gboolean is_quer
     if (safe_str_eq(section, XML_CIB_TAG_STATUS)) {
         /* Throttle the amount of costly validation we perform due to status updates
          * a) we don't really care whats in the status section
-         * b) we don't validate any of it's contents at the moment anyway
+         * b) we don't validate any of its contents at the moment anyway
          */
-        check_dtd = FALSE;
+        check_schema = FALSE;
     }
 
     /* === scratch must not be modified after this point ===
@@ -551,11 +473,13 @@ cib_perform_op(const char *op, int call_options, cib_op_t * fn, gboolean is_quer
         }
     }
 
-    crm_trace("Perform validation: %s", check_dtd ? "true" : "false");
-    if (rc == pcmk_ok && check_dtd && validate_xml(scratch, NULL, TRUE) == FALSE) {
-        const char *current_dtd = crm_element_value(scratch, XML_ATTR_VALIDATION);
+    crm_trace("Perform validation: %s", (check_schema? "true" : "false"));
+    if ((rc == pcmk_ok) && check_schema && !validate_xml(scratch, NULL, TRUE)) {
+        const char *current_schema = crm_element_value(scratch,
+                                                       XML_ATTR_VALIDATION);
 
-        crm_warn("Updated CIB does not validate against %s schema/dtd", crm_str(current_dtd));
+        crm_warn("Updated CIB does not validate against %s schema",
+                 crm_str(current_schema));
         rc = -pcmk_err_schema_validation;
     }
 
@@ -698,11 +622,27 @@ cib_native_notify(gpointer data, gpointer user_data)
     crm_trace("Callback invoked...");
 }
 
-pe_cluster_option cib_opts[] = {
-    /* name, old-name, validate, default, description */
-    {"enable-acl", NULL, "boolean", NULL, "false", &check_boolean,
-     "Enable CIB ACL", NULL}
-    ,
+static pe_cluster_option cib_opts[] = {
+    /*
+     * name, legacy name,
+     * type, allowed values, default, validator,
+     * short description,
+     * long description
+     */
+    {
+        "enable-acl", NULL,
+        "boolean", NULL, "false", &check_boolean,
+        "Enable CIB ACL",
+        NULL
+    },
+    {
+        "cluster-ipc-limit", NULL,
+        "integer", NULL, "500", &check_positive_number,
+        "Maximum IPC message backlog before disconnecting a cluster daemon",
+        "Raise this if log has \"Evicting client\" messages for cluster daemon"
+            " PIDs (a good value is the number of resources in the cluster"
+            " multiplied by the number of nodes)"
+    },
 };
 
 void
@@ -743,7 +683,7 @@ cib_read_config(GHashTable * options, xmlNode * current_cib)
     config = get_object_root(XML_CIB_TAG_CRMCONFIG, current_cib);
     if (config) {
         unpack_instance_attributes(current_cib, config, XML_CIB_TAG_PROPSET, NULL, options,
-                                   CIB_OPTIONS_FIRST, FALSE, now);
+                                   CIB_OPTIONS_FIRST, TRUE, now);
     }
 
     verify_cib_options(options);
@@ -794,23 +734,24 @@ cib_apply_patch_event(xmlNode * event, xmlNode * input, xmlNode ** output, int l
     return rc;
 }
 
+/* v2 and v2 patch formats */
+#define XPATH_CONFIG_CHANGE \
+    "//" XML_CIB_TAG_CRMCONFIG " | " \
+    "//" XML_DIFF_CHANGE "[contains(@" XML_DIFF_PATH ",'/" XML_CIB_TAG_CRMCONFIG "/')]"
+
 gboolean
-cib_internal_config_changed(xmlNode * diff)
+cib_internal_config_changed(xmlNode *diff)
 {
     gboolean changed = FALSE;
-    xmlXPathObject *xpathObj = NULL;
 
-    if (diff == NULL) {
-        return FALSE;
+    if (diff) {
+        xmlXPathObject *xpathObj = xpath_search(diff, XPATH_CONFIG_CHANGE);
+
+        if (numXpathResults(xpathObj) > 0) {
+            changed = TRUE;
+        }
+        freeXpathObject(xpathObj);
     }
-
-    xpathObj = xpath_search(diff, "//" XML_CIB_TAG_CRMCONFIG);
-    if (numXpathResults(xpathObj) > 0) {
-        changed = TRUE;
-    }
-
-    freeXpathObject(xpathObj);
-
     return changed;
 }
 
