@@ -17,11 +17,14 @@
  */
 
 #include <crm_internal.h>
-#include <bzlib.h>
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <inttypes.h>  /* U64T ~ PRIu64 */
+
+#include <bzlib.h>
 
 #include <crm/common/ipc.h>
 #include <crm/cluster/internal.h>
@@ -44,6 +47,19 @@ quorum_handle_t pcmk_quorum_handle = 0;
 
 gboolean(*quorum_app_callback) (unsigned long long seq, gboolean quorate) = NULL;
 
+char *
+get_corosync_uuid(crm_node_t *node)
+{
+    if (node && is_corosync_cluster()) {
+        if (node->id > 0) {
+            return crm_strdup_printf("%u", node->id);
+        } else {
+            crm_info("Node %s is not yet known by corosync", node->uname);
+        }
+    }
+    return NULL;
+}
+
 /*
  * CFG functionality stolen from node_name() in corosync-quorumtool.c
  * This resolves the first address assigned to a node and returns the name or IP address.
@@ -57,7 +73,6 @@ corosync_node_name(uint64_t /*cmap_handle_t */ cmap_handle, uint32_t nodeid)
     char *name = NULL;
     cmap_handle_t local_handle = 0;
 
-    /* nodeid == 0 == CMAN_NODEID_US */
     if (nodeid == 0) {
         nodeid = get_local_nodeid(0);
     }
@@ -102,6 +117,12 @@ corosync_node_name(uint64_t /*cmap_handle_t */ cmap_handle, uint32_t nodeid)
 
         if (nodeid == id) {
             crm_trace("Searching for node name for %u in nodelist.node.%d %s", nodeid, lpc, name);
+            if (name == NULL) {
+                key = crm_strdup_printf("nodelist.node.%d.name", lpc);
+                cmap_get_string(cmap_handle, key, &name);
+                crm_trace("%s = %s", key, name);
+                free(key);
+            }
             if (name == NULL) {
                 key = crm_strdup_printf("nodelist.node.%d.ring0_addr", lpc);
                 cmap_get_string(cmap_handle, key, &name);
@@ -113,13 +134,6 @@ corosync_node_name(uint64_t /*cmap_handle_t */ cmap_handle, uint32_t nodeid)
                 }
                 free(key);
             }
-
-            if (name == NULL) {
-                key = crm_strdup_printf("nodelist.node.%d.name", lpc);
-                cmap_get_string(cmap_handle, key, &name);
-                crm_trace("%s = %s %d", key, name, rc);
-                free(key);
-            }
             break;
         }
 
@@ -139,26 +153,14 @@ corosync_node_name(uint64_t /*cmap_handle_t */ cmap_handle, uint32_t nodeid)
 void
 terminate_cs_connection(crm_cluster_t *cluster)
 {
-    crm_info("Disconnecting from Corosync");
-
     cluster_disconnect_cpg(cluster);
-
     if (pcmk_quorum_handle) {
-        crm_trace("Disconnecting quorum");
         quorum_finalize(pcmk_quorum_handle);
         pcmk_quorum_handle = 0;
-
-    } else {
-        crm_info("No Quorum connection");
     }
-
     crm_notice("Disconnected from Corosync");
 }
 
-int ais_membership_timer = 0;
-gboolean ais_membership_force = FALSE;
-
-
 static int
 pcmk_quorum_dispatch(gpointer user_data)
 {
@@ -185,16 +187,16 @@ pcmk_quorum_notification(quorum_handle_t handle,
 
     if (quorate != crm_have_quorum) {
         if (quorate) {
-            crm_notice("Quorum acquired " CRM_XS " membership=" U64T " members=%lu",
+            crm_notice("Quorum acquired " CRM_XS " membership=%" U64T " members=%lu",
                        ring_id, (long unsigned int)view_list_entries);
         } else {
-            crm_warn("Quorum lost " CRM_XS " membership=" U64T " members=%lu",
+            crm_warn("Quorum lost " CRM_XS " membership=%" U64T " members=%lu",
                      ring_id, (long unsigned int)view_list_entries);
         }
         crm_have_quorum = quorate;
 
     } else {
-        crm_info("Quorum %s " CRM_XS " membership=" U64T " members=%lu",
+        crm_info("Quorum %s " CRM_XS " membership=%" U64T " members=%lu",
                  (quorate? "retained" : "still lost"), ring_id,
                  (long unsigned int)view_list_entries);
     }
@@ -533,7 +535,7 @@ corosync_initialize_nodelist(void *cluster, gboolean force_member, xmlNode * xml
                     if (node->id && node->id != nodeid) {
                         crm_crit("Nodes %u and %u share the same name '%s': shutting down", node->id,
                                  nodeid, name);
-                        crm_exit(DAEMON_RESPAWN_STOP);
+                        crm_exit(CRM_EX_FATAL);
                     }
                 }
             }
