@@ -1,7 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
- *
- * The version control history for this file may have further details.
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -29,8 +27,6 @@
 
 #include <crm/msg_xml.h>
 
-#include <crm/common/ipc_internal.h>  /* PCMK__SPECIAL_PID* */
-
 cpg_handle_t pcmk_cpg_handle = 0; /* TODO: Remove, use cluster.cpg_handle */
 
 static bool cpg_evicted = FALSE;
@@ -64,56 +60,19 @@ cluster_disconnect_cpg(crm_cluster_t *cluster)
 
 uint32_t get_local_nodeid(cpg_handle_t handle)
 {
-    cs_error_t rc = CS_OK;
+    int rc = CS_OK;
     int retries = 0;
     static uint32_t local_nodeid = 0;
     cpg_handle_t local_handle = handle;
     cpg_callbacks_t cb = { };
-    int fd = -1;
-    uid_t found_uid = 0;
-    gid_t found_gid = 0;
-    pid_t found_pid = 0;
-    int rv;
 
     if(local_nodeid != 0) {
         return local_nodeid;
     }
 
-#if 0
-    /* Should not be necessary */
-    if(get_cluster_type() == pcmk_cluster_classic_ais) {
-        get_ais_details(&local_nodeid, NULL);
-        goto done;
-    }
-#endif
-
     if(handle == 0) {
         crm_trace("Creating connection");
         cs_repeat(retries, 5, rc = cpg_initialize(&local_handle, &cb));
-        if (rc != CS_OK) {
-            crm_err("Could not connect to the CPG API (rc=%d)", rc);
-            return 0;
-        }
-
-        rc = cpg_fd_get(local_handle, &fd);
-        if (rc != CS_OK) {
-            crm_err("Could not obtain the CPG API connection (rc=%d)", rc);
-            goto bail;
-        }
-
-        /* CPG provider run as root (in given user namespace, anyway)? */
-        if (!(rv = crm_ipc_is_authentic_process(fd, (uid_t) 0,(gid_t) 0, &found_pid,
-                                                &found_uid, &found_gid))) {
-            crm_err("CPG provider is not authentic:"
-                    " process %lld (uid: %lld, gid: %lld)",
-                    (long long) PCMK__SPECIAL_PID_AS_0(found_pid),
-                    (long long) found_uid, (long long) found_gid);
-            goto bail;
-        } else if (rv < 0) {
-            crm_err("Could not verify authenticity of CPG provider: %s (%d)",
-                    strerror(-rv), -rv);
-            goto bail;
-        }
     }
 
     if (rc == CS_OK) {
@@ -125,8 +84,6 @@ uint32_t get_local_nodeid(cpg_handle_t handle)
     if (rc != CS_OK) {
         crm_err("Could not get local node id from the CPG API: %s (%d)", ais_error2text(rc), rc);
     }
-
-bail:
     if(handle == 0) {
         crm_trace("Closing connection");
         cpg_finalize(local_handle);
@@ -261,9 +218,7 @@ pcmk_message_common_cs(cpg_handle_t handle, uint32_t nodeid, uint32_t pid, void
     AIS_Message *msg = (AIS_Message *) content;
 
     if(handle) {
-        /* 'msg' came from CPG not the plugin
-         * Do filtering and field massaging
-         */
+        // Do filtering and field massaging
         uint32_t local_nodeid = get_local_nodeid(handle);
         const char *local_name = get_local_node_name();
 
@@ -325,7 +280,8 @@ pcmk_message_common_cs(cpg_handle_t handle, uint32_t nodeid, uint32_t pid, void
         rc = BZ2_bzBuffToBuffDecompress(uncompressed, &new_size, msg->data, msg->compressed_size, 1, 0);
 
         if (rc != BZ_OK) {
-            crm_err("Decompression failed: %d", rc);
+            crm_err("Decompression failed: %s " CRM_XS " bzerror=%d",
+                    bz2_strerror(rc), rc);
             free(uncompressed);
             goto badmsg;
         }
@@ -349,24 +305,8 @@ pcmk_message_common_cs(cpg_handle_t handle, uint32_t nodeid, uint32_t pid, void
         data = strdup(msg->data);
     }
 
-    if (msg->header.id != crm_class_members) {
-        /* Is this even needed anymore? */
-        crm_get_peer(msg->sender.id, msg->sender.uname);
-    }
-
-    if (msg->header.id == crm_class_rmpeer) {
-        uint32_t id = crm_int_helper(data, NULL);
-
-        crm_info("Removing peer %s/%u", data, id);
-        reap_crm_member(id, NULL);
-        free(data);
-        return NULL;
-
-#if SUPPORT_PLUGIN
-    } else if (is_classic_ais_cluster()) {
-        plugin_handle_membership(msg);
-#endif
-    }
+    // Is this necessary?
+    crm_get_peer(msg->sender.id, msg->sender.uname);
 
     crm_trace("Payload: %.200s", data);
     return data;
@@ -383,45 +323,7 @@ pcmk_message_common_cs(cpg_handle_t handle, uint32_t nodeid, uint32_t pid, void
     return NULL;
 }
 
-static int cmp_member_list_nodeid(const void *first,
-                                  const void *second)
-{
-    const struct cpg_address *const a = *((const struct cpg_address **) first),
-                             *const b = *((const struct cpg_address **) second);
-    if (a->nodeid < b->nodeid) {
-        return -1;
-    } else if (a->nodeid > b->nodeid) {
-        return 1;
-    }
-    /* don't bother with "reason" nor "pid" */
-    return 0;
-}
-
-static const char *
-cpgreason2str(cpg_reason_t reason)
-{
-    switch (reason) {
-        case CPG_REASON_JOIN:       return " via cpg_join";
-        case CPG_REASON_LEAVE:      return " via cpg_leave";
-        case CPG_REASON_NODEDOWN:   return " via cluster exit";
-        case CPG_REASON_NODEUP:     return " via cluster join";
-        case CPG_REASON_PROCDOWN:   return " for unknown reason";
-        default:                    break;
-    }
-    return "";
-}
-
-static inline const char *
-peer_name(crm_node_t *peer)
-{
-    if (peer == NULL) {
-        return "unknown node";
-    } else if (peer->uname == NULL) {
-        return "peer node";
-    } else {
-        return peer->uname;
-    }
-}
+#define PEER_NAME(peer) ((peer)? ((peer)->uname? (peer)->uname : "<unknown>") : "<none>")
 
 void
 pcmk_cpg_membership(cpg_handle_t handle,
@@ -434,111 +336,54 @@ pcmk_cpg_membership(cpg_handle_t handle,
     gboolean found = FALSE;
     static int counter = 0;
     uint32_t local_nodeid = get_local_nodeid(handle);
-    const struct cpg_address *key, **sorted;
-
-    sorted = malloc(member_list_entries * sizeof(const struct cpg_address *));
-    CRM_ASSERT(sorted != NULL);
-
-    for (size_t iter = 0; iter < member_list_entries; iter++) {
-        sorted[iter] = member_list + iter;
-    }
-    /* so that the cross-matching multiply-subscribed nodes is then cheap */
-    qsort(sorted, member_list_entries, sizeof(const struct cpg_address *),
-          cmp_member_list_nodeid);
 
     for (i = 0; i < left_list_entries; i++) {
         crm_node_t *peer = crm_find_peer(left_list[i].nodeid, NULL);
-        const struct cpg_address **rival = NULL;
 
-        /* in CPG world, NODE:PROCESS-IN-MEMBERSHIP-OF-G is an 1:N relation
-           and not playing by this rule may go wild in case of multiple
-           residual instances of the same pacemaker daemon at the same node
-           -- we must ensure that the possible local rival(s) won't make us
-           cry out and bail (e.g. when they quit themselves), since all the
-           surrounding logic denies this simple fact that the full membership
-           is discriminated also per the PID of the process beside mere node
-           ID (and implicitly, group ID); practically, this will be sound in
-           terms of not preventing progress, since all the CPG joiners are
-           also API end-point carriers, and that's what matters locally
-           (who's the winner);
-           remotely, we will just compare leave_list and member_list and if
-           the left process has its node retained in member_list (under some
-           other PID, anyway) we will just ignore it as well
-           XXX: long-term fix is to establish in-out PID-aware tracking? */
+        crm_info("Group event %s.%d: node %u (%s) left",
+                 groupName->value, counter, left_list[i].nodeid,
+                 PEER_NAME(peer));
         if (peer) {
-            key = &left_list[i];
-            rival = bsearch(&key, sorted, member_list_entries,
-                            sizeof(const struct cpg_address *),
-                            cmp_member_list_nodeid);
-        }
-
-        if (rival == NULL) {
-            crm_info("Group %s event %d: %s (node %u pid %u) left%s",
-                     groupName->value, counter, peer_name(peer),
-                     left_list[i].nodeid, left_list[i].pid,
-                     cpgreason2str(left_list[i].reason));
-            if (peer) {
-                crm_update_peer_proc(__FUNCTION__, peer, crm_proc_cpg,
-                                     OFFLINESTATUS);
-            }
-        } else if (left_list[i].nodeid == local_nodeid) {
-            crm_warn("Group %s event %d: duplicate local pid %u left%s",
-                     groupName->value, counter,
-                     left_list[i].pid, cpgreason2str(left_list[i].reason));
-        } else {
-            crm_warn("Group %s event %d: "
-                     "%s (node %u) duplicate pid %u left%s (%u remains)",
-                     groupName->value, counter, peer_name(peer),
-                     left_list[i].nodeid, left_list[i].pid,
-                     cpgreason2str(left_list[i].reason), (*rival)->pid);
+            crm_update_peer_proc(__FUNCTION__, peer, crm_proc_cpg, OFFLINESTATUS);
         }
     }
-    free(sorted);
-    sorted = NULL;
 
     for (i = 0; i < joined_list_entries; i++) {
-        crm_info("Group %s event %d: node %u pid %u joined%s",
-                 groupName->value, counter, joined_list[i].nodeid,
-                 joined_list[i].pid, cpgreason2str(joined_list[i].reason));
+        crm_info("Group event %s.%d: node %u joined",
+                 groupName->value, counter, joined_list[i].nodeid);
     }
 
     for (i = 0; i < member_list_entries; i++) {
         crm_node_t *peer = crm_get_peer(member_list[i].nodeid, NULL);
 
-        if (member_list[i].nodeid == local_nodeid
-                && member_list[i].pid != getpid()) {
-            /* see the note above */
-            crm_warn("Group %s event %d: detected duplicate local pid %u",
-                     groupName->value, counter, member_list[i].pid);
-            continue;
-        }
-        crm_info("Group %s event %d: %s (node %u pid %u) is member",
-                 groupName->value, counter, peer_name(peer),
-                 member_list[i].nodeid, member_list[i].pid);
+        crm_info("Group event %s.%d: node %u (%s) is member",
+                 groupName->value, counter, member_list[i].nodeid,
+                 PEER_NAME(peer));
 
-        /* Anyone that is sending us CPG messages must also be a _CPG_ member.
-         * But it's _not_ safe to assume it's in the quorum membership.
-         * We may have just found out it's dead and are processing the last couple of messages it sent
+        /* If the caller left auto-reaping enabled, this will also update the
+         * state to member.
          */
         peer = crm_update_peer_proc(__FUNCTION__, peer, crm_proc_cpg, ONLINESTATUS);
-        if(peer && peer->state && crm_is_peer_active(peer) == FALSE) {
+
+        if (peer && peer->state && strcmp(peer->state, CRM_NODE_MEMBER)) {
+            /* The node is a CPG member, but we currently think it's not a
+             * cluster member. This is possible only if auto-reaping was
+             * disabled. The node may be joining, and we happened to get the CPG
+             * notification before the quorum notification; or the node may have
+             * just died, and we are processing its final messages; or a bug
+             * has affected the peer cache.
+             */
             time_t now = time(NULL);
 
-            /* Co-opt the otherwise unused votes field */
-            if(peer->votes == 0) {
-                peer->votes = now;
+            if (peer->when_lost == 0) {
+                // Track when we first got into this contradictory state
+                peer->when_lost = now;
 
-            } else if(now > (60 + peer->votes)) {
-                /* On the otherhand, if we're still getting messages, at a certain point
-                 * we need to acknowledge our internal cache is probably wrong
-                 *
-                 * Set the threshold to 1 minute
-                 */
-                crm_warn("Node %u is member of group %s but was believed offline",
+            } else if (now > (peer->when_lost + 60)) {
+                // If it persists for more than a minute, update the state
+                crm_warn("Node %u member of group %s but believed offline",
                          member_list[i].nodeid, groupName->value);
-                if (crm_update_peer_state(__FUNCTION__, peer, CRM_NODE_MEMBER, 0)) {
-                    peer->votes = 0;
-                }
+                crm_update_peer_state(__FUNCTION__, peer, CRM_NODE_MEMBER, 0);
             }
         }
 
@@ -548,7 +393,7 @@ pcmk_cpg_membership(cpg_handle_t handle,
     }
 
     if (!found) {
-        crm_err("Local node was evicted from group %s", groupName->value);
+        crm_err("We're not part of CPG group '%s' anymore!", groupName->value);
         cpg_evicted = TRUE;
     }
 
@@ -558,16 +403,13 @@ pcmk_cpg_membership(cpg_handle_t handle,
 gboolean
 cluster_connect_cpg(crm_cluster_t *cluster)
 {
-    cs_error_t rc;
-    int fd = -1;
+    int rc = -1;
+    int fd = 0;
     int retries = 0;
     uint32_t id = 0;
     crm_node_t *peer = NULL;
     cpg_handle_t handle = 0;
-    uid_t found_uid = 0;
-    gid_t found_gid = 0;
-    pid_t found_pid = 0;
-    int rv;
+    const char *message_name = pcmk_message_name(crm_system_name);
 
     struct mainloop_fd_callbacks cpg_fd_callbacks = {
         .dispatch = pcmk_cpg_dispatch,
@@ -586,35 +428,13 @@ cluster_connect_cpg(crm_cluster_t *cluster)
     cluster->group.value[0] = 0;
 
     /* group.value is char[128] */
-    strncpy(cluster->group.value, crm_system_name?crm_system_name:"unknown", 127);
+    strncpy(cluster->group.value, message_name, 127);
     cluster->group.value[127] = 0;
     cluster->group.length = 1 + QB_MIN(127, strlen(cluster->group.value));
 
     cs_repeat(retries, 30, rc = cpg_initialize(&handle, &cpg_callbacks));
     if (rc != CS_OK) {
-        crm_err("Could not connect to the CPG API (rc=%d)", rc);
-        goto bail;
-    }
-
-    rc = cpg_fd_get(handle, &fd);
-    if (rc != CS_OK) {
-        crm_err("Could not obtain the CPG API connection (rc=%d)", rc);
-        goto bail;
-    }
-
-    /* CPG provider run as root (in given user namespace, anyway)? */
-    if (!(rv = crm_ipc_is_authentic_process(fd, (uid_t) 0,(gid_t) 0, &found_pid,
-                                            &found_uid, &found_gid))) {
-        crm_err("CPG provider is not authentic:"
-                " process %lld (uid: %lld, gid: %lld)",
-                (long long) PCMK__SPECIAL_PID_AS_0(found_pid),
-                (long long) found_uid, (long long) found_gid);
-        rc = CS_ERR_ACCESS;
-        goto bail;
-    } else if (rv < 0) {
-        crm_err("Could not verify authenticity of CPG provider: %s (%d)",
-                strerror(-rv), -rv);
-        rc = CS_ERR_ACCESS;
+        crm_err("Could not connect to the Cluster Process Group API: %d", rc);
         goto bail;
     }
 
@@ -629,7 +449,13 @@ cluster_connect_cpg(crm_cluster_t *cluster)
     retries = 0;
     cs_repeat(retries, 30, rc = cpg_join(handle, &cluster->group));
     if (rc != CS_OK) {
-        crm_err("Could not join the CPG group '%s': %d", crm_system_name, rc);
+        crm_err("Could not join the CPG group '%s': %d", message_name, rc);
+        goto bail;
+    }
+
+    rc = cpg_fd_get(handle, &fd);
+    if (rc != CS_OK) {
+        crm_err("Could not obtain the CPG API connection: %d", rc);
         goto bail;
     }
 
@@ -661,8 +487,8 @@ send_cluster_message_cs(xmlNode * msg, gboolean local, crm_node_t * node, enum c
 }
 
 gboolean
-send_cluster_text(int class, const char *data,
-              gboolean local, crm_node_t * node, enum crm_ais_msg_types dest)
+send_cluster_text(enum crm_ais_msg_class msg_class, const char *data,
+                  gboolean local, crm_node_t *node, enum crm_ais_msg_types dest)
 {
     static int msg_id = 0;
     static int local_pid = 0;
@@ -674,13 +500,15 @@ send_cluster_text(int class, const char *data,
     AIS_Message *msg = NULL;
     enum crm_ais_msg_types sender = text2msg_type(crm_system_name);
 
-    /* There are only 6 handlers registered to crm_lib_service in plugin.c */
-    CRM_CHECK(class < 6, crm_err("Invalid message class: %d", class);
-              return FALSE);
+    switch (msg_class) {
+        case crm_class_cluster:
+            break;
+        default:
+            crm_err("Invalid message class: %d", msg_class);
+            return FALSE;
+    }
 
-#if !SUPPORT_PLUGIN
     CRM_CHECK(dest != crm_msg_ais, return FALSE);
-#endif
 
     if(local_name == NULL) {
         local_name = get_local_node_name();
@@ -705,7 +533,7 @@ send_cluster_text(int class, const char *data,
 
     msg_id++;
     msg->id = msg_id;
-    msg->header.id = class;
+    msg->header.id = msg_class;
     msg->header.error = CS_OK;
 
     msg->host.type = dest;
@@ -779,13 +607,6 @@ send_cluster_text(int class, const char *data,
     }
     free(target);
 
-#if SUPPORT_PLUGIN
-    /* The plugin is the only time we don't use CPG messaging */
-    if(get_cluster_type() == pcmk_cluster_classic_ais) {
-        return send_plugin_text(class, iov);
-    }
-#endif
-
     send_cpg_iov(iov);
 
     return TRUE;
@@ -797,15 +618,13 @@ text2msg_type(const char *text)
     int type = crm_msg_none;
 
     CRM_CHECK(text != NULL, return type);
+    text = pcmk_message_name(text);
     if (safe_str_eq(text, "ais")) {
         type = crm_msg_ais;
-    } else if (safe_str_eq(text, "crm_plugin")) {
-        type = crm_msg_ais;
     } else if (safe_str_eq(text, CRM_SYSTEM_CIB)) {
         type = crm_msg_cib;
-    } else if (safe_str_eq(text, CRM_SYSTEM_CRMD)) {
-        type = crm_msg_crmd;
-    } else if (safe_str_eq(text, CRM_SYSTEM_DC)) {
+    } else if (safe_str_eq(text, CRM_SYSTEM_CRMD)
+               || safe_str_eq(text, CRM_SYSTEM_DC)) {
         type = crm_msg_crmd;
     } else if (safe_str_eq(text, CRM_SYSTEM_TENGINE)) {
         type = crm_msg_te;
