@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -73,32 +62,11 @@ static char *
 generateReference(const char *custom1, const char *custom2)
 {
     static uint ref_counter = 0;
-    const char *local_cust1 = custom1;
-    const char *local_cust2 = custom2;
-    int reference_len = 4;
-    char *since_epoch = NULL;
 
-    reference_len += 20;        /* too big */
-    reference_len += 40;        /* too big */
-
-    if (local_cust1 == NULL) {
-        local_cust1 = "_empty_";
-    }
-    reference_len += strlen(local_cust1);
-
-    if (local_cust2 == NULL) {
-        local_cust2 = "_empty_";
-    }
-    reference_len += strlen(local_cust2);
-
-    since_epoch = calloc(1, reference_len);
-
-    if (since_epoch != NULL) {
-        sprintf(since_epoch, "%s-%s-%lu-%u",
-                local_cust1, local_cust2, (unsigned long)time(NULL), ref_counter++);
-    }
-
-    return since_epoch;
+    return crm_strdup_printf("%s-%s-%lu-%u",
+                             (custom1? custom1 : "_empty_"),
+                             (custom2? custom2 : "_empty_"),
+                             (unsigned long)time(NULL), ref_counter++);
 }
 
 xmlNode *
@@ -118,7 +86,7 @@ create_request_adv(const char *task, xmlNode * msg_data,
         crm_err("No sys from specified");
     }
 
-    /* host_from will get set for us if necessary by CRMd when routed */
+    // host_from will get set for us if necessary by the controller when routed
     request = create_xml_node(NULL, __FUNCTION__);
     crm_xml_add(request, F_CRM_ORIGIN, origin);
     crm_xml_add(request, F_TYPE, T_CRM);
@@ -248,6 +216,23 @@ crm_client_name(crm_client_t * c)
     }
 }
 
+const char *
+crm_client_type_text(enum client_type client_type)
+{
+    switch (client_type) {
+        case CRM_CLIENT_IPC:
+            return "IPC";
+        case CRM_CLIENT_TCP:
+            return "TCP";
+#ifdef HAVE_GNUTLS_GNUTLS_H
+        case CRM_CLIENT_TLS:
+            return "TLS";
+#endif
+        default:
+            return "unknown";
+    }
+}
+
 void
 crm_client_init(void)
 {
@@ -293,6 +278,58 @@ crm_client_disconnect_all(qb_ipcs_service_t *service)
     }
 }
 
+/*!
+ * \internal
+ * \brief Allocate a new crm_client_t object based on an IPC connection
+ *
+ * \param[in] c           IPC connection (or NULL to allocate generic client)
+ * \param[in] key         Connection table key (or NULL to use sane default)
+ * \param[in] uid_client  UID corresponding to c (ignored if c is NULL)
+ *
+ * \return Pointer to new crm_client_t (or NULL on error)
+ */
+static crm_client_t *
+client_from_connection(qb_ipcs_connection_t *c, void *key, uid_t uid_client)
+{
+    crm_client_t *client = calloc(1, sizeof(crm_client_t));
+
+    if (client == NULL) {
+        crm_perror(LOG_ERR, "Allocating client");
+        return NULL;
+    }
+
+    if (c) {
+#if ENABLE_ACL
+        client->user = uid2username(uid_client);
+        if (client->user == NULL) {
+            client->user = strdup("#unprivileged");
+            CRM_CHECK(client->user != NULL, free(client); return NULL);
+            crm_err("Unable to enforce ACLs for user ID %d, assuming unprivileged",
+                    uid_client);
+        }
+#endif
+        client->ipcs = c;
+        client->kind = CRM_CLIENT_IPC;
+        client->pid = crm_ipcs_client_pid(c);
+        if (key == NULL) {
+            key = c;
+        }
+    }
+
+    client->id = crm_generate_uuid();
+    if (client->id == NULL) {
+        crm_err("Could not generate UUID for client");
+        free(client->user);
+        free(client);
+        return NULL;
+    }
+    if (key == NULL) {
+        key = client->id;
+    }
+    g_hash_table_insert(client_connections, key, client);
+    return client;
+}
+
 /*!
  * \brief Allocate a new crm_client_t object and generate its ID
  *
@@ -303,11 +340,9 @@ crm_client_disconnect_all(qb_ipcs_service_t *service)
 crm_client_t *
 crm_client_alloc(void *key)
 {
-    crm_client_t *client = calloc(1, sizeof(crm_client_t));
+    crm_client_t *client = client_from_connection(NULL, key, 0);
 
     CRM_ASSERT(client != NULL);
-    client->id = crm_generate_uuid();
-    g_hash_table_insert(client_connections, (key? key : client->id), client);
     return client;
 }
 
@@ -319,18 +354,16 @@ crm_client_new(qb_ipcs_connection_t * c, uid_t uid_client, gid_t gid_client)
 
     crm_client_t *client = NULL;
 
-    CRM_LOG_ASSERT(c);
-    if (c == NULL) {
-        return NULL;
-    }
+    CRM_CHECK(c != NULL, return NULL);
 
     if (uid_cluster == 0) {
         if (crm_user_lookup(CRM_DAEMON_USER, &uid_cluster, &gid_cluster) < 0) {
-            static bool have_error = FALSE;
-            if(have_error == FALSE) {
+            static bool need_log = TRUE;
+
+            if (need_log) {
                 crm_warn("Could not find user and group IDs for user %s",
                          CRM_DAEMON_USER);
-                have_error = TRUE;
+                need_log = FALSE;
             }
         }
     }
@@ -344,10 +377,10 @@ crm_client_new(qb_ipcs_connection_t * c, uid_t uid_client, gid_t gid_client)
     crm_client_init();
 
     /* TODO: Do our own auth checking, return NULL if unauthorized */
-    client = crm_client_alloc(c);
-    client->ipcs = c;
-    client->kind = CRM_CLIENT_IPC;
-    client->pid = crm_ipcs_client_pid(c);
+    client = client_from_connection(c, NULL, uid_client);
+    if (client == NULL) {
+        return NULL;
+    }
 
     if ((uid_client == 0) || (uid_client == uid_cluster)) {
         /* Remember when a connection came from root or hacluster */
@@ -356,12 +389,48 @@ crm_client_new(qb_ipcs_connection_t * c, uid_t uid_client, gid_t gid_client)
 
     crm_debug("Connecting %p for uid=%d gid=%d pid=%u id=%s", c, uid_client, gid_client, client->pid, client->id);
 
-#if ENABLE_ACL
-    client->user = uid2username(uid_client);
-#endif
     return client;
 }
 
+static struct iovec *
+pcmk__new_ipc_event()
+{
+    struct iovec *iov = calloc(2, sizeof(struct iovec));
+
+    CRM_ASSERT(iov != NULL);
+    return iov;
+}
+
+/*!
+ * \brief Free an I/O vector created by crm_ipc_prepare()
+ *
+ * \param[in] event  I/O vector to free
+ */
+void
+pcmk_free_ipc_event(struct iovec *event)
+{
+    if (event != NULL) {
+        free(event[0].iov_base);
+        free(event[1].iov_base);
+        free(event);
+    }
+}
+
+static void
+free_event(gpointer data)
+{
+    pcmk_free_ipc_event((struct iovec *) data);
+}
+
+static void
+add_event(crm_client_t *c, struct iovec *iov)
+{
+    if (c->event_queue == NULL) {
+        c->event_queue = g_queue_new();
+    }
+    g_queue_push_tail(c->event_queue, iov);
+}
+
 void
 crm_client_destroy(crm_client_t * c)
 {
@@ -386,14 +455,9 @@ crm_client_destroy(crm_client_t * c)
         g_source_remove(c->event_timer);
     }
 
-    crm_debug("Destroying %d events", g_list_length(c->event_queue));
-    while (c->event_queue) {
-        struct iovec *event = c->event_queue->data;
-
-        c->event_queue = g_list_remove(c->event_queue, event);
-        free(event[0].iov_base);
-        free(event[1].iov_base);
-        free(event);
+    if (c->event_queue) {
+        crm_debug("Destroying %d events", g_queue_get_length(c->event_queue));
+        g_queue_free_full(c->event_queue, free_event);
     }
 
     free(c->id);
@@ -457,9 +521,10 @@ crm_ipcs_recv(crm_client_t * c, void *data, size_t size, uint32_t * id, uint32_t
     }
 
     if (is_set(header->flags, crm_ipc_proxied)) {
-        /* mark this client as being the endpoint of a proxy connection.
-         * Proxy connections responses are sent on the event channel to avoid
-         * blocking the proxy daemon (crmd) */
+        /* Mark this client as being the endpoint of a proxy connection.
+         * Proxy connections responses are sent on the event channel, to avoid
+         * blocking the controller serving as proxy.
+         */
         c->flags |= crm_client_flag_ipc_proxied;
     }
 
@@ -481,7 +546,8 @@ crm_ipcs_recv(crm_client_t * c, void *data, size_t size, uint32_t * id, uint32_t
         text = uncompressed;
 
         if (rc != BZ_OK) {
-            crm_err("Decompression failed: %s (%d)", bz2_strerror(rc), rc);
+            crm_err("Decompression failed: %s " CRM_XS " bzerror=%d",
+                    bz2_strerror(rc), rc);
             free(uncompressed);
             return NULL;
         }
@@ -518,8 +584,8 @@ crm_ipcs_flush_events_cb(gpointer data)
 static inline void
 delay_next_flush(crm_client_t *c, unsigned int queue_len)
 {
-    /* Delay a maximum of 5 seconds */
-    guint delay = (queue_len < 40)? (1000 + 100 * queue_len) : 5000;
+    /* Delay a maximum of 1.5 seconds */
+    guint delay = (queue_len < 5)? (1000 + 100 * queue_len) : 1500;
 
     c->event_timer = g_timeout_add(delay, crm_ipcs_flush_events_cb, c);
 }
@@ -540,15 +606,26 @@ crm_ipcs_flush_events(crm_client_t * c)
         return pcmk_ok;
     }
 
-    queue_len = g_list_length(c->event_queue);
-    while (c->event_queue && sent < 100) {
+    if (c->event_queue) {
+        queue_len = g_queue_get_length(c->event_queue);
+    }
+    while (sent < 100) {
         struct crm_ipc_response_header *header = NULL;
-        struct iovec *event = c->event_queue->data;
+        struct iovec *event = NULL;
+
+        if (c->event_queue) {
+            // We don't pop unless send is successful
+            event = g_queue_peek_head(c->event_queue);
+        }
+        if (event == NULL) { // Queue is empty
+            break;
+        }
 
         rc = qb_ipcs_event_sendv(c->ipcs, event, 2);
         if (rc < 0) {
             break;
         }
+        event = g_queue_pop_head(c->event_queue);
 
         sent++;
         header = event[0].iov_base;
@@ -560,11 +637,7 @@ crm_ipcs_flush_events(crm_client_t * c)
                       header->qb.id, c->ipcs, c->pid, (long long) rc,
                       (char *) (event[1].iov_base));
         }
-
-        c->event_queue = g_list_remove(c->event_queue, event);
-        free(event[0].iov_base);
-        free(event[1].iov_base);
-        free(event);
+        pcmk_free_ipc_event(event);
     }
 
     queue_len -= sent;
@@ -626,9 +699,7 @@ crm_ipc_prepare(uint32_t request, xmlNode * message, struct iovec ** result, uin
     CRM_LOG_ASSERT(max_send_size != 0);
 
     *result = NULL;
-    iov = calloc(2, sizeof(struct iovec));
-
-
+    iov = pcmk__new_ipc_event();
     iov[0].iov_len = hdr_offset;
     iov[0].iov_base = header;
 
@@ -668,10 +739,7 @@ crm_ipc_prepare(uint32_t request, xmlNode * message, struct iovec ** result, uin
                  header->size_uncompressed, max_send_size, 4 * biggest);
 
             free(compressed);
-            free(buffer);
-            free(header);
-            free(iov);
-
+            pcmk_free_ipc_event(iov);
             return rc;
         }
     }
@@ -707,10 +775,10 @@ crm_ipcs_sendv(crm_client_t * c, struct iovec * iov, enum crm_ipc_flags flags)
 
         if (flags & crm_ipc_server_free) {
             crm_trace("Sending the original to %p[%d]", c->ipcs, c->pid);
-            c->event_queue = g_list_append(c->event_queue, iov);
+            add_event(c, iov);
 
         } else {
-            struct iovec *iov_copy = calloc(2, sizeof(struct iovec));
+            struct iovec *iov_copy = pcmk__new_ipc_event();
 
             crm_trace("Sending a copy to %p[%d]", c->ipcs, c->pid);
             iov_copy[0].iov_len = iov[0].iov_len;
@@ -721,7 +789,7 @@ crm_ipcs_sendv(crm_client_t * c, struct iovec * iov, enum crm_ipc_flags flags)
             iov_copy[1].iov_base = malloc(iov[1].iov_len);
             memcpy(iov_copy[1].iov_base, iov[1].iov_base, iov[1].iov_len);
 
-            c->event_queue = g_list_append(c->event_queue, iov_copy);
+            add_event(c, iov_copy);
         }
 
     } else {
@@ -729,8 +797,10 @@ crm_ipcs_sendv(crm_client_t * c, struct iovec * iov, enum crm_ipc_flags flags)
 
         rc = qb_ipcs_response_sendv(c->ipcs, iov, 2);
         if (rc < header->qb.size) {
-            crm_notice("Response %d to %p[%d] (%u bytes) failed: %s (%d)",
-                       header->qb.id, c->ipcs, c->pid, header->qb.size, pcmk_strerror(rc), rc);
+            crm_notice("Response %d to pid %d failed: %s "
+                       CRM_XS " bytes=%u rc=%lld ipcs=%p",
+                       header->qb.id, c->pid, pcmk_strerror(rc),
+                       header->qb.size, (long long) rc, c->ipcs);
 
         } else {
             crm_trace("Response %d sent, %lld bytes to %p[%d]",
@@ -738,9 +808,7 @@ crm_ipcs_sendv(crm_client_t * c, struct iovec * iov, enum crm_ipc_flags flags)
         }
 
         if (flags & crm_ipc_server_free) {
-            free(iov[0].iov_base);
-            free(iov[1].iov_base);
-            free(iov);
+            pcmk_free_ipc_event(iov);
         }
     }
 
@@ -772,13 +840,11 @@ crm_ipcs_send(crm_client_t * c, uint32_t request, xmlNode * message,
     rc = crm_ipc_prepare(request, message, &iov, ipc_buffer_max);
     if (rc > 0) {
         rc = crm_ipcs_sendv(c, iov, flags | crm_ipc_server_free);
-
     } else {
-        free(iov);
-        crm_notice("Message to %p[%d] failed: %s (%d)",
-                   c->ipcs, c->pid, pcmk_strerror(rc), rc);
+        pcmk_free_ipc_event(iov);
+        crm_notice("Message to pid %d failed: %s " CRM_XS " rc=%lld ipcs=%p",
+                   c->pid, pcmk_strerror(rc), (long long) rc, c->ipcs);
     }
-
     return rc;
 }
 
@@ -814,7 +880,6 @@ struct crm_ipc_s {
     int need_reply;
     char *buffer;
     char *name;
-    uint32_t buffer_flags;
 
     qb_ipcc_connection_t *ipc;
 
@@ -1019,7 +1084,8 @@ crm_ipc_decompress(crm_ipc_t * client)
                                         client->buffer + hdr_offset, header->size_compressed, 1, 0);
 
         if (rc != BZ_OK) {
-            crm_err("Decompression failed: %s (%d)", bz2_strerror(rc), rc);
+            crm_err("Decompression failed: %s " CRM_XS " bzerror=%d",
+                    bz2_strerror(rc), rc);
             free(uncompressed);
             return -EILSEQ;
         }
@@ -1058,7 +1124,8 @@ crm_ipc_read(crm_ipc_t * client)
     crm_ipc_init();
 
     client->buffer[0] = 0;
-    client->msg_size = qb_ipcc_event_recv(client->ipc, client->buffer, client->buf_size - 1, 0);
+    client->msg_size = qb_ipcc_event_recv(client->ipc, client->buffer,
+                                          client->buf_size, 0);
     if (client->msg_size >= 0) {
         int rc = crm_ipc_decompress(client);
 
@@ -1323,9 +1390,7 @@ crm_ipc_send(crm_ipc_t * client, xmlNode * message, enum crm_ipc_flags flags, in
                  header->qb.id, client->name, client->ipc, pcmk_strerror(rc), rc);
     }
 
-    free(header);
-    free(iov[1].iov_base);
-    free(iov);
+    pcmk_free_ipc_event(iov);
     return rc;
 }
 
