@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -39,7 +28,6 @@ struct mainloop_child_s {
     pid_t pid;
     char *desc;
     unsigned timerid;
-    unsigned watchid;
     gboolean timeout;
     void *privatedata;
 
@@ -323,6 +311,17 @@ crm_signal(int sig, void (*dispatch) (int sig))
     return TRUE;
 }
 
+static void
+mainloop_destroy_signal_entry(int sig)
+{
+    crm_signal_t *tmp = crm_signals[sig];
+
+    crm_signals[sig] = NULL;
+
+    crm_trace("Destroying signal %d", sig);
+    mainloop_destroy_trigger((crm_trigger_t *) tmp);
+}
+
 gboolean
 mainloop_add_signal(int sig, void (*dispatch) (int sig))
 {
@@ -360,11 +359,7 @@ mainloop_add_signal(int sig, void (*dispatch) (int sig))
     crm_signals[sig]->signal = sig;
 
     if (crm_signal(sig, mainloop_signal_handler) == FALSE) {
-        crm_signal_t *tmp = crm_signals[sig];
-
-        crm_signals[sig] = NULL;
-
-        mainloop_destroy_trigger((crm_trigger_t *) tmp);
+        mainloop_destroy_signal_entry(sig);
         return FALSE;
     }
 #if 0
@@ -384,8 +379,6 @@ mainloop_add_signal(int sig, void (*dispatch) (int sig))
 gboolean
 mainloop_destroy_signal(int sig)
 {
-    crm_signal_t *tmp = NULL;
-
     if (sig >= NSIG || sig < 0) {
         crm_err("Signal %d is out of range", sig);
         return FALSE;
@@ -397,11 +390,7 @@ mainloop_destroy_signal(int sig)
     } else if (crm_signals[sig] == NULL) {
         return TRUE;
     }
-
-    crm_trace("Destroying signal %d", sig);
-    tmp = crm_signals[sig];
-    crm_signals[sig] = NULL;
-    mainloop_destroy_trigger((crm_trigger_t *) tmp);
+    mainloop_destroy_signal_entry(sig);
     return TRUE;
 }
 
@@ -410,9 +399,13 @@ static qb_array_t *gio_map = NULL;
 void
 mainloop_cleanup(void) 
 {
-    if(gio_map) {
+    if (gio_map) {
         qb_array_free(gio_map);
     }
+
+    for (int sig = 0; sig < NSIG; ++sig) {
+        mainloop_destroy_signal_entry(sig);
+    }
 }
 
 /*
@@ -681,7 +674,8 @@ mainloop_gio_callback(GIOChannel * gio, GIOCondition condition, gpointer data)
     }
 
     if (client->ipc && crm_ipc_connected(client->ipc) == FALSE) {
-        crm_err("Connection to %s[%p] closed (I/O condition=%d)", client->name, client, condition);
+        crm_err("Connection to %s closed " CRM_XS "client=%p condition=%d",
+                client->name, client, condition);
         keep = FALSE;
 
     } else if (condition & (G_IO_HUP | G_IO_NVAL | G_IO_ERR)) {
@@ -1143,8 +1137,6 @@ struct mainloop_timer_s {
         void *userdata;
 };
 
-struct mainloop_timer_s mainloop;
-
 static gboolean mainloop_timer_cb(gpointer user_data)
 {
     int id = 0;
@@ -1249,3 +1241,51 @@ mainloop_timer_del(mainloop_timer_t *t)
     }
 }
 
+/*
+ * Helpers to make sure certain events aren't lost at shutdown
+ */
+
+static gboolean
+drain_timeout_cb(gpointer user_data)
+{
+    bool *timeout_popped = (bool*) user_data;
+
+    *timeout_popped = TRUE;
+    return FALSE;
+}
+
+/*!
+ * \brief Process main loop events while a certain condition is met
+ *
+ * \param[in] mloop     Main loop to process
+ * \param[in] timer_ms  Don't process longer than this amount of time
+ * \param[in] check     Function that returns TRUE if events should be processed
+ *
+ * \note This function is intended to be called at shutdown if certain important
+ *       events should not be missed. The caller would likely quit the main loop
+ *       or exit after calling this function. The check() function will be
+ *       passed the remaining timeout in milliseconds.
+ */
+void
+pcmk_drain_main_loop(GMainLoop *mloop, guint timer_ms, bool (*check)(guint))
+{
+    bool timeout_popped = FALSE;
+    guint timer = 0;
+    GMainContext *ctx = NULL;
+
+    CRM_CHECK(mloop && check, return);
+
+    ctx = g_main_loop_get_context(mloop);
+    if (ctx) {
+        time_t start_time = time(NULL);
+
+        timer = g_timeout_add(timer_ms, drain_timeout_cb, &timeout_popped);
+        while (!timeout_popped
+               && check(timer_ms - (time(NULL) - start_time) * 1000)) {
+            g_main_context_iteration(ctx, TRUE);
+        }
+    }
+    if (!timeout_popped && (timer > 0)) {
+        g_source_remove(timer);
+    }
+}
