@@ -33,7 +33,7 @@
  * \internal
  * \brief Allocate a new name/value pair
  *
- * \param[in] name   New name (required)
+ * \param[in] name   New name
  * \param[in] value  New value
  *
  * \return Newly allocated name/value pair
@@ -43,15 +43,11 @@
 static pcmk_nvpair_t *
 pcmk__new_nvpair(const char *name, const char *value)
 {
-    pcmk_nvpair_t *nvpair = NULL;
+    pcmk_nvpair_t *nvpair = calloc(1, sizeof(pcmk_nvpair_t));
 
-    CRM_ASSERT(name);
-
-    nvpair = calloc(1, sizeof(pcmk_nvpair_t));
     CRM_ASSERT(nvpair);
-
     nvpair->name = strdup(name);
-    nvpair->value = value? strdup(value) : NULL;
+    nvpair->value = strdup(value);
     return nvpair;
 }
 
@@ -352,35 +348,6 @@ crm_xml_add_ms(xmlNode *node, const char *name, guint ms)
     return added;
 }
 
-// Maximum size of null-terminated string representation of 64-bit integer
-// -9223372036854775808
-#define LLSTRSIZE 21
-
-/*!
- * \brief Create an XML attribute with specified name and long long int value
- *
- * This is like \c crm_xml_add() but taking a long long int value. It is a
- * useful equivalent for defined types like time_t, etc.
- *
- * \param[in,out] xml    XML node to modify
- * \param[in]     name   Attribute name to set
- * \param[in]     value  Attribute value to set
- *
- * \return New value as string on success, \c NULL otherwise
- * \note This does nothing if xml or name are \c NULL or empty.
- *       This does not support greater than 64-bit values.
- */
-const char *
-crm_xml_add_ll(xmlNode *xml, const char *name, long long value)
-{
-    char s[LLSTRSIZE] = { '\0', };
-
-    if (snprintf(s, LLSTRSIZE, "%lld", (long long) value) == LLSTRSIZE) {
-        return NULL;
-    }
-    return crm_xml_add(xml, name, s);
-}
-
 /*!
  * \brief Retrieve the value of an XML attribute
  *
@@ -439,70 +406,25 @@ crm_element_value_int(const xmlNode *data, const char *name, int *dest)
 }
 
 /*!
- * \brief Retrieve the long long integer value of an XML attribute
+ * \brief Retrieve the millisecond value of an XML attribute
  *
- * This is like \c crm_element_value() but getting the value as a long long int.
+ * This is like \c crm_element_value() but returning the value as a guint.
  *
- * \param[in] data   XML node to check
- * \param[in] name   Attribute name to check
- * \param[in] dest   Where to store element value
- *
- * \return 0 on success, -1 otherwise
- */
-int
-crm_element_value_ll(const xmlNode *data, const char *name, long long *dest)
-{
-    const char *value = NULL;
-
-    CRM_CHECK(dest != NULL, return -1);
-    value = crm_element_value(data, name);
-    if (value) {
-        errno = 0;
-        *dest = crm_int_helper(value, NULL);
-        if (errno == 0) {
-            return 0;
-        }
-    }
-    return -1;
-}
-
-int
-crm_element_value_const_int(const xmlNode * data, const char *name, int *dest)
-{
-    return crm_element_value_int((xmlNode *) data, name, dest);
-}
-
-const char *
-crm_element_value_const(const xmlNode * data, const char *name)
-{
-    return crm_element_value((xmlNode *) data, name);
-}
-
-/*!
- * \brief Retrieve the seconds-since-epoch value of an XML attribute
- *
- * This is like \c crm_element_value() but returning the value as a time_t.
- *
- * \param[in]  xml    XML node to check
+ * \param[in]  data   XML node to check
  * \param[in]  name   Attribute name to check
  * \param[out] dest   Where to store attribute value
  *
  * \return \c pcmk_ok on success, -1 otherwise
  */
 int
-crm_element_value_epoch(const xmlNode *xml, const char *name, time_t *dest)
+crm_element_value_ms(const xmlNode *data, const char *name, guint *dest)
 {
-    long long value_ll = 0;
+    const char *value = NULL;
 
-    if (crm_element_value_ll(xml, name, &value_ll) < 0) {
-        return -1;
-    }
-
-    /* Unfortunately, we can't do any bounds checking, since time_t has neither
-     * standardized bounds nor constants defined for them.
-     */
-    *dest = (time_t) value_ll;
-    return pcmk_ok;
+    CRM_CHECK(dest != NULL, return -1);
+    value = crm_element_value(data, name);
+    *dest = crm_parse_ms(value);
+    return errno? -1 : 0;
 }
 
 /*!
