@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2017 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -25,79 +25,83 @@
 /*!
  * \brief Generate an operation key
  *
- * \param[in] rsc_id    ID of resource being operated on
- * \param[in] op_type   Operation name
- * \param[in] interval  Operation interval
+ * \param[in] rsc_id       ID of resource being operated on
+ * \param[in] op_type      Operation name
+ * \param[in] interval_ms  Operation interval
  *
  * \return Newly allocated memory containing operation key as string
  *
  * \note It is the caller's responsibility to free() the result.
  */
 char *
-generate_op_key(const char *rsc_id, const char *op_type, int interval)
+generate_op_key(const char *rsc_id, const char *op_type, guint interval_ms)
 {
     CRM_ASSERT(rsc_id != NULL);
     CRM_ASSERT(op_type != NULL);
-    CRM_ASSERT(interval >= 0);
-    return crm_strdup_printf("%s_%s_%d", rsc_id, op_type, interval);
+    return crm_strdup_printf(CRM_OP_FMT, rsc_id, op_type, interval_ms);
 }
 
 gboolean
-parse_op_key(const char *key, char **rsc_id, char **op_type, int *interval)
+parse_op_key(const char *key, char **rsc_id, char **op_type, guint *interval_ms)
 {
     char *notify = NULL;
     char *mutable_key = NULL;
     char *mutable_key_ptr = NULL;
-    int len = 0, offset = 0, ch = 0;
+    size_t len = 0, offset = 0;
+    unsigned long long ch = 0;
+    guint local_interval_ms = 0;
 
-    CRM_CHECK(key != NULL, return FALSE);
-
-    *interval = 0;
-    len = strlen(key);
-    offset = len - 1;
-
-    crm_trace("Source: %s", key);
-
-    while (offset > 0 && isdigit(key[offset])) {
-        int digits = len - offset;
-
-        ch = key[offset] - '0';
-        CRM_CHECK(ch < 10, return FALSE);
-        CRM_CHECK(ch >= 0, return FALSE);
-        while (digits > 1) {
-            digits--;
-            ch = ch * 10;
-        }
-        *interval += ch;
-        offset--;
+    // Initialize output variables in case of early return
+    if (rsc_id) {
+        *rsc_id = NULL;
+    }
+    if (op_type) {
+        *op_type = NULL;
+    }
+    if (interval_ms) {
+        *interval_ms = 0;
     }
 
-    crm_trace("  Interval: %d", *interval);
-    CRM_CHECK(key[offset] == '_', return FALSE);
+    CRM_CHECK(key && *key, return FALSE);
 
-    mutable_key = strdup(key);
-    mutable_key[offset] = 0;
+    // Parse interval at end of string
+    len = strlen(key);
+    offset = len - 1;
+    while ((offset > 0) && isdigit(key[offset])) {
+        ch = key[offset] - '0';
+        for (int digits = len - offset; digits > 1; --digits) {
+            ch = ch * 10;
+        }
+        local_interval_ms += ch;
+        offset--;
+    }
+    crm_trace("Operation key '%s' has interval %ums", key, local_interval_ms);
+    if (interval_ms) {
+        *interval_ms = local_interval_ms;
+    }
+
+    CRM_CHECK((offset != (len - 1)) && (key[offset] == '_'), return FALSE);
+
+    mutable_key = strndup(key, offset);
     offset--;
 
     while (offset > 0 && key[offset] != '_') {
         offset--;
     }
 
-    CRM_CHECK(key[offset] == '_', free(mutable_key);
-              return FALSE);
+    CRM_CHECK(key[offset] == '_',
+              free(mutable_key); return FALSE);
 
     mutable_key_ptr = mutable_key + offset + 1;
 
     crm_trace("  Action: %s", mutable_key_ptr);
-
-    *op_type = strdup(mutable_key_ptr);
+    if (op_type) {
+        *op_type = strdup(mutable_key_ptr);
+    }
 
     mutable_key[offset] = 0;
     offset--;
 
-    CRM_CHECK(mutable_key != mutable_key_ptr, free(mutable_key);
-              return FALSE);
-
     notify = strstr(mutable_key, "_post_notify");
     if (notify && safe_str_eq(notify, "_post_notify")) {
         notify[0] = 0;
@@ -109,7 +113,11 @@ parse_op_key(const char *key, char **rsc_id, char **op_type, int *interval)
     }
 
     crm_trace("  Resource: %s", mutable_key);
-    *rsc_id = mutable_key;
+    if (rsc_id) {
+        *rsc_id = mutable_key;
+    } else {
+        free(mutable_key);
+    }
 
     return TRUE;
 }
@@ -117,57 +125,18 @@ parse_op_key(const char *key, char **rsc_id, char **op_type, int *interval)
 char *
 generate_notify_key(const char *rsc_id, const char *notify_type, const char *op_type)
 {
-    int len = 12;
-    char *op_id = NULL;
-
     CRM_CHECK(rsc_id != NULL, return NULL);
     CRM_CHECK(op_type != NULL, return NULL);
     CRM_CHECK(notify_type != NULL, return NULL);
-
-    len += strlen(op_type);
-    len += strlen(rsc_id);
-    len += strlen(notify_type);
-    if(len > 0) {
-        op_id = malloc(len);
-    }
-    if (op_id != NULL) {
-        sprintf(op_id, "%s_%s_notify_%s_0", rsc_id, notify_type, op_type);
-    }
-    return op_id;
-}
-
-char *
-generate_transition_magic_v202(const char *transition_key, int op_status)
-{
-    int len = 80;
-    char *fail_state = NULL;
-
-    CRM_CHECK(transition_key != NULL, return NULL);
-
-    len += strlen(transition_key);
-
-    fail_state = malloc(len);
-    if (fail_state != NULL) {
-        snprintf(fail_state, len, "%d:%s", op_status, transition_key);
-    }
-    return fail_state;
+    return crm_strdup_printf("%s_%s_notify_%s_0",
+                             rsc_id, notify_type, op_type);
 }
 
 char *
 generate_transition_magic(const char *transition_key, int op_status, int op_rc)
 {
-    int len = 80;
-    char *fail_state = NULL;
-
     CRM_CHECK(transition_key != NULL, return NULL);
-
-    len += strlen(transition_key);
-
-    fail_state = malloc(len);
-    if (fail_state != NULL) {
-        snprintf(fail_state, len, "%d:%d;%s", op_status, op_rc, transition_key);
-    }
-    return fail_state;
+    return crm_strdup_printf("%d:%d;%s", op_status, op_rc, transition_key);
 }
 
 gboolean
@@ -199,84 +168,35 @@ decode_transition_magic(const char *magic, char **uuid, int *transition_id, int
 char *
 generate_transition_key(int transition_id, int action_id, int target_rc, const char *node)
 {
-    int len = 40;
-    char *fail_state = NULL;
-
     CRM_CHECK(node != NULL, return NULL);
-
-    len += strlen(node);
-
-    fail_state = malloc(len);
-    if (fail_state != NULL) {
-        snprintf(fail_state, len, "%d:%d:%d:%-*s", action_id, transition_id, target_rc, 36, node);
-    }
-    return fail_state;
+    return crm_strdup_printf("%d:%d:%d:%-*s",
+                             action_id, transition_id, target_rc, 36, node);
 }
 
 gboolean
 decode_transition_key(const char *key, char **uuid, int *transition_id, int *action_id,
                       int *target_rc)
 {
-    int res = 0;
-    gboolean done = FALSE;
-
     CRM_CHECK(uuid != NULL, return FALSE);
     CRM_CHECK(target_rc != NULL, return FALSE);
     CRM_CHECK(action_id != NULL, return FALSE);
     CRM_CHECK(transition_id != NULL, return FALSE);
 
-    *uuid = calloc(1, 37);
-    res = sscanf(key, "%d:%d:%d:%36s", action_id, transition_id, target_rc, *uuid);
-    switch (res) {
-        case 4:
-            /* Post Pacemaker 0.6 */
-            done = TRUE;
-            break;
-        case 3:
-        case 2:
-            /* this can be tricky - the UUID might start with an integer */
-
-            /* Until Pacemaker 0.6 */
-            done = TRUE;
-            *target_rc = -1;
-            res = sscanf(key, "%d:%d:%36s", action_id, transition_id, *uuid);
-            if (res == 2) {
-                *action_id = -1;
-                res = sscanf(key, "%d:%36s", transition_id, *uuid);
-                CRM_CHECK(res == 2, done = FALSE);
-
-            } else if (res != 3) {
-                CRM_CHECK(res == 3, done = FALSE);
-            }
-            break;
-
-        case 1:
-            /* Prior to Heartbeat 2.0.8 */
-            done = TRUE;
-            *action_id = -1;
-            *target_rc = -1;
-            res = sscanf(key, "%d:%36s", transition_id, *uuid);
-            CRM_CHECK(res == 2, done = FALSE);
-            break;
-        default:
-            crm_crit("Unhandled sscanf result (%d) for %s", res, key);
-    }
-
-    if (strlen(*uuid) != 36) {
-        crm_warn("Bad UUID (%s) in sscanf result (%d) for %s", *uuid, res, key);
-    }
-
-    if (done == FALSE) {
-        crm_err("Cannot decode '%s' rc=%d", key, res);
-
+    *uuid = calloc(37, sizeof(char));
+    if (sscanf(key, "%d:%d:%d:%36s",
+               action_id, transition_id, target_rc, *uuid) != 4) {
+        crm_err("Invalid transition key '%s'", key);
         free(*uuid);
         *uuid = NULL;
         *target_rc = -1;
         *action_id = -1;
         *transition_id = -1;
+        return FALSE;
     }
-
-    return done;
+    if (strlen(*uuid) != 36) {
+        crm_warn("Invalid UUID '%s' in transition key '%s'", *uuid, key);
+    }
+    return TRUE;
 }
 
 void
@@ -284,7 +204,7 @@ filter_action_parameters(xmlNode * param_set, const char *version)
 {
     char *key = NULL;
     char *timeout = NULL;
-    char *interval = NULL;
+    char *interval_ms_s = NULL;
 
     const char *attr_filter[] = {
         XML_ATTR_ID,
@@ -311,8 +231,8 @@ filter_action_parameters(xmlNode * param_set, const char *version)
         xml_remove_prop(param_set, attr_filter[lpc]);
     }
 
-    key = crm_meta_name(XML_LRM_ATTR_INTERVAL);
-    interval = crm_element_value_copy(param_set, key);
+    key = crm_meta_name(XML_LRM_ATTR_INTERVAL_MS);
+    interval_ms_s = crm_element_value_copy(param_set, key);
     free(key);
 
     key = crm_meta_name(XML_ATTR_TIMEOUT);
@@ -336,14 +256,14 @@ filter_action_parameters(xmlNode * param_set, const char *version)
         }
     }
 
-    if (crm_get_msec(interval) > 0 && compare_version(version, "1.0.8") > 0) {
+    if (interval_ms_s && strcmp(interval_ms_s, "0")) {
         /* Re-instate the operation's timeout value */
         if (timeout != NULL) {
             crm_xml_add(param_set, key, timeout);
         }
     }
 
-    free(interval);
+    free(interval_ms_s);
     free(timeout);
     free(key);
 }
@@ -371,7 +291,7 @@ append_digest(lrmd_event_data_t * op, xmlNode * update, const char *version, con
 
 #if 0
     if (level < get_crm_log_level()
-        && op->interval == 0 && crm_str_eq(op->op_type, CRMD_ACTION_START, TRUE)) {
+        && op->interval_ms == 0 && crm_str_eq(op->op_type, CRMD_ACTION_START, TRUE)) {
         char *digest_source = dump_xml_unformatted(args_xml);
 
         do_crm_log(level, "Calculated digest %s for %s (%s). Source: %s\n",
@@ -427,25 +347,25 @@ did_rsc_op_fail(lrmd_event_data_t * op, int target_rc)
 /*!
  * \brief Create a CIB XML element for an operation
  *
- * \param[in] parent    If not NULL, make new XML node a child of this one
- * \param[in] prefix    Generate an ID using this prefix
- * \param[in] task      Operation task to set
- * \param[in] interval  Operation interval to set
- * \param[in] timeout   If not NULL, operation timeout to set
+ * \param[in] parent         If not NULL, make new XML node a child of this one
+ * \param[in] prefix         Generate an ID using this prefix
+ * \param[in] task           Operation task to set
+ * \param[in] interval_spec  Operation interval to set
+ * \param[in] timeout        If not NULL, operation timeout to set
  *
  * \return New XML object on success, NULL otherwise
  */
 xmlNode *
 crm_create_op_xml(xmlNode *parent, const char *prefix, const char *task,
-                  const char *interval, const char *timeout)
+                  const char *interval_spec, const char *timeout)
 {
     xmlNode *xml_op;
 
-    CRM_CHECK(prefix && task && interval, return NULL);
+    CRM_CHECK(prefix && task && interval_spec, return NULL);
 
     xml_op = create_xml_node(parent, XML_ATTR_OP);
-    crm_xml_set_id(xml_op, "%s-%s-%s", prefix, task, interval);
-    crm_xml_add(xml_op, XML_LRM_ATTR_INTERVAL, interval);
+    crm_xml_set_id(xml_op, "%s-%s-%s", prefix, task, interval_spec);
+    crm_xml_add(xml_op, XML_LRM_ATTR_INTERVAL, interval_spec);
     crm_xml_add(xml_op, "name", task);
     if (timeout) {
         crm_xml_add(xml_op, XML_ATTR_TIMEOUT, timeout);
@@ -466,19 +386,18 @@ create_operation_update(xmlNode * parent, lrmd_event_data_t * op, const char * c
 
     xmlNode *xml_op = NULL;
     const char *task = NULL;
-    gboolean dc_munges_migrate_ops = (compare_version(caller_version, "3.0.3") < 0);
-    gboolean dc_needs_unique_ops = (compare_version(caller_version, "3.0.6") < 0);
 
     CRM_CHECK(op != NULL, return NULL);
-    do_crm_log(level, "%s: Updating resource %s after %s op %s (interval=%d)",
+    do_crm_log(level, "%s: Updating resource %s after %s op %s (interval=%u)",
                origin, op->rsc_id, op->op_type, services_lrm_status_str(op->op_status),
-               op->interval);
+               op->interval_ms);
 
     crm_trace("DC version: %s", caller_version);
 
     task = op->op_type;
-    /* remap the task name under various scenarios
-     * this makes life easier for the PE when trying determine the current state
+
+    /* Record a successful reload as a start, and a failed reload as a monitor,
+     * to make life easier for the scheduler when determining the current state.
      */
     if (crm_str_eq(task, "reload", TRUE)) {
         if (op->op_status == PCMK_LRM_OP_DONE) {
@@ -486,26 +405,10 @@ create_operation_update(xmlNode * parent, lrmd_event_data_t * op, const char * c
         } else {
             task = CRMD_ACTION_STATUS;
         }
-
-    } else if (dc_munges_migrate_ops && crm_str_eq(task, CRMD_ACTION_MIGRATE, TRUE)) {
-        /* if the migrate_from fails it will have enough info to do the right thing */
-        if (op->op_status == PCMK_LRM_OP_DONE) {
-            task = CRMD_ACTION_STOP;
-        } else {
-            task = CRMD_ACTION_STATUS;
-        }
-
-    } else if (dc_munges_migrate_ops
-               && op->op_status == PCMK_LRM_OP_DONE
-               && crm_str_eq(task, CRMD_ACTION_MIGRATED, TRUE)) {
-        task = CRMD_ACTION_START;
     }
 
-    key = generate_op_key(op->rsc_id, task, op->interval);
-    if (dc_needs_unique_ops && op->interval > 0) {
-        op_id = strdup(key);
-
-    } else if (crm_str_eq(task, CRMD_ACTION_NOTIFY, TRUE)) {
+    key = generate_op_key(op->rsc_id, task, op->interval_ms);
+    if (crm_str_eq(task, CRMD_ACTION_NOTIFY, TRUE)) {
         const char *n_type = crm_meta_value(op->params, "notify_type");
         const char *n_task = crm_meta_value(op->params, "notify_operation");
 
@@ -513,19 +416,25 @@ create_operation_update(xmlNode * parent, lrmd_event_data_t * op, const char * c
         CRM_LOG_ASSERT(n_task != NULL);
         op_id = generate_notify_key(op->rsc_id, n_type, n_task);
 
-        /* these are not yet allowed to fail */
-        op->op_status = PCMK_LRM_OP_DONE;
-        op->rc = 0;
+        if (op->op_status != PCMK_LRM_OP_PENDING) {
+            /* Ignore notify errors.
+             *
+             * @TODO It might be better to keep the correct result here, and
+             * ignore it in process_graph_event().
+             */
+            op->op_status = PCMK_LRM_OP_DONE;
+            op->rc = 0;
+        }
 
     } else if (did_rsc_op_fail(op, target_rc)) {
         op_id = generate_op_key(op->rsc_id, "last_failure", 0);
-        if (op->interval == 0) {
-            /* Ensure 'last' gets updated too in case recording-pending="true" */
+        if (op->interval_ms == 0) {
+            // Ensure 'last' gets updated, in case record-pending is true
             op_id_additional = generate_op_key(op->rsc_id, "last", 0);
         }
         exit_reason = op->exit_reason;
 
-    } else if (op->interval > 0) {
+    } else if (op->interval_ms > 0) {
         op_id = strdup(key);
 
     } else {
@@ -539,9 +448,9 @@ create_operation_update(xmlNode * parent, lrmd_event_data_t * op, const char * c
     }
 
     if (op->user_data == NULL) {
-        crm_debug("Generating fake transition key for:"
-                  " %s_%s_%d %d from %s",
-                  op->rsc_id, op->op_type, op->interval, op->call_id, origin);
+        crm_debug("Generating fake transition key for: " CRM_OP_FMT " %d from %s",
+                  op->rsc_id, op->op_type, op->interval_ms,
+                  op->call_id, origin);
         local_user_data = generate_transition_key(-1, op->call_id, target_rc, FAKE_TE_ID);
         op->user_data = local_user_data;
     }
@@ -563,15 +472,15 @@ create_operation_update(xmlNode * parent, lrmd_event_data_t * op, const char * c
     crm_xml_add_int(xml_op, XML_LRM_ATTR_CALLID, op->call_id);
     crm_xml_add_int(xml_op, XML_LRM_ATTR_RC, op->rc);
     crm_xml_add_int(xml_op, XML_LRM_ATTR_OPSTATUS, op->op_status);
-    crm_xml_add_int(xml_op, XML_LRM_ATTR_INTERVAL, op->interval);
+    crm_xml_add_ms(xml_op, XML_LRM_ATTR_INTERVAL_MS, op->interval_ms);
 
     if (compare_version("2.1", caller_version) <= 0) {
         if (op->t_run || op->t_rcchange || op->exec_time || op->queue_time) {
-            crm_trace("Timing data (%s_%s_%d): last=%u change=%u exec=%u queue=%u",
-                      op->rsc_id, op->op_type, op->interval,
+            crm_trace("Timing data (" CRM_OP_FMT "): last=%u change=%u exec=%u queue=%u",
+                      op->rsc_id, op->op_type, op->interval_ms,
                       op->t_run, op->t_rcchange, op->exec_time, op->queue_time);
 
-            if (op->interval == 0) {
+            if (op->interval_ms == 0) {
                 /* The values are the same for non-recurring ops */
                 crm_xml_add_int(xml_op, XML_RSC_OP_LAST_RUN, op->t_run);
                 crm_xml_add_int(xml_op, XML_RSC_OP_LAST_CHANGE, op->t_run);
@@ -642,9 +551,7 @@ crm_op_needs_metadata(const char *rsc_class, const char *op)
     CRM_CHECK(rsc_class || op, return FALSE);
 
     if (rsc_class
-        && strcmp(rsc_class, PCMK_RESOURCE_CLASS_OCF)
-        && strcmp(rsc_class, PCMK_RESOURCE_CLASS_STONITH)) {
-
+        && is_not_set(pcmk_get_ra_caps(rsc_class), pcmk_ra_cap_params)) {
         /* Meta-data is only needed for resource classes that use parameters */
         return FALSE;
     }
