@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -29,24 +18,6 @@
 #include <bzlib.h>
 #include <sys/types.h>
 
-char *
-crm_concat(const char *prefix, const char *suffix, char join)
-{
-    int len = 0;
-    char *new_str = NULL;
-
-    CRM_ASSERT(prefix != NULL);
-    CRM_ASSERT(suffix != NULL);
-    len = strlen(prefix) + strlen(suffix) + 2;
-
-    new_str = malloc(len);
-    if(new_str) {
-        sprintf(new_str, "%s%c%s", prefix, join, suffix);
-        new_str[len - 1] = 0;
-    }
-    return new_str;
-}
-
 char *
 crm_itoa_stack(int an_int, char *buffer, size_t len)
 {
@@ -57,26 +28,6 @@ crm_itoa_stack(int an_int, char *buffer, size_t len)
     return buffer;
 }
 
-char *
-crm_itoa(int an_int)
-{
-    int len = 32;
-    char *buffer = NULL;
-
-    buffer = malloc(len + 1);
-    if (buffer != NULL) {
-        snprintf(buffer, len, "%d", an_int);
-    }
-
-    return buffer;
-}
-
-void
-g_hash_destroy_str(gpointer data)
-{
-    free(data);
-}
-
 long long
 crm_int_helper(const char *text, char **end_text)
 {
@@ -150,34 +101,42 @@ crm_parse_ll(const char *text, const char *default_text)
  * \param[in] text          The string to parse
  * \param[in] default_text  Default string to parse if text is NULL
  *
- * \return Parsed value on success, INT_MIN or INT_MAX (and set errno to ERANGE)
- *         if parsed value is out of integer range, otherwise -1 (and set errno)
+ * \return Parsed value on success, -1 (and set errno) on error
  */
 int
 crm_parse_int(const char *text, const char *default_text)
 {
     long long result = crm_parse_ll(text, default_text);
 
-    if (result < INT_MIN) {
-        // If errno is ERANGE, crm_parse_ll() has already logged a message
-        if (errno != ERANGE) {
-            crm_err("Conversion of %s was clipped: %lld", text, result);
-            errno = ERANGE;
-        }
-        return INT_MIN;
-
-    } else if (result > INT_MAX) {
-        // If errno is ERANGE, crm_parse_ll() has already logged a message
-        if (errno != ERANGE) {
-            crm_err("Conversion of %s was clipped: %lld", text, result);
-            errno = ERANGE;
-        }
-        return INT_MAX;
+    if ((result < INT_MIN) || (result > INT_MAX)) {
+        errno = ERANGE;
+        return -1;
     }
-
     return (int) result;
 }
 
+/*!
+ * \internal
+ * \brief Parse a milliseconds value (without units) from a string
+ *
+ * \param[in] text  String to parse
+ *
+ * \return Milliseconds on success, 0 otherwise (and errno will be set)
+ */
+guint
+crm_parse_ms(const char *text)
+{
+    if (text) {
+        long long ms = crm_int_helper(text, NULL);
+
+        if ((ms < 0) || (ms > G_MAXUINT)) {
+            errno = ERANGE;
+        }
+        return errno? 0 : (guint) ms;
+    }
+    return 0;
+}
+
 gboolean
 safe_str_neq(const char *a, const char *b)
 {
@@ -390,6 +349,13 @@ g_str_hash_traditional(gconstpointer v)
     return h;
 }
 
+/* used with hash tables where case does not matter */
+gboolean
+crm_strcase_equal(gconstpointer a, gconstpointer b)
+{
+    return crm_str_eq((const char *) a, (const char *) b, FALSE);
+}
+
 guint
 crm_strcase_hash(gconstpointer v)
 {
@@ -447,8 +413,10 @@ crm_compress_string(const char *data, int length, int max, char **result, unsign
     int rc;
     char *compressed = NULL;
     char *uncompressed = strdup(data);
+#ifdef CLOCK_MONOTONIC
     struct timespec after_t;
     struct timespec before_t;
+#endif
 
     if(max == 0) {
         max = (length * 1.1) + 600; /* recommended size */
@@ -458,8 +426,8 @@ crm_compress_string(const char *data, int length, int max, char **result, unsign
     clock_gettime(CLOCK_MONOTONIC, &before_t);
 #endif
 
-    /* coverity[returned_null] Ignore */
-    compressed = malloc(max);
+    compressed = calloc(max, sizeof(char));
+    CRM_ASSERT(compressed);
 
     *result_len = max;
     rc = BZ2_bzBuffToBuffCompress(compressed, result_len, uncompressed, length, CRM_BZ2_BLOCKS, 0,
@@ -468,7 +436,8 @@ crm_compress_string(const char *data, int length, int max, char **result, unsign
     free(uncompressed);
 
     if (rc != BZ_OK) {
-        crm_err("Compression of %d bytes failed: %s (%d)", length, bz2_strerror(rc), rc);
+        crm_err("Compression of %d bytes failed: %s " CRM_XS " bzerror=%d",
+                length, bz2_strerror(rc), rc);
         free(compressed);
         return FALSE;
     }
@@ -512,3 +481,17 @@ crm_alpha_sort(gconstpointer a, gconstpointer b)
     }
     return strcasecmp(a, b);
 }
+
+char *
+crm_strdup_printf(char const *format, ...)
+{
+    va_list ap;
+    int len = 0;
+    char *string = NULL;
+
+    va_start(ap, format);
+    len = vasprintf (&string, format, ap);
+    CRM_ASSERT(len > 0);
+    va_end(ap);
+    return string;
+}
