@@ -1,7 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
- *
- * The version control history for this file may have further details.
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -63,41 +61,12 @@ char *crm_system_name = NULL;
 int node_score_red = 0;
 int node_score_green = 0;
 int node_score_yellow = 0;
-int node_score_infinity = INFINITY;
 
 static struct crm_option *crm_long_options = NULL;
 static const char *crm_app_description = NULL;
 static char *crm_short_options = NULL;
 static const char *crm_app_usage = NULL;
 
-int
-crm_exit(int rc)
-{
-    mainloop_cleanup();
-
-#if HAVE_LIBXML2
-    crm_trace("cleaning up libxml");
-    crm_xml_cleanup();
-#endif
-
-    crm_trace("exit %d", rc);
-    qb_log_fini();
-
-    free(crm_short_options);
-    free(crm_system_name);
-
-    exit(ABS(rc)); /* Always exit with a positive value so that it can be passed to crm_error
-                    *
-                    * Otherwise the system wraps it around and people
-                    * have to jump through hoops figuring out what the
-                    * error was
-                    */
-    return rc;     /* Can never happen, but allows return crm_exit(rc)
-                    * where "return rc" was used previously - which
-                    * keeps compilers happy.
-                    */
-}
-
 gboolean
 check_time(const char *value)
 {
@@ -134,9 +103,9 @@ check_number(const char *value)
     if (value == NULL) {
         return FALSE;
 
-    } else if (safe_str_eq(value, MINUS_INFINITY_S)) {
+    } else if (safe_str_eq(value, CRM_MINUS_INFINITY_S)) {
 
-    } else if (safe_str_eq(value, INFINITY_S)) {
+    } else if (safe_str_eq(value, CRM_INFINITY_S)) {
 
     } else {
         crm_int_helper(value, NULL);
@@ -151,7 +120,7 @@ check_number(const char *value)
 gboolean
 check_positive_number(const char* value)
 {
-    if (safe_str_eq(value, INFINITY_S) || (crm_int_helper(value, NULL))) {
+    if (safe_str_eq(value, CRM_INFINITY_S) || (crm_int_helper(value, NULL))) {
         return TRUE;
     }
     return FALSE;
@@ -217,6 +186,13 @@ check_utilization(const char *value)
     return TRUE;
 }
 
+void
+crm_args_fini()
+{
+    free(crm_short_options);
+    crm_short_options = NULL;
+}
+
 int
 char2score(const char *score)
 {
@@ -224,14 +200,14 @@ char2score(const char *score)
 
     if (score == NULL) {
 
-    } else if (safe_str_eq(score, MINUS_INFINITY_S)) {
-        score_f = -node_score_infinity;
+    } else if (safe_str_eq(score, CRM_MINUS_INFINITY_S)) {
+        score_f = -CRM_SCORE_INFINITY;
 
-    } else if (safe_str_eq(score, INFINITY_S)) {
-        score_f = node_score_infinity;
+    } else if (safe_str_eq(score, CRM_INFINITY_S)) {
+        score_f = CRM_SCORE_INFINITY;
 
-    } else if (safe_str_eq(score, "+" INFINITY_S)) {
-        score_f = node_score_infinity;
+    } else if (safe_str_eq(score, CRM_PLUS_INFINITY_S)) {
+        score_f = CRM_SCORE_INFINITY;
 
     } else if (safe_str_eq(score, "red")) {
         score_f = node_score_red;
@@ -244,11 +220,11 @@ char2score(const char *score)
 
     } else {
         score_f = crm_parse_int(score, NULL);
-        if (score_f > 0 && score_f > node_score_infinity) {
-            score_f = node_score_infinity;
+        if (score_f > 0 && score_f > CRM_SCORE_INFINITY) {
+            score_f = CRM_SCORE_INFINITY;
 
-        } else if (score_f < 0 && score_f < -node_score_infinity) {
-            score_f = -node_score_infinity;
+        } else if (score_f < 0 && score_f < -CRM_SCORE_INFINITY) {
+            score_f = -CRM_SCORE_INFINITY;
         }
     }
 
@@ -258,10 +234,10 @@ char2score(const char *score)
 char *
 score2char_stack(int score, char *buf, size_t len)
 {
-    if (score >= node_score_infinity) {
-        strncpy(buf, INFINITY_S, 9);
-    } else if (score <= -node_score_infinity) {
-        strncpy(buf, MINUS_INFINITY_S , 10);
+    if (score >= CRM_SCORE_INFINITY) {
+        strncpy(buf, CRM_INFINITY_S, 9);
+    } else if (score <= -CRM_SCORE_INFINITY) {
+        strncpy(buf, CRM_MINUS_INFINITY_S , 10);
     } else {
         return crm_itoa_stack(score, buf, len);
     }
@@ -272,11 +248,11 @@ score2char_stack(int score, char *buf, size_t len)
 char *
 score2char(int score)
 {
-    if (score >= node_score_infinity) {
-        return strdup(INFINITY_S);
+    if (score >= CRM_SCORE_INFINITY) {
+        return strdup(CRM_INFINITY_S);
 
-    } else if (score <= -node_score_infinity) {
-        return strdup("-" INFINITY_S);
+    } else if (score <= -CRM_SCORE_INFINITY) {
+        return strdup(CRM_MINUS_INFINITY_S);
     }
     return crm_itoa(score);
 }
@@ -560,13 +536,13 @@ gboolean do_stderr = FALSE;
 #  define	WHITESPACE	" \t\n\r\f"
 #endif
 
-unsigned long long
-crm_get_interval(const char *input)
+guint
+crm_parse_interval_spec(const char *input)
 {
-    unsigned long long msec = 0;
+    long long msec = 0;
 
     if (input == NULL) {
-        return msec;
+        return 0;
 
     } else if (input[0] != 'P') {
         long long tmp = crm_get_msec(input);
@@ -576,13 +552,13 @@ crm_get_interval(const char *input)
         }
 
     } else {
-        crm_time_t *interval = crm_time_parse_duration(input);
+        crm_time_t *period_s = crm_time_parse_duration(input);
 
-        msec = 1000 * crm_time_get_seconds(interval);
-        crm_time_free(interval);
+        msec = 1000 * crm_time_get_seconds(period_s);
+        crm_time_free(period_s);
     }
 
-    return msec;
+    return (msec <= 0)? 0 : ((msec >= G_MAXUINT)? G_MAXUINT : (guint) msec);
 }
 
 long long
@@ -705,185 +681,6 @@ crm_abort(const char *file, const char *function, int line,
     crm_perror(LOG_ERR, "Cannot wait on forked child %d", pid);
 }
 
-int
-crm_pid_active(long pid, const char *daemon)
-{
-    static int last_asked_pid = 0;  /* log spam prevention */
-#if SUPPORT_PROCFS
-    static int have_proc_pid = 0;
-#else
-    static int have_proc_pid = -1;
-#endif
-    int rc = 0;
-
-    if (have_proc_pid == 0) {
-        /* evaluation of /proc/PID/exe applicability via self-introspection */
-        char proc_path[PATH_MAX], exe_path[PATH_MAX];
-        snprintf(proc_path, sizeof(proc_path), "/proc/%lu/exe",
-                 (long unsigned int) getpid());
-        have_proc_pid = 1;
-        if (readlink(proc_path, exe_path, sizeof(exe_path) - 1) < 0) {
-            have_proc_pid = -1;
-        }
-    }
-
-    if (pid <= 0) {
-        return -1;
-
-    } else if ((rc = kill(pid, 0)) < 0 && errno == ESRCH) {
-        return 0;  /* no such PID detected */
-
-    } else if (rc < 0 && (daemon == NULL || have_proc_pid == -1)) {
-        if (last_asked_pid != pid) {
-            crm_info("Cannot examine PID %ld: %s", pid, strerror(errno));
-            last_asked_pid = pid;
-        }
-        return -2;  /* errno != ESRCH */
-
-    } else if (rc == 0 && (daemon == NULL || have_proc_pid == -1)) {
-        return 1;  /* kill as the only indicator, cannot double check */
-
-    } else if (daemon != NULL) {
-        /* make sure PID hasn't been reused by another process
-           XXX: might still be just a zombie, which could confuse decisions */
-        bool checked_through_kill = (rc == 0);
-        char proc_path[PATH_MAX], exe_path[PATH_MAX], myexe_path[PATH_MAX];
-        snprintf(proc_path, sizeof(proc_path), "/proc/%ld/exe", pid);
-
-        rc = readlink(proc_path, exe_path, sizeof(exe_path) - 1);
-        if ((rc < 0) && (errno == EACCES)) {
-            if (last_asked_pid != pid) {
-                crm_info("Could not read from %s: %s", proc_path,
-                         strerror(errno));
-                last_asked_pid = pid;
-            }
-            return checked_through_kill ? 1 : -2;
-        } else if (rc < 0) {
-            if (last_asked_pid != pid) {
-                crm_err("Could not read from %s: %s (%d)", proc_path,
-                        strerror(errno), errno);
-                last_asked_pid = pid;
-            }
-            return 0;  /* most likely errno == ENOENT */
-        }
-        exe_path[rc] = '\0';
-
-        if (daemon[0] != '/') {
-            rc = snprintf(myexe_path, sizeof(myexe_path), CRM_DAEMON_DIR"/%s",
-                          daemon);
-        } else {
-            rc = snprintf(myexe_path, sizeof(myexe_path), "%s", daemon);
-        }
-
-        if (rc > 0 && rc < sizeof(myexe_path) && !strcmp(exe_path, myexe_path)) {
-            return 1;
-        }
-    }
-
-    return 0;
-}
-
-#define	LOCKSTRLEN	11
-
-long
-crm_read_pidfile(const char *filename)
-{
-    int fd;
-    struct stat sbuf;
-    long pid = -ENOENT;
-    char buf[LOCKSTRLEN + 1];
-
-    if ((fd = open(filename, O_RDONLY)) < 0) {
-        goto bail;
-    }
-
-    if (fstat(fd, &sbuf) >= 0 && sbuf.st_size < LOCKSTRLEN) {
-        sleep(2);           /* if someone was about to create one,
-                             * give'm a sec to do so
-                             */
-    }
-
-    if (read(fd, buf, sizeof(buf)) < 1) {
-        goto bail;
-    }
-
-    if (sscanf(buf, "%lu", &pid) > 0) {
-        if (pid <= 0) {
-            pid = -ESRCH;
-        } else {
-            crm_trace("Got pid %lu from %s\n", pid, filename);
-        }
-    }
-
-  bail:
-    if (fd >= 0) {
-        close(fd);
-    }
-    return pid;
-}
-
-long
-crm_pidfile_inuse(const char *filename, long mypid, const char *daemon)
-{
-    long pid = crm_read_pidfile(filename);
-
-    if (pid < 2) {
-        /* Invalid pid */
-        pid = -ENOENT;
-        unlink(filename);
-
-    } else if (mypid && pid == mypid) {
-        /* In use by us */
-        pid = pcmk_ok;
-
-    } else if (crm_pid_active(pid, daemon) == FALSE) {
-        /* Contains a stale value */
-        unlink(filename);
-        pid = -ENOENT;
-
-    } else if (mypid && pid != mypid) {
-        /* locked by existing process - give up */
-        pid = -EEXIST;
-    }
-
-    return pid;
-}
-
-static int
-crm_lock_pidfile(const char *filename, const char *name)
-{
-    long mypid = 0;
-    int fd = 0, rc = 0;
-    char buf[LOCKSTRLEN + 1];
-
-    mypid = (unsigned long)getpid();
-
-    rc = crm_pidfile_inuse(filename, 0, name);
-    if (rc == -ENOENT) {
-        /* exists but the process is not active */
-
-    } else if (rc != pcmk_ok) {
-        /* locked by existing process - give up */
-        return rc;
-    }
-
-    if ((fd = open(filename, O_CREAT | O_WRONLY | O_EXCL, 0644)) < 0) {
-        /* Hmmh, why did we fail? Anyway, nothing we can do about it */
-        return -errno;
-    }
-
-    snprintf(buf, sizeof(buf), "%*lu\n", LOCKSTRLEN - 1, mypid);
-    rc = write(fd, buf, LOCKSTRLEN);
-    close(fd);
-
-    if (rc != LOCKSTRLEN) {
-        crm_perror(LOG_ERR, "Incomplete write to %s", filename);
-        return -errno;
-    }
-
-    return crm_pidfile_inuse(filename, mypid, name);
-}
-
 void
 crm_make_daemon(const char *name, gboolean daemonize, const char *pidfile)
 {
@@ -901,24 +698,24 @@ crm_make_daemon(const char *name, gboolean daemonize, const char *pidfile)
         pid = crm_read_pidfile(pidfile);
         crm_err("%s: already running [pid %ld in %s]", name, pid, pidfile);
         printf("%s: already running [pid %ld in %s]\n", name, pid, pidfile);
-        crm_exit(rc);
+        crm_exit(CRM_EX_ERROR);
     }
 
     pid = fork();
     if (pid < 0) {
         fprintf(stderr, "%s: could not start daemon\n", name);
         crm_perror(LOG_ERR, "fork");
-        crm_exit(EINVAL);
+        crm_exit(CRM_EX_OSERR);
 
     } else if (pid > 0) {
-        crm_exit(pcmk_ok);
+        crm_exit(CRM_EX_OK);
     }
 
     rc = crm_lock_pidfile(pidfile, name);
     if(rc < pcmk_ok) {
         crm_err("Could not lock '%s' for %s: %s (%d)", pidfile, name, pcmk_strerror(rc), rc);
         printf("Could not lock '%s' for %s: %s (%d)\n", pidfile, name, pcmk_strerror(rc), rc);
-        crm_exit(rc);
+        crm_exit(CRM_EX_ERROR);
     }
 
     umask(S_IWGRP | S_IWOTH | S_IROTH);
@@ -1091,10 +888,10 @@ crm_get_option_long(int argc, char **argv, int *index, const char **longname)
                 break;
             case ':':
                 crm_trace("Missing argument");
-                crm_help('?', 1);
+                crm_help('?', CRM_EX_USAGE);
                 break;
             case '?':
-                crm_help('?', *index ? 0 : 1);
+                crm_help('?', (*index? CRM_EX_OK : CRM_EX_USAGE));
                 break;
         }
         return flag;
@@ -1108,8 +905,8 @@ crm_get_option_long(int argc, char **argv, int *index, const char **longname)
     return -1;
 }
 
-int
-crm_help(char cmd, int exit_code)
+crm_exit_t
+crm_help(char cmd, crm_exit_t exit_code)
 {
     int i = 0;
     FILE *stream = (exit_code ? stderr : stdout);
@@ -1190,14 +987,14 @@ void cib_ipc_servers_init(qb_ipcs_service_t **ipcs_ro,
         struct qb_ipcs_service_handlers *ro_cb,
         struct qb_ipcs_service_handlers *rw_cb)
 {
-    *ipcs_ro = mainloop_add_ipc_server(cib_channel_ro, QB_IPC_NATIVE, ro_cb);
-    *ipcs_rw = mainloop_add_ipc_server(cib_channel_rw, QB_IPC_NATIVE, rw_cb);
-    *ipcs_shm = mainloop_add_ipc_server(cib_channel_shm, QB_IPC_SHM, rw_cb);
+    *ipcs_ro = mainloop_add_ipc_server(CIB_CHANNEL_RO, QB_IPC_NATIVE, ro_cb);
+    *ipcs_rw = mainloop_add_ipc_server(CIB_CHANNEL_RW, QB_IPC_NATIVE, rw_cb);
+    *ipcs_shm = mainloop_add_ipc_server(CIB_CHANNEL_SHM, QB_IPC_SHM, rw_cb);
 
     if (*ipcs_ro == NULL || *ipcs_rw == NULL || *ipcs_shm == NULL) {
-        crm_err("Failed to create cib servers: exiting and inhibiting respawn.");
-        crm_warn("Verify pacemaker and pacemaker_remote are not both enabled.");
-        crm_exit(DAEMON_RESPAWN_STOP);
+        crm_err("Failed to create the CIB manager: exiting and inhibiting respawn");
+        crm_warn("Verify pacemaker and pacemaker_remote are not both enabled");
+        crm_exit(CRM_EX_FATAL);
     }
 }
 
@@ -1222,22 +1019,21 @@ attrd_ipc_server_init(qb_ipcs_service_t **ipcs, struct qb_ipcs_service_handlers
     *ipcs = mainloop_add_ipc_server(T_ATTRD, QB_IPC_NATIVE, cb);
 
     if (*ipcs == NULL) {
-        crm_err("Failed to create attrd servers: exiting and inhibiting respawn.");
+        crm_err("Failed to create pacemaker-attrd server: exiting and inhibiting respawn");
         crm_warn("Verify pacemaker and pacemaker_remote are not both enabled.");
-        crm_exit(DAEMON_RESPAWN_STOP);
+        crm_exit(CRM_EX_FATAL);
     }
 }
 
 void
 stonith_ipc_server_init(qb_ipcs_service_t **ipcs, struct qb_ipcs_service_handlers *cb)
 {
-    *ipcs = mainloop_add_ipc_server_with_prio("stonith-ng", QB_IPC_NATIVE, cb,
-                                              QB_LOOP_HIGH);
+    *ipcs = mainloop_add_ipc_server("stonith-ng", QB_IPC_NATIVE, cb);
 
     if (*ipcs == NULL) {
-        crm_err("Failed to create stonith-ng servers: exiting and inhibiting respawn.");
+        crm_err("Failed to create fencer: exiting and inhibiting respawn.");
         crm_warn("Verify pacemaker and pacemaker_remote are not both enabled.");
-        crm_exit(DAEMON_RESPAWN_STOP);
+        crm_exit(CRM_EX_FATAL);
     }
 }
 
@@ -1254,7 +1050,7 @@ find_library_function(void **handle, const char *lib, const char *fn, gboolean f
     if (!(*handle)) {
         crm_err("%sCould not open %s: %s", fatal ? "Fatal: " : "", lib, dlerror());
         if (fatal) {
-            crm_exit(DAEMON_RESPAWN_STOP);
+            crm_exit(CRM_EX_FATAL);
         }
         return NULL;
     }
@@ -1264,7 +1060,7 @@ find_library_function(void **handle, const char *lib, const char *fn, gboolean f
         error = dlerror();
         crm_err("%sCould not find %s in %s: %s", fatal ? "Fatal: " : "", fn, lib, error);
         if (fatal) {
-            crm_exit(DAEMON_RESPAWN_STOP);
+            crm_exit(CRM_EX_FATAL);
         }
     }
 
@@ -1286,6 +1082,46 @@ crm_generate_uuid(void)
     return buffer;
 }
 
+/*!
+ * \brief Get name to be used as identifier for cluster messages
+ *
+ * \param[in] name  Actual system name to check
+ *
+ * \return Non-NULL cluster message identifier corresponding to name
+ *
+ * \note The Pacemaker daemons were renamed in version 2.0.0, but the old names
+ *       must continue to be used as the identifier for cluster messages, so
+ *       that mixed-version clusters are possible during a rolling upgrade.
+ */
+const char *
+pcmk_message_name(const char *name)
+{
+    if (name == NULL) {
+        return "unknown";
+
+    } else if (!strcmp(name, "pacemaker-attrd")) {
+        return "attrd";
+
+    } else if (!strcmp(name, "pacemaker-based")) {
+        return CRM_SYSTEM_CIB;
+
+    } else if (!strcmp(name, "pacemaker-controld")) {
+        return CRM_SYSTEM_CRMD;
+
+    } else if (!strcmp(name, "pacemaker-execd")) {
+        return CRM_SYSTEM_LRMD;
+
+    } else if (!strcmp(name, "pacemaker-fenced")) {
+        return "stonith-ng";
+
+    } else if (!strcmp(name, "pacemaker-schedulerd")) {
+        return CRM_SYSTEM_PENGINE;
+
+    } else {
+        return name;
+    }
+}
+
 /*!
  * \brief Check whether a string represents a cluster daemon name
  *
@@ -1296,15 +1132,16 @@ crm_generate_uuid(void)
 bool
 crm_is_daemon_name(const char *name)
 {
-    return (name &&
-            (!strcmp(name, CRM_SYSTEM_CRMD)
+    name = pcmk_message_name(name);
+    return (!strcmp(name, CRM_SYSTEM_CRMD)
             || !strcmp(name, CRM_SYSTEM_STONITHD)
-            || !strcmp(name, T_ATTRD)
+            || !strcmp(name, "stonith-ng")
+            || !strcmp(name, "attrd")
             || !strcmp(name, CRM_SYSTEM_CIB)
             || !strcmp(name, CRM_SYSTEM_MCP)
             || !strcmp(name, CRM_SYSTEM_DC)
             || !strcmp(name, CRM_SYSTEM_TENGINE)
-            || !strcmp(name, CRM_SYSTEM_LRMD)));
+            || !strcmp(name, CRM_SYSTEM_LRMD));
 }
 
 #include <md5.h>
@@ -1345,3 +1182,16 @@ crm_gnutls_global_init(void)
     gnutls_global_init();
 }
 #endif
+
+/*!
+ * \brief Get the local hostname
+ *
+ * \return Newly allocated string with name, or NULL (and set errno) on error
+ */
+char *
+pcmk_hostname()
+{
+    struct utsname hostinfo;
+
+    return (uname(&hostinfo) < 0)? NULL : strdup(hostinfo.nodename);
+}
