@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2013 Lars Marowsky-Bree <lmb@suse.com>
- *               2014 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2013 Lars Marowsky-Bree <lmb@suse.com>
+ *           2014-2018 Andrew Beekhof <andrew@beekhof.net>
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -17,6 +17,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <signal.h>
 
 #ifdef _POSIX_MEMLOCK
 #  include <sys/mman.h>
@@ -32,61 +33,22 @@ enum pcmk_panic_flags
     pcmk_panic_shutdown = 0x04,
 };
 
-#define SYSRQ "/proc/sys/kernel/sysrq"
-
-void
-sysrq_init(void)
-{
-    static bool need_init = true;
-    FILE* procf;
-    int c;
-
-    if(need_init) {
-        need_init = false;
-    } else {
-        return;
-    }
-
-    procf = fopen(SYSRQ, "r");
-    if (!procf) {
-        crm_perror(LOG_ERR, "Cannot open "SYSRQ" for read");
-        return;
-    }
-    if (fscanf(procf, "%d", &c) != 1) {
-        crm_perror(LOG_ERR, "Parsing "SYSRQ" failed");
-        c = 0;
-    }
-    fclose(procf);
-    if (c == 1)
-        return;
-
-    /* 8 for debugging dumps of processes, 128 for reboot/poweroff */
-    c |= 136;
-    procf = fopen(SYSRQ, "w");
-    if (!procf) {
-        crm_perror(LOG_ERR, "Cannot write to "SYSRQ);
-        return;
-    }
-    fprintf(procf, "%d", c);
-    fclose(procf);
-    return;
-}
-
 static void
 sysrq_trigger(char t)
 {
+#if SUPPORT_PROCFS
     FILE *procf;
 
-    sysrq_init();
-
+    // Root can always write here, regardless of kernel.sysrq value
     procf = fopen("/proc/sysrq-trigger", "a");
     if (!procf) {
-        crm_perror(LOG_ERR, "Opening sysrq-trigger failed");
+        crm_perror(LOG_WARNING, "Opening sysrq-trigger failed");
         return;
     }
     crm_info("sysrq-trigger: %c", t);
     fprintf(procf, "%c\n", t);
     fclose(procf);
+#endif // SUPPORT_PROCFS
     return;
 }
 
@@ -99,19 +61,21 @@ pcmk_panic_local(void)
     pid_t ppid = getppid();
 
     if(uid != 0 && ppid > 1) {
-        /* We're a non-root pacemaker daemon (cib, crmd, pengine,
-         * attrd, etc) with the original pacemakerd parent
+        /* We're a non-root pacemaker daemon (pacemaker-based,
+         * pacemaker-controld, pacemaker-schedulerd, pacemaker-attrd, etc.) with
+         * the original pacemakerd parent.
          *
-         * Of these, only crmd is likely to be initiating resets
+         * Of these, only the controller is likely to be initiating resets.
          */
         do_crm_log_always(LOG_EMERG, "Signaling parent %d to panic", ppid);
-        crm_exit(pcmk_err_panic);
+        crm_exit(CRM_EX_PANIC);
         return;
 
     } else if (uid != 0) {
+#if SUPPORT_PROCFS
         /*
-         * No permissions and no pacemakerd parent to escalate to
-         * Track down the new pacakerd process and send a signal instead
+         * No permissions, and no pacemakerd parent to escalate to.
+         * Track down the new pacemakerd process and send a signal instead.
          */
         union sigval signal_value;
 
@@ -122,8 +86,10 @@ pcmk_panic_local(void)
         if(ppid > 1 && sigqueue(ppid, SIGQUIT, signal_value) < 0) {
             crm_perror(LOG_EMERG, "Cannot signal pacemakerd(%d) to panic", ppid);
         }
+#endif // SUPPORT_PROCFS
+
         /* The best we can do now is die */
-        crm_exit(pcmk_err_panic);
+        crm_exit(CRM_EX_PANIC);
         return;
     }
 
@@ -142,10 +108,10 @@ pcmk_panic_local(void)
 
     if(ppid > 1) {
         /* child daemon */
-        exit(pcmk_err_panic);
+        exit(CRM_EX_PANIC);
     } else {
         /* pacemakerd or orphan child */
-        exit(DAEMON_RESPAWN_STOP);
+        exit(CRM_EX_FATAL);
     }
 }
 
@@ -166,10 +132,10 @@ pcmk_panic_sbd(void)
 
     if(ppid > 1) {
         /* child daemon */
-        exit(pcmk_err_panic);
+        exit(CRM_EX_PANIC);
     } else {
         /* pacemakerd or orphan child */
-        exit(DAEMON_RESPAWN_STOP);
+        exit(CRM_EX_FATAL);
     }
 }
 
@@ -190,7 +156,7 @@ pcmk_panic(const char *origin)
         do_crm_log_always(LOG_EMERG,
                           "Shutting down instead of panicking the node: origin=%s, sbd=%d, parent=%d",
                           origin, sbd_pid, getppid());
-        crm_exit(DAEMON_RESPAWN_STOP);
+        crm_exit(CRM_EX_FATAL);
         return;
     }
 
@@ -225,10 +191,12 @@ pcmk_locate_sbd(void)
     if(sbd_pid > 0) {
         crm_trace("SBD detected at pid=%d (file)", sbd_pid);
 
+#if SUPPORT_PROCFS
     } else {
         /* Fall back to /proc for systems that support it */
         sbd_pid = crm_procfs_pid_of("sbd");
         crm_trace("SBD detected at pid=%d (proc)", sbd_pid);
+#endif // SUPPORT_PROCFS
     }
 
     if(sbd_pid < 0) {
@@ -245,26 +213,42 @@ pcmk_locate_sbd(void)
 long
 crm_get_sbd_timeout(void)
 {
-    const char *env_value = getenv("SBD_WATCHDOG_TIMEOUT");
-    long sbd_timeout = crm_get_msec(env_value);
+    static long sbd_timeout = -2;
 
+    if (sbd_timeout == -2) {
+        sbd_timeout = crm_get_msec(getenv("SBD_WATCHDOG_TIMEOUT"));
+    }
     return sbd_timeout;
 }
 
+long
+crm_auto_watchdog_timeout()
+{
+    long sbd_timeout = crm_get_sbd_timeout();
+
+    return (sbd_timeout <= 0)? 0 : (2 * sbd_timeout);
+}
+
 gboolean
 check_sbd_timeout(const char *value)
 {
     long st_timeout = value? crm_get_msec(value) : 0;
 
-    if (st_timeout <= 0) {
+    if (st_timeout < 0) {
+        st_timeout = crm_auto_watchdog_timeout();
+        crm_debug("Using calculated value %ld for stonith-watchdog-timeout (%s)",
+                  st_timeout, value);
+    }
+
+    if (st_timeout == 0) {
         crm_debug("Watchdog may be enabled but stonith-watchdog-timeout is disabled (%s)",
                   value? value : "default");
 
     } else if (pcmk_locate_sbd() == 0) {
         do_crm_log_always(LOG_EMERG,
                           "Shutting down: stonith-watchdog-timeout configured (%s) but SBD not active",
-                          value);
-        crm_exit(DAEMON_RESPAWN_STOP);
+                          (value? value : "auto"));
+        crm_exit(CRM_EX_FATAL);
         return FALSE;
 
     } else {
@@ -274,7 +258,7 @@ check_sbd_timeout(const char *value)
             do_crm_log_always(LOG_EMERG,
                               "Shutting down: stonith-watchdog-timeout (%s) too short (must be >%ldms)",
                               value, sbd_timeout);
-            crm_exit(DAEMON_RESPAWN_STOP);
+            crm_exit(CRM_EX_FATAL);
             return FALSE;
         }
         crm_info("Watchdog configured with stonith-watchdog-timeout %s and SBD timeout %ldms",
