@@ -97,7 +97,8 @@ pe__create_clone_child(pe_resource_t *rsc, pe_working_set_t *data_set)
     }
 
     add_hash_param(child_rsc->meta, XML_RSC_ATTR_INCARNATION_MAX, inc_max);
-    pe_rsc_trace(rsc, "Added %s instance %s", rsc->id, child_rsc->id);
+
+    print_resource(LOG_TRACE, "Added ", child_rsc, FALSE);
 
   bail:
     free(inc_num);
@@ -106,24 +107,6 @@ pe__create_clone_child(pe_resource_t *rsc, pe_working_set_t *data_set)
     return child_rsc;
 }
 
-gboolean
-master_unpack(resource_t * rsc, pe_working_set_t * data_set)
-{
-    const char *master_max = g_hash_table_lookup(rsc->meta, XML_RSC_ATTR_MASTER_MAX);
-    const char *master_node_max = g_hash_table_lookup(rsc->meta, XML_RSC_ATTR_MASTER_NODEMAX);
-
-    g_hash_table_replace(rsc->meta, strdup("stateful"), strdup(XML_BOOLEAN_TRUE));
-    if (clone_unpack(rsc, data_set)) {
-        clone_variant_data_t *clone_data = NULL;
-
-        get_clone_variant_data(clone_data, rsc);
-        clone_data->master_max = crm_parse_int(master_max, "1");
-        clone_data->master_node_max = crm_parse_int(master_node_max, "1");
-        return TRUE;
-    }
-    return FALSE;
-}
-
 gboolean
 clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
 {
@@ -142,8 +125,33 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
     clone_data = calloc(1, sizeof(clone_variant_data_t));
     rsc->variant_opaque = clone_data;
 
-    clone_data->active_clones = 0;
-    clone_data->xml_obj_child = NULL;
+    if (is_set(rsc->flags, pe_rsc_promotable)) {
+        const char *promoted_max = NULL;
+        const char *promoted_node_max = NULL;
+
+        promoted_max = g_hash_table_lookup(rsc->meta,
+                                           XML_RSC_ATTR_PROMOTED_MAX);
+        if (promoted_max == NULL) {
+            // @COMPAT deprecated since 2.0.0
+            promoted_max = g_hash_table_lookup(rsc->meta,
+                                               XML_RSC_ATTR_MASTER_MAX);
+        }
+
+        promoted_node_max = g_hash_table_lookup(rsc->meta,
+                                                XML_RSC_ATTR_PROMOTED_NODEMAX);
+        if (promoted_node_max == NULL) {
+            // @COMPAT deprecated since 2.0.0
+            promoted_node_max = g_hash_table_lookup(rsc->meta,
+                                                    XML_RSC_ATTR_MASTER_NODEMAX);
+        }
+
+        clone_data->promoted_max = crm_parse_int(promoted_max, "1");
+        clone_data->promoted_node_max = crm_parse_int(promoted_node_max, "1");
+    }
+
+    // Implied by calloc()
+    /* clone_data->xml_obj_child = NULL; */
+
     clone_data->clone_node_max = crm_parse_int(max_clones_node, "1");
 
     if (max_clones) {
@@ -169,9 +177,11 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
     pe_rsc_trace(rsc, "\tClone node max: %d", clone_data->clone_node_max);
     pe_rsc_trace(rsc, "\tClone is unique: %s",
                  is_set(rsc->flags, pe_rsc_unique) ? "true" : "false");
+    pe_rsc_trace(rsc, "\tClone is promotable: %s",
+                 is_set(rsc->flags, pe_rsc_promotable) ? "true" : "false");
 
     // Clones may contain a single group or primitive
-    for (a_child = __xml_first_child_element(xml_obj); a_child != NULL;
+    for (a_child = __xml_first_child(xml_obj); a_child != NULL;
          a_child = __xml_next_element(a_child)) {
 
         if (crm_str_eq((const char *)a_child->name, XML_CIB_TAG_RESOURCE, TRUE)
@@ -196,7 +206,6 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
         add_hash_param(rsc->meta, XML_RSC_ATTR_STICKINESS, "1");
     }
 
-    clone_data->notify_confirm = is_set(rsc->flags, pe_rsc_notify);
     /* This ensures that the globally-unique value always exists for children to
      * inherit when being unpacked, as well as in resource agents' environment.
      */
@@ -299,14 +308,13 @@ configured_role(resource_t * rsc)
 static void
 clone_print_xml(resource_t * rsc, const char *pre_text, long options, void *print_data)
 {
-    int is_master_slave = rsc->variant == pe_master ? 1 : 0;
     char *child_text = crm_concat(pre_text, "   ", ' ');
     const char *target_role = configured_role_str(rsc);
     GListPtr gIter = rsc->children;
 
     status_print("%s<clone ", pre_text);
     status_print("id=\"%s\" ", rsc->id);
-    status_print("multi_state=\"%s\" ", is_master_slave ? "true" : "false");
+    status_print("multi_state=\"%s\" ", is_set(rsc->flags, pe_rsc_promotable)? "true" : "false");
     status_print("unique=\"%s\" ", is_set(rsc->flags, pe_rsc_unique) ? "true" : "false");
     status_print("managed=\"%s\" ", is_set(rsc->flags, pe_rsc_managed) ? "true" : "false");
     status_print("failed=\"%s\" ", is_set(rsc->flags, pe_rsc_failed) ? "true" : "false");
@@ -363,7 +371,6 @@ clone_print(resource_t * rsc, const char *pre_text, long options, void *print_da
     char *list_text = NULL;
     char *child_text = NULL;
     char *stopped_list = NULL;
-    const char *type = "Clone";
 
     GListPtr master_list = NULL;
     GListPtr started_list = NULL;
@@ -385,12 +392,9 @@ clone_print(resource_t * rsc, const char *pre_text, long options, void *print_da
 
     child_text = crm_concat(pre_text, "   ", ' ');
 
-    if (rsc->variant == pe_master) {
-        type = "Master/Slave";
-    }
-
-    status_print("%s%s Set: %s [%s]%s%s",
-                 pre_text ? pre_text : "", type, rsc->id, ID(clone_data->xml_obj_child),
+    status_print("%sClone Set: %s [%s]%s%s%s",
+                 pre_text ? pre_text : "", rsc->id, ID(clone_data->xml_obj_child),
+                 is_set(rsc->flags, pe_rsc_promotable) ? " (promotable)" : "",
                  is_set(rsc->flags, pe_rsc_unique) ? " (unique)" : "",
                  is_set(rsc->flags, pe_rsc_managed) ? "" : " (unmanaged)");
 
@@ -502,7 +506,7 @@ clone_print(resource_t * rsc, const char *pre_text, long options, void *print_da
 	active_instances++;
     }
 
-    if(rsc->variant == pe_master) {
+    if (is_set(rsc->flags, pe_rsc_promotable)) {
         enum rsc_role_e role = configured_role(rsc);
 
         if(role == RSC_ROLE_SLAVE) {
@@ -617,3 +621,25 @@ clone_resource_state(const resource_t * rsc, gboolean current)
     pe_rsc_trace(rsc, "%s role: %s", rsc->id, role2text(clone_role));
     return clone_role;
 }
+
+/*!
+ * \internal
+ * \brief Check whether a clone has an instance for every node
+ *
+ * \param[in] rsc       Clone to check
+ * \param[in] data_set  Cluster state
+ */
+bool
+pe__is_universal_clone(pe_resource_t *rsc,
+                       pe_working_set_t *data_set)
+{
+    if (pe_rsc_is_clone(rsc)) {
+        clone_variant_data_t *clone_data = NULL;
+
+        get_clone_variant_data(clone_data, rsc);
+        if (clone_data->clone_max == g_list_length(data_set->nodes)) {
+            return TRUE;
+        }
+    }
+    return FALSE;
+}
