@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -29,12 +18,40 @@
 #include <crm/pengine/internal.h>
 #include <unpack.h>
 
-#define MEMCHECK_STAGE_0 0
+/*!
+ * \brief Create a new working set
+ *
+ * \return New, initialized working set on success, else NULL (and set errno)
+ * \note Only pe_working_set_t objects created with this function (as opposed
+ *       to statically declared or directly allocated) should be used with the
+ *       functions in this library, to allow for future extensions to the
+ *       data type. The caller is responsible for freeing the memory with
+ *       pe_free_working_set() when the instance is no longer needed.
+ */
+pe_working_set_t *
+pe_new_working_set()
+{
+    pe_working_set_t *data_set = calloc(1, sizeof(pe_working_set_t));
 
-#define check_and_exit(stage) 	cleanup_calculations(data_set);		\
-	crm_mem_stats(NULL);						\
-	crm_err("Exiting: stage %d", stage);				\
-	crm_exit(pcmk_err_generic);
+    if (data_set != NULL) {
+        set_working_set_defaults(data_set);
+    }
+    return data_set;
+}
+
+/*!
+ * \brief Free a working set
+ *
+ * \param[in] data_set  Working set to free
+ */
+void
+pe_free_working_set(pe_working_set_t *data_set)
+{
+    if (data_set != NULL) {
+        pe_reset_working_set(data_set);
+        free(data_set);
+    }
+}
 
 /*
  * Unpack everything
@@ -58,7 +75,6 @@ cluster_status(pe_working_set_t * data_set)
     const char *value = crm_element_value(data_set->input, XML_ATTR_HAVE_QUORUM);
 
     crm_trace("Beginning unpack");
-    pe_dataset = data_set;
 
     /* reset remaining global variables */
     data_set->failed = create_xml_node(NULL, "failed-ops");
@@ -71,11 +87,9 @@ cluster_status(pe_working_set_t * data_set)
         data_set->now = crm_time_new(NULL);
     }
 
-    if (data_set->dc_uuid == NULL
-        && data_set->input != NULL
-        && crm_element_value(data_set->input, XML_ATTR_DC_UUID) != NULL) {
-        /* this should always be present */
-        data_set->dc_uuid = crm_element_value_copy(data_set->input, XML_ATTR_DC_UUID);
+    if (data_set->dc_uuid == NULL) {
+        data_set->dc_uuid = crm_element_value_copy(data_set->input,
+                                                   XML_ATTR_DC_UUID);
     }
 
     clear_bit(data_set->flags, pe_flag_have_quorum);
@@ -111,6 +125,17 @@ cluster_status(pe_working_set_t * data_set)
     return TRUE;
 }
 
+/*!
+ * \internal
+ * \brief Free a list of pe_resource_t
+ *
+ * \param[in] resources  List to free
+ *
+ * \note When a working set's resource list is freed, that includes the original
+ *       storage for the uname and id of any Pacemaker Remote nodes in the
+ *       working set's node list, so take care not to use those afterward.
+ * \todo Refactor pe_node_t to strdup() the node name.
+ */
 static void
 pe_free_resources(GListPtr resources)
 {
@@ -144,32 +169,36 @@ pe_free_actions(GListPtr actions)
 static void
 pe_free_nodes(GListPtr nodes)
 {
-    GListPtr iterator = nodes;
+    for (GList *iterator = nodes; iterator != NULL; iterator = iterator->next) {
+        pe_node_t *node = (pe_node_t *) iterator->data;
 
-    while (iterator != NULL) {
-        node_t *node = (node_t *) iterator->data;
-        struct node_shared_s *details = node->details;
-
-        iterator = iterator->next;
-
-        crm_trace("deleting node");
-        print_node("delete", node, FALSE);
-
-        if (details != NULL) {
-            crm_trace("%s is being deleted", details->uname);
-            if (details->attrs != NULL) {
-                g_hash_table_destroy(details->attrs);
-            }
-            if (details->utilization != NULL) {
-                g_hash_table_destroy(details->utilization);
-            }
-            if (details->digest_cache != NULL) {
-                g_hash_table_destroy(details->digest_cache);
-            }
-            g_list_free(details->running_rsc);
-            g_list_free(details->allocated_rsc);
-            free(details);
+        // Shouldn't be possible, but to be safe ...
+        if (node == NULL) {
+            continue;
         }
+        if (node->details == NULL) {
+            free(node);
+            continue;
+        }
+
+        /* This is called after pe_free_resources(), which means that we can't
+         * use node->details->uname for Pacemaker Remote nodes.
+         */
+        crm_trace("Freeing node %s", (is_remote_node(node)?
+                  "(Pacemaker Remote)" : node->details->uname));
+
+        if (node->details->attrs != NULL) {
+            g_hash_table_destroy(node->details->attrs);
+        }
+        if (node->details->utilization != NULL) {
+            g_hash_table_destroy(node->details->utilization);
+        }
+        if (node->details->digest_cache != NULL) {
+            g_hash_table_destroy(node->details->digest_cache);
+        }
+        g_list_free(node->details->running_rsc);
+        g_list_free(node->details->allocated_rsc);
+        free(node->details);
         free(node);
     }
     if (nodes != NULL) {
@@ -177,10 +206,55 @@ pe_free_nodes(GListPtr nodes)
     }
 }
 
+static void
+pe__free_ordering(GListPtr constraints)
+{
+    GListPtr iterator = constraints;
+
+    while (iterator != NULL) {
+        pe__ordering_t *order = iterator->data;
+
+        iterator = iterator->next;
+
+        free(order->lh_action_task);
+        free(order->rh_action_task);
+        free(order);
+    }
+    if (constraints != NULL) {
+        g_list_free(constraints);
+    }
+}
+
+static void
+pe__free_location(GListPtr constraints)
+{
+    GListPtr iterator = constraints;
+
+    while (iterator != NULL) {
+        pe__location_t *cons = iterator->data;
+
+        iterator = iterator->next;
+
+        g_list_free_full(cons->node_list_rh, free);
+        free(cons->id);
+        free(cons);
+    }
+    if (constraints != NULL) {
+        g_list_free(constraints);
+    }
+}
+
+/*!
+ * \brief Reset working set to default state without freeing it or constraints
+ *
+ * \param[in,out] data_set  Working set to reset
+ *
+ * \deprecated This function is deprecated as part of the API;
+ *             pe_reset_working_set() should be used instead.
+ */
 void
 cleanup_calculations(pe_working_set_t * data_set)
 {
-    pe_dataset = NULL;
     if (data_set == NULL) {
         return;
     }
@@ -217,6 +291,8 @@ cleanup_calculations(pe_working_set_t * data_set)
     crm_trace("deleting nodes");
     pe_free_nodes(data_set->nodes);
 
+    pe__free_param_checks(data_set);
+    g_list_free(data_set->stop_needed);
     free_xml(data_set->graph);
     crm_time_free(data_set->now);
     free_xml(data_set->input);
@@ -230,13 +306,46 @@ cleanup_calculations(pe_working_set_t * data_set)
         );
 }
 
+/*!
+ * \brief Reset a working set to default state without freeing it
+ *
+ * \param[in,out] data_set  Working set to reset
+ */
+void
+pe_reset_working_set(pe_working_set_t *data_set)
+{
+    if (data_set == NULL) {
+        return;
+    }
+
+    crm_trace("Deleting %d ordering constraints",
+              g_list_length(data_set->ordering_constraints));
+    pe__free_ordering(data_set->ordering_constraints);
+    data_set->ordering_constraints = NULL;
+
+    crm_trace("Deleting %d location constraints",
+              g_list_length(data_set->placement_constraints));
+    pe__free_location(data_set->placement_constraints);
+    data_set->placement_constraints = NULL;
+
+    crm_trace("Deleting %d colocation constraints",
+              g_list_length(data_set->colocation_constraints));
+    g_list_free_full(data_set->colocation_constraints, free);
+    data_set->colocation_constraints = NULL;
+
+    crm_trace("Deleting %d ticket constraints",
+              g_list_length(data_set->ticket_constraints));
+    g_list_free_full(data_set->ticket_constraints, free);
+    data_set->ticket_constraints = NULL;
+
+    cleanup_calculations(data_set);
+}
+
 void
 set_working_set_defaults(pe_working_set_t * data_set)
 {
-    pe_dataset = data_set;
     memset(data_set, 0, sizeof(pe_working_set_t));
 
-    data_set->dc_uuid = NULL;
     data_set->order_id = 1;
     data_set->action_id = 1;
     data_set->no_quorum_policy = no_quorum_freeze;
@@ -244,12 +353,17 @@ set_working_set_defaults(pe_working_set_t * data_set)
     data_set->flags = 0x0ULL;
     set_bit(data_set->flags, pe_flag_stop_rsc_orphans);
     set_bit(data_set->flags, pe_flag_symmetric_cluster);
-    set_bit(data_set->flags, pe_flag_is_managed_default);
     set_bit(data_set->flags, pe_flag_stop_action_orphans);
 }
 
 resource_t *
 pe_find_resource(GListPtr rsc_list, const char *id)
+{
+    return pe_find_resource_with_flags(rsc_list, id, pe_find_renamed);
+}
+
+resource_t *
+pe_find_resource_with_flags(GListPtr rsc_list, const char *id, enum pe_find flags)
 {
     GListPtr rIter = NULL;
 
@@ -257,7 +371,7 @@ pe_find_resource(GListPtr rsc_list, const char *id)
         resource_t *parent = rIter->data;
 
         resource_t *match =
-            parent->fns->find_rsc(parent, id, NULL, pe_find_renamed | pe_find_current);
+            parent->fns->find_rsc(parent, id, NULL, flags);
         if (match != NULL) {
             return match;
         }
