@@ -1,20 +1,10 @@
-/* 
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+/*
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #ifndef PENGINE_UNPACK__H
 #  define PENGINE_UNPACK__H
 
@@ -30,8 +20,6 @@ extern gboolean unpack_tags(xmlNode * xml_tags, pe_working_set_t * data_set);
 
 extern gboolean unpack_status(xmlNode * status, pe_working_set_t * data_set);
 
-extern gboolean unpack_remote_status(xmlNode * status, pe_working_set_t * data_set);
-
 extern gint sort_op_by_callid(gconstpointer a, gconstpointer b);
 
 extern gboolean unpack_lrm_resources(node_t * node, xmlNode * lrm_state,
@@ -97,17 +85,11 @@ extern gboolean determine_online_status(xmlNode * node_state, node_t * this_node
 
 enum pe_warn_once_e {
     pe_wo_blind         = 0x0001,
-    pe_wo_poweroff      = 0x0002,
-    pe_wo_arg_map       = 0x0004,
-    pe_wo_stonith_cmd   = 0x0008,
-    pe_wo_requires      = 0x0010,
-    pe_wo_isolation     = 0x0020,
-    pe_wo_default_stick = 0x0040,
-    pe_wo_default_isman = 0x0080,
-    pe_wo_default_timeo = 0x0100,
-    pe_wo_rsc_failstick = 0x0200,
-    pe_wo_default_rscfs = 0x0400,
-    pe_wo_legacy_notifs = 0x0800,
+    pe_wo_restart_type  = 0x0002,
+    pe_wo_role_after    = 0x0004,
+    pe_wo_poweroff      = 0x0008,
+    pe_wo_require_all   = 0x0010,
+    pe_wo_order_score   = 0x0020,
 };
 
 extern uint32_t pe_wo;
