@@ -1,20 +1,10 @@
-/* 
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+/*
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #ifndef PE_VARIANT__H
 #  define PE_VARIANT__H
 
@@ -24,16 +14,12 @@ typedef struct clone_variant_data_s {
     int clone_max;
     int clone_node_max;
 
-    int master_max;
-    int master_node_max;
+    int promoted_max;
+    int promoted_node_max;
 
     int total_clones;
-    int active_clones;
-    int max_nodes;
-
-    int masters_active;
-    int masters_allocated;
 
+    // @TODO make these a bitmask
     gboolean interleave;
     gboolean ordered;
     gboolean applied_master_prefs;
@@ -45,14 +31,11 @@ typedef struct clone_variant_data_s {
     notify_data_t *promote_notify;
 
     xmlNode *xml_obj_child;
-
-    gboolean notify_confirm;
-
 } clone_variant_data_t;
 
 #    define get_clone_variant_data(data, rsc)				\
 	CRM_ASSERT(rsc != NULL);					\
-	CRM_ASSERT(rsc->variant == pe_clone || rsc->variant == pe_master); \
+	CRM_ASSERT(rsc->variant == pe_clone); \
 	data = (clone_variant_data_t *)rsc->variant_opaque;
 
 #  elif VARIANT_CONTAINER
@@ -88,15 +71,17 @@ typedef struct
 enum container_type {
         PE_CONTAINER_TYPE_UNKNOWN,
         PE_CONTAINER_TYPE_DOCKER,
-        PE_CONTAINER_TYPE_RKT
+        PE_CONTAINER_TYPE_RKT,
+        PE_CONTAINER_TYPE_PODMAN
 };
 
 #define PE_CONTAINER_TYPE_UNKNOWN_S "unknown"
 #define PE_CONTAINER_TYPE_DOCKER_S  "Docker"
 #define PE_CONTAINER_TYPE_RKT_S     "rkt"
+#define PE_CONTAINER_TYPE_PODMAN_S  "podman"
 
 typedef struct container_variant_data_s {
-        int masters;
+        int promoted_max;
         int replicas;
         int replicas_per_host;
         char *prefix;
@@ -107,6 +92,7 @@ typedef struct container_variant_data_s {
         char *control_port;
         char *docker_network;
         char *ip_range_start;
+        gboolean add_host;
         char *docker_host_options;
         char *docker_run_options;
         char *docker_run_command;
