@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2016 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2010-2019 Andrew Beekhof <andrew@beekhof.net>
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -17,6 +17,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <grp.h>
 #include <string.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -66,8 +67,8 @@ svc_read_output(int fd, svc_action_t * op, bool is_stderr)
     do {
         rc = read(fd, buf, buf_read_len);
         if (rc > 0) {
-            crm_trace("Got %d chars: %.80s", rc, buf);
             buf[rc] = 0;
+            crm_trace("Got %d chars: %.80s", rc, buf);
             data = realloc_safe(data, len + rc + 1);
             len += sprintf(data + len, "%s", buf);
 
@@ -219,13 +220,13 @@ operation_finalize(svc_action_t * op)
 {
     int recurring = 0;
 
-    if (op->interval) {
+    if (op->interval_ms) {
         if (op->cancel) {
             op->status = PCMK_LRM_OP_CANCELLED;
             cancel_recurring_action(op);
         } else {
             recurring = 1;
-            op->opaque->repeat_timer = g_timeout_add(op->interval,
+            op->opaque->repeat_timer = g_timeout_add(op->interval_ms,
                                                      recurring_action_timer, (void *)op);
         }
     }
@@ -290,9 +291,16 @@ operation_finished(mainloop_child_t * p, pid_t pid, int core, int signo, int exi
             op->status = PCMK_LRM_OP_TIMEOUT;
             op->rc = PCMK_OCF_TIMEOUT;
 
+        } else if (op->cancel) {
+            /* If an in-flight recurring operation was killed because it was
+             * cancelled, don't treat that as a failure.
+             */
+            crm_info("%s - terminated with signal %d", prefix, signo);
+            op->status = PCMK_LRM_OP_CANCELLED;
+            op->rc = PCMK_OCF_OK;
+
         } else {
-            do_crm_log_unlikely((op->cancel) ? LOG_INFO : LOG_WARNING,
-                                "%s - terminated with signal %d", prefix, signo);
+            crm_warn("%s - terminated with signal %d", prefix, signo);
             op->status = PCMK_LRM_OP_ERROR;
             op->rc = PCMK_OCF_SIGNAL;
         }
@@ -403,7 +411,7 @@ action_launch_child(svc_action_t *op)
      */
     setpgid(0, 0);
 
-    /* close all descriptors except stdin/out/err and channels to logd */
+    // Close all file descriptors except stdin/stdout/stderr
     for (lpc = getdtablesize() - 1; lpc > STDERR_FILENO; lpc--) {
         close(lpc);
     }
@@ -427,15 +435,25 @@ action_launch_child(svc_action_t *op)
 
     /* Become the desired user */
     if (op->opaque->uid && (geteuid() == 0)) {
+
+        // If requested, set effective group
         if (op->opaque->gid && (setgid(op->opaque->gid) < 0)) {
-            crm_perror(LOG_ERR, "setting group to %d", op->opaque->gid);
+            crm_perror(LOG_ERR, "Could not set child group to %d", op->opaque->gid);
             _exit(PCMK_OCF_NOT_CONFIGURED);
         }
+
+        // Erase supplementary group list
+        // (We could do initgroups() if we kept a copy of the username)
+        if (setgroups(0, NULL) < 0) {
+            crm_perror(LOG_ERR, "Could not set child groups");
+            _exit(PCMK_OCF_NOT_CONFIGURED);
+        }
+
+        // Set effective user
         if (setuid(op->opaque->uid) < 0) {
             crm_perror(LOG_ERR, "setting user to %d", op->opaque->uid);
             _exit(PCMK_OCF_NOT_CONFIGURED);
         }
-        /* We could do initgroups() here if we kept a copy of the username */
     }
 
     /* execute the RA */
@@ -860,12 +878,6 @@ services_os_get_directory_list(const char *root, gboolean files, gboolean execut
     return list;
 }
 
-GList *
-resources_os_list_lsb_agents(void)
-{
-    return get_directory_list(LSB_ROOT_DIR, TRUE, TRUE);
-}
-
 GList *
 resources_os_list_ocf_providers(void)
 {
