@@ -1,79 +1,37 @@
-
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
-
-#include <sys/param.h>
-
-#include <crm/crm.h>
-
 #include <stdio.h>
-#include <sys/types.h>
-#include <unistd.h>
-
-#include <stdlib.h>
-#include <errno.h>
-#include <fcntl.h>
-
+#include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 #include <crm/common/ipc.h>
 #include <crm/cib/internal.h>
 
-int exit_code = pcmk_ok;
-int message_timer_id = -1;
-int message_timeout_ms = 30;
+static int message_timeout_ms = 30;
+static int command_options = 0;
+static int request_id = 0;
+static int bump_log_num = 0;
 
-GMainLoop *mainloop = NULL;
+static const char *host = NULL;
+static const char *cib_user = NULL;
+static const char *cib_action = NULL;
+static const char *obj_type = NULL;
+
+static cib_t *the_cib = NULL;
+static GMainLoop *mainloop = NULL;
+static gboolean force_flag = FALSE;
+static crm_exit_t exit_code = CRM_EX_OK;
 
-const char *host = NULL;
-void usage(const char *cmd, int exit_status);
 int do_init(void);
-int do_work(xmlNode * input, int command_options, xmlNode ** output);
-
-gboolean admin_message_timeout(gpointer data);
-void cib_connection_destroy(gpointer user_data);
-void cibadmin_op_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *user_data);
-
-int command_options = 0;
-const char *cib_user = NULL;
-const char *cib_action = NULL;
-
-typedef struct str_list_s {
-    int num_items;
-    char *value;
-    struct str_list_s *next;
-} str_list_t;
-
-const char *obj_type = NULL;
-char *status = NULL;
-char *migrate_from = NULL;
-char *migrate_res = NULL;
-char *subtype = NULL;
-char *reset = NULL;
-
-int request_id = 0;
-int operation_status = 0;
-cib_t *the_cib = NULL;
-gboolean force_flag = FALSE;
-gboolean quiet = FALSE;
-int bump_log_num = 0;
+int do_work(xmlNode *input, int command_options, xmlNode **output);
+void cibadmin_op_callback(xmlNode *msg, int call_id, int rc, xmlNode *output,
+                          void *user_data);
 
 /* *INDENT-OFF* */
 static struct crm_option long_options[] = {
@@ -118,7 +76,7 @@ static struct crm_option long_options[] = {
 
     {"xpath",       1, 0, 'A', "A valid XPath to use instead of --scope,-o"},
     {"node-path",   0, 0, 'e',  "When performing XPath queries, return the address of any matches found."},
-    {"-spacer-",    0, 0, '-', " Eg: /cib/configuration/resources/master[@id='ms_RH1_SCS']/primitive[@id='prm_RH1_SCS']", pcmk_option_paragraph},
+    {"-spacer-",    0, 0, '-', " Eg: /cib/configuration/resources/clone[@id='ms_RH1_SCS']/primitive[@id='prm_RH1_SCS']", pcmk_option_paragraph},
     {"node",	    1, 0, 'N', "(Advanced) Send command to the specified host\n"},
     {"-space-",	    0, 0, '!', NULL, 1},
 
@@ -197,10 +155,22 @@ print_xml_output(xmlNode * xml)
     }
 }
 
+// Upgrade requested but already at latest schema
+static void
+report_schema_unchanged()
+{
+    const char *err = pcmk_strerror(pcmk_err_schema_unchanged);
+
+    crm_info("Upgrade unnecessary: %s\n", err);
+    printf("Upgrade unnecessary: %s\n", err);
+    exit_code = CRM_EX_OK;
+}
+
 int
 main(int argc, char **argv)
 {
     int argerr = 0;
+    int rc = pcmk_ok;
     int flag;
     const char *source = NULL;
     const char *admin_input_xml = NULL;
@@ -221,7 +191,7 @@ main(int argc, char **argv)
                     "\n\nWhere necessary, XML data will be obtained using the -X, -x, or -p options.\n");
 
     if (argc < 2) {
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
     }
 
     while (1) {
@@ -253,7 +223,6 @@ main(int argc, char **argv)
                 break;
             case 'Q':
                 cib_action = CIB_OP_QUERY;
-                quiet = TRUE;
                 break;
             case 'P':
                 cib_action = CIB_OP_APPLY_DIFF;
@@ -296,7 +265,7 @@ main(int argc, char **argv)
             case '?':
             case '$':
             case '!':
-                crm_help(flag, EX_OK);
+                crm_help(flag, CRM_EX_OK);
                 break;
             case 'o':
                 crm_trace("Option %c => %s", flag, optarg);
@@ -348,7 +317,7 @@ main(int argc, char **argv)
                 }
                 admin_input_xml = dump_xml_formatted(output);
                 fprintf(stdout, "%s\n", crm_str(admin_input_xml));
-                goto bail;
+                crm_exit(CRM_EX_OK);
                 break;
             default:
                 printf("Argument code 0%o (%c)" " is not (?yet?) supported\n", flag, flag);
@@ -357,10 +326,6 @@ main(int argc, char **argv)
         }
     }
 
-    if (bump_log_num > 0) {
-        quiet = FALSE;
-    }
-
     while (bump_log_num > 0) {
         crm_bump_log_level(argc, argv);
         bump_log_num--;
@@ -371,7 +336,7 @@ main(int argc, char **argv)
         while (optind < argc)
             printf("%s ", argv[optind++]);
         printf("\n");
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
     }
 
     if (optind > argc || cib_action == NULL) {
@@ -379,7 +344,7 @@ main(int argc, char **argv)
     }
 
     if (argerr) {
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
     }
 
     if (dangerous_cmd && force_flag == FALSE) {
@@ -387,8 +352,7 @@ main(int argc, char **argv)
                 "  To prevent accidental destruction of the cluster,"
                 " the --force flag is required in order to proceed.\n");
         fflush(stderr);
-        exit_code = -EINVAL;
-        goto bail;
+        crm_exit(CRM_EX_UNSAFE);
     }
 
     if (admin_input_file != NULL) {
@@ -409,8 +373,7 @@ main(int argc, char **argv)
 
     } else if (source) {
         fprintf(stderr, "Couldn't parse input from %s.\n", source);
-        exit_code = -EINVAL;
-        goto bail;
+        crm_exit(CRM_EX_CONFIG);
     }
 
     if (safe_str_eq(cib_action, "md5-sum")) {
@@ -418,15 +381,15 @@ main(int argc, char **argv)
 
         if (input == NULL) {
             fprintf(stderr, "Please supply XML to process with -X, -x or -p\n");
-            exit_code = -EINVAL;
-            goto bail;
+            crm_exit(CRM_EX_USAGE);
         }
 
         digest = calculate_on_disk_digest(input);
         fprintf(stderr, "Digest: ");
         fprintf(stdout, "%s\n", crm_str(digest));
         free(digest);
-        goto bail;
+        free_xml(input);
+        crm_exit(CRM_EX_OK);
 
     } else if (safe_str_eq(cib_action, "md5-sum-versioned")) {
         char *digest = NULL;
@@ -434,8 +397,7 @@ main(int argc, char **argv)
 
         if (input == NULL) {
             fprintf(stderr, "Please supply XML to process with -X, -x or -p\n");
-            exit_code = -EINVAL;
-            goto bail;
+            crm_exit(CRM_EX_USAGE);
         }
 
         version = crm_element_value(input, XML_ATTR_CRM_VERSION);
@@ -443,39 +405,44 @@ main(int argc, char **argv)
         fprintf(stderr, "Versioned (%s) digest: ", version);
         fprintf(stdout, "%s\n", crm_str(digest));
         free(digest);
-        goto bail;
+        free_xml(input);
+        crm_exit(CRM_EX_OK);
     }
 
-    exit_code = do_init();
-    if (exit_code != pcmk_ok) {
+    rc = do_init();
+    if (rc != pcmk_ok) {
         crm_err("Init failed, could not perform requested operations");
         fprintf(stderr, "Init failed, could not perform requested operations\n");
-        return crm_exit(-exit_code);
+        free_xml(input);
+        crm_exit(crm_errno2exit(rc));
     }
 
-    exit_code = do_work(input, command_options, &output);
-    if (exit_code > 0) {
+    rc = do_work(input, command_options, &output);
+    if (rc > 0) {
         /* wait for the reply by creating a mainloop and running it until
          * the callbacks are invoked...
          */
-        request_id = exit_code;
+        request_id = rc;
 
         the_cib->cmds->register_callback(the_cib, request_id, message_timeout_ms, FALSE, NULL,
                                          "cibadmin_op_callback", cibadmin_op_callback);
 
-        mainloop = g_main_new(FALSE);
+        mainloop = g_main_loop_new(NULL, FALSE);
 
         crm_trace("%s waiting for reply from the local CIB", crm_system_name);
 
         crm_info("Starting mainloop");
-        g_main_run(mainloop);
+        g_main_loop_run(mainloop);
 
-    } else if (exit_code < 0) {
-        crm_err("Call failed: %s", pcmk_strerror(exit_code));
-        fprintf(stderr, "Call failed: %s\n", pcmk_strerror(exit_code));
-        operation_status = exit_code;
+    } else if ((rc == -pcmk_err_schema_unchanged)
+               && crm_str_eq(cib_action, CIB_OP_UPGRADE, TRUE)) {
+        report_schema_unchanged();
 
-        if (exit_code == -pcmk_err_schema_validation) {
+    } else if (rc < 0) {
+        crm_err("Call failed: %s", pcmk_strerror(rc));
+        fprintf(stderr, "Call failed: %s\n", pcmk_strerror(rc));
+
+        if (rc == -pcmk_err_schema_validation) {
             if (crm_str_eq(cib_action, CIB_OP_UPGRADE, TRUE)) {
                 xmlNode *obj = NULL;
                 int version = 0, rc = 0;
@@ -489,6 +456,7 @@ main(int argc, char **argv)
                 validate_xml_verbose(output);
             }
         }
+        exit_code = crm_errno2exit(rc);
     }
 
     if (output != NULL) {
@@ -499,13 +467,12 @@ main(int argc, char **argv)
     crm_trace("%s exiting normally", crm_system_name);
 
     free_xml(input);
-    flag = the_cib->cmds->signoff(the_cib);
+    rc = the_cib->cmds->signoff(the_cib);
+    if (exit_code == CRM_EX_OK) {
+        exit_code = crm_errno2exit(rc);
+    }
     cib_delete(the_cib);
 
-    if(exit_code == pcmk_ok) {
-        exit_code = flag;
-    }
-  bail:
     return crm_exit(exit_code);
 }
 
@@ -541,33 +508,29 @@ do_init(void)
     the_cib = cib_new();
     rc = the_cib->cmds->signon(the_cib, crm_system_name, cib_command);
     if (rc != pcmk_ok) {
-        crm_err("Signon to CIB failed: %s", pcmk_strerror(rc));
-        fprintf(stderr, "Signon to CIB failed: %s\n", pcmk_strerror(rc));
+        crm_err("Connection to the CIB manager failed: %s", pcmk_strerror(rc));
+        fprintf(stderr, "Connection to the CIB manager failed: %s\n",
+                pcmk_strerror(rc));
     }
 
     return rc;
 }
 
-void
-cib_connection_destroy(gpointer user_data)
-{
-    crm_err("Connection to the CIB terminated... exiting");
-    g_main_quit(mainloop);
-    return;
-}
-
 void
 cibadmin_op_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *user_data)
 {
-    exit_code = rc;
+    exit_code = crm_errno2exit(rc);
 
-    if (rc != 0) {
+    if (rc == -pcmk_err_schema_unchanged) {
+        report_schema_unchanged();
+
+    } else if (rc != pcmk_ok) {
         crm_warn("Call %s failed (%d): %s", cib_action, rc, pcmk_strerror(rc));
         fprintf(stderr, "Call %s failed (%d): %s\n", cib_action, rc, pcmk_strerror(rc));
         print_xml_output(output);
 
     } else if (safe_str_eq(cib_action, CIB_OP_QUERY) && output == NULL) {
-        crm_err("Output expected in query response");
+        crm_err("Query returned no output");
         crm_log_xml_err(msg, "no output");
 
     } else if (output == NULL) {
@@ -579,10 +542,10 @@ cibadmin_op_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void
     }
 
     if (call_id == request_id) {
-        g_main_quit(mainloop);
+        g_main_loop_quit(mainloop);
 
     } else {
-        crm_info("Message was not the response we were looking for (%d vs. %d", call_id,
-                 request_id);
+        crm_info("Message was not the response we were looking for (%d vs. %d)",
+                 call_id, request_id);
     }
 }
