@@ -1,20 +1,8 @@
-
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -88,7 +76,6 @@ static struct crm_option long_options[] = {
     /* legacy */
     {"quiet",       0, 0, 'Q', NULL, 1},
     {"node-uname",  1, 0, 'U', NULL, 1},
-    {"node-uuid",   1, 0, 'u', NULL, 1},
     {"get-value",   0, 0, 'G', NULL, 1},
     {"delete-attr", 0, 0, 'D', NULL, 1},
     {"attr-value",  1, 0, 'v', NULL, 1},
@@ -126,15 +113,13 @@ main(int argc, char **argv)
     int option_index = 0;
     int is_remote_node = 0;
 
-    bool try_attrd = true;
-
     crm_log_cli_init("crm_attribute");
     crm_set_options(NULL, "<command> -n <attribute> [options]", long_options,
                     "Manage node's attributes and cluster options."
                     "\n\nAllows node attributes and cluster options to be queried, modified and deleted.\n");
 
     if (argc < 2) {
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
     }
 
     while (1) {
@@ -148,7 +133,7 @@ main(int argc, char **argv)
                 break;
             case '$':
             case '?':
-                crm_help(flag, EX_OK);
+                crm_help(flag, CRM_EX_OK);
                 break;
             case 'G':
                 command = flag;
@@ -168,9 +153,6 @@ main(int argc, char **argv)
             case 'N':
                 dest_uname = strdup(optarg);
                 break;
-            case 'u':
-                dest_node = strdup(optarg);
-                break;
             case 's':
                 set_name = strdup(optarg);
                 break;
@@ -220,15 +202,16 @@ main(int argc, char **argv)
     }
 
     if (argerr) {
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
     }
 
     the_cib = cib_new();
     rc = the_cib->cmds->signon(the_cib, crm_system_name, cib_command);
 
     if (rc != pcmk_ok) {
-        fprintf(stderr, "Error signing on to the CIB service: %s\n", pcmk_strerror(rc));
-        return crm_exit(rc);
+        fprintf(stderr, "Error connecting to the CIB manager: %s\n",
+                pcmk_strerror(rc));
+        return crm_exit(crm_errno2exit(rc));
     }
 
     if (type == NULL && dest_uname != NULL) {
@@ -262,13 +245,13 @@ main(int argc, char **argv)
             fprintf(stderr, "Could not map name=%s to a UUID\n", dest_uname);
             the_cib->cmds->signoff(the_cib);
             cib_delete(the_cib);
-            return crm_exit(rc);
+            return crm_exit(crm_errno2exit(rc));
         }
     }
 
     if ((command == 'D') && (attr_name == NULL) && (attr_pattern == NULL)) {
         fprintf(stderr, "Error: must specify attribute name or pattern to delete\n");
-        return crm_exit(1);
+        return crm_exit(CRM_EX_USAGE);
     }
 
     if (attr_pattern) {
@@ -276,26 +259,20 @@ main(int argc, char **argv)
             || safe_str_neq(type, XML_CIB_TAG_STATUS)) {
 
             fprintf(stderr, "Error: pattern can only be used with till-reboot update or delete\n");
-            return crm_exit(1);
+            return crm_exit(CRM_EX_USAGE);
         }
         command = 'u';
         free(attr_name);
         attr_name = attr_pattern;
     }
 
-    // Only go through attribute manager for transient attributes
-    try_attrd = safe_str_eq(type, XML_CIB_TAG_STATUS);
-
-    // Don't try to contact attribute manager if we're using a file as CIB
-    if (getenv("CIB_file") || getenv("CIB_shadow")) {
-        try_attrd = FALSE;
-    }
-
-    if (((command == 'v') || (command == 'D') || (command == 'u')) && try_attrd
+    if (((command == 'v') || (command == 'D') || (command == 'u'))
+        && safe_str_eq(type, XML_CIB_TAG_STATUS)
         && pcmk_ok == attrd_update_delegate(NULL, command, dest_uname, attr_name,
                                             attr_value, type, set_name, NULL, NULL,
                                             is_remote_node?attrd_opt_remote:attrd_opt_none)) {
-        crm_info("Update %s=%s sent via attrd", attr_name, command == 'D' ? "<none>" : attr_value);
+        crm_info("Update %s=%s sent via pacemaker-attrd",
+                 attr_name, ((command == 'D')? "<none>" : attr_value));
 
     } else if (command == 'D') {
         rc = delete_attr_delegate(the_cib, cib_opts, type, dest_node, set_type, set_name,
@@ -358,5 +335,5 @@ main(int argc, char **argv)
 
     the_cib->cmds->signoff(the_cib);
     cib_delete(the_cib);
-    return crm_exit(rc);
+    return crm_exit(crm_errno2exit(rc));
 }
