@@ -1,20 +1,8 @@
-
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_resource.h>
@@ -37,7 +25,7 @@ bool BE_QUIET = FALSE;
 bool scope_master = FALSE;
 int cib_options = cib_sync_call;
 
-GMainLoop *mainloop = NULL;
+static GMainLoop *mainloop = NULL;
 
 #define MESSAGE_TIMEOUT_S 60
 
@@ -47,14 +35,14 @@ resource_ipc_timeout(gpointer data)
     fprintf(stderr, "Aborting because no messages received in %d seconds\n",
             MESSAGE_TIMEOUT_S);
     crm_err("No messages received in %d seconds", MESSAGE_TIMEOUT_S);
-    return crm_exit(-1);
+    return crm_exit(CRM_EX_TIMEOUT);
 }
 
 static void
 resource_ipc_connection_destroy(gpointer user_data)
 {
-    crm_info("Connection to CRMd was terminated");
-    crm_exit(1);
+    crm_info("Connection to controller was terminated");
+    crm_exit(CRM_EX_DISCONNECT);
 }
 
 static void
@@ -64,14 +52,14 @@ start_mainloop(void)
         return;
     }
 
-    mainloop = g_main_new(FALSE);
-    fprintf(stderr, "Waiting for %d repl%s from the CRMd",
+    mainloop = g_main_loop_new(NULL, FALSE);
+    fprintf(stderr, "Waiting for %d repl%s from the controller",
             crmd_replies_needed, (crmd_replies_needed == 1)? "y" : "ies");
-    crm_debug("Waiting for %d repl%s from the CRMd",
+    crm_debug("Waiting for %d repl%s from the controller",
               crmd_replies_needed, (crmd_replies_needed == 1)? "y" : "ies");
 
     g_timeout_add(MESSAGE_TIMEOUT_S * 1000, resource_ipc_timeout, NULL);
-    g_main_run(mainloop);
+    g_main_loop_run(mainloop);
 }
 
 static int
@@ -88,7 +76,7 @@ resource_ipc_callback(const char *buffer, ssize_t length, gpointer userdata)
 
         fprintf(stderr, " OK\n");
         crm_debug("Got all the replies we expected");
-        return crm_exit(pcmk_ok);
+        return crm_exit(CRM_EX_OK);
     }
 
     free_xml(msg);
@@ -147,11 +135,6 @@ static struct crm_option long_options[] = {
         "list-all-operations", no_argument, NULL, 'o',
         "List all resource operations, optionally filtered by --resource and/or --node"
     },
-    {
-        "pending", no_argument, NULL, 'j',
-        "\t\tDisplay pending state if 'record-pending' is enabled",
-        pcmk_option_hidden
-    },
     {
         "list-standards", no_argument, NULL, 0,
         "\tList supported standards"
@@ -214,22 +197,16 @@ static struct crm_option long_options[] = {
     },
     {
         "cleanup", no_argument, NULL, 'C',
-#if 0
-        // new behavior disabled until 2.0.0
         "\t\tIf resource has any past failures, clear its history and fail count.\n"
         "\t\t\t\tOptionally filtered by --resource, --node, --operation, and --interval (otherwise all).\n"
         "\t\t\t\t--operation and --interval apply to fail counts, but entire history is always cleared,\n"
-        "\t\t\t\tto allow current state to be rechecked. If the named resource is part of a group, or\n"
-        "\t\t\t\tone numbered instance of a clone or bundled resource, the clean-up applies to the\n"
-        "\t\t\t\twhole collective resource unless --force is given."
+        "\t\t\t\tto allow current state to be rechecked.\n"
     },
     {
         "refresh", no_argument, NULL, 'R',
-#endif
         "\t\tDelete resource's history (including failures) so its current state is rechecked.\n"
-        "\t\t\t\tOptionally filtered by --resource and --node (otherwise all). If the named resource is\n"
-        "\t\t\t\tpart of a group, or one numbered instance of a clone or bundled resource, the clean-up\n"
-        "applies to the whole collective resource unless --force is given."
+        "\t\t\t\tOptionally filtered by --resource and --node (otherwise all).\n"
+        "\t\t\t\tUnless --force is specified, resource's group or clone (if any) will also be refreshed."
     },
     {
         "set-parameter", required_argument, NULL, 'p',
@@ -264,7 +241,7 @@ static struct crm_option long_options[] = {
         "\t\t\t\tNOTE: This will prevent the resource from running on the affected node\n"
         "\t\t\t\tuntil the implicit constraint expires or is removed with --clear.\n"
         "\t\t\t\tIf --node is not specified, it defaults to the node currently running the resource\n"
-        "\t\t\t\tfor primitives and groups, or the master for master/slave clones with master-max=1\n"
+        "\t\t\t\tfor primitives and groups, or the master for promotable clones with promoted-max=1\n"
         "\t\t\t\t(all other situations result in an error as there is no sane default).\n"
     },
     {
@@ -363,13 +340,11 @@ static struct crm_option long_options[] = {
     },
     {
         "operation", required_argument, NULL, 'n',
-        "\tOperation to clear instead of all (with -C -r)",
-        pcmk_option_hidden // only used with 2.0 -C behavior
+        "\tOperation to clear instead of all (with -C -r)"
     },
     {
         "interval", required_argument, NULL, 'I',
-        "\tInterval of operation to clear (default 0) (with -C -r -n)",
-        pcmk_option_hidden // only used with 2.0 -C behavior
+        "\tInterval of operation to clear (default 0) (with -C -r -n)"
     },
     {
         "set-name", required_argument, NULL, 's',
@@ -395,12 +370,6 @@ static struct crm_option long_options[] = {
 
     /* legacy options */
     {"host-uname", required_argument, NULL, 'H', NULL, pcmk_option_hidden},
-    {"migrate", no_argument, NULL, 'M', NULL, pcmk_option_hidden},
-    {"un-migrate", no_argument, NULL, 'U', NULL, pcmk_option_hidden},
-    {"un-move", no_argument, NULL, 'U', NULL, pcmk_option_hidden},
-
-    {"refresh",    0, 0, 'R', NULL, pcmk_option_hidden}, // remove this line for 2.0.0
-    {"reprobe", no_argument, NULL, 'P', NULL, pcmk_option_hidden},
 
     {"-spacer-", 1, NULL, '-', "\nExamples:", pcmk_option_paragraph},
     {"-spacer-", 1, NULL, '-', "List the available OCF agents:", pcmk_option_paragraph},
@@ -442,7 +411,7 @@ main(int argc, char **argv)
     const char *rsc_long_cmd = NULL;
     const char *longname = NULL;
     const char *operation = NULL;
-    const char *interval = NULL;
+    const char *interval_spec = NULL;
     const char *cib_file = getenv("CIB_file");
     GHashTable *override_params = NULL;
 
@@ -456,7 +425,7 @@ main(int argc, char **argv)
 
     bool require_resource = TRUE; /* whether command requires that resource be specified */
     bool require_dataset = TRUE;  /* whether command requires populated dataset instance */
-    bool require_crmd = FALSE;    /* whether command requires connection to CRMd */
+    bool require_crmd = FALSE;    // whether command requires controller connection
     bool clear_expired = FALSE;
 
     int rc = pcmk_ok;
@@ -466,6 +435,7 @@ main(int argc, char **argv)
     int argerr = 0;
     int flag;
     int find_flags = 0;           // Flags to use when searching for resource
+    crm_exit_t exit_code = CRM_EX_OK;
 
     crm_log_cli_init("crm_resource");
     crm_set_options(NULL, "(query|command) [options]", long_options,
@@ -522,21 +492,22 @@ main(int argc, char **argv)
                     }
 
                     if (rc > 0) {
-                        rc = 0;
                         for (iter = list; iter != NULL; iter = iter->next) {
-                            rc++;
                             printf("%s\n", iter->val);
                         }
                         lrmd_list_freeall(list);
 
                     } else if (optarg) {
                         fprintf(stderr, "No %s found for %s\n", text, optarg);
+                        exit_code = CRM_EX_NOSUCH;
+
                     } else {
                         fprintf(stderr, "No %s found\n", text);
+                        exit_code = CRM_EX_NOSUCH;
                     }
 
                     lrmd_api_delete(lrmd_conn);
-                    return crm_exit(rc);
+                    crm_exit(exit_code);
 
                 } else if (safe_str_eq("show-metadata", longname)) {
                     char *standard = NULL;
@@ -554,6 +525,7 @@ main(int argc, char **argv)
                         fprintf(stderr,
                                 "'%s' is not a valid agent specification\n",
                                 optarg);
+                        rc = -ENXIO;
                     }
 
                     if (metadata) {
@@ -561,9 +533,10 @@ main(int argc, char **argv)
                     } else {
                         fprintf(stderr, "Metadata query for %s failed: %s\n",
                                 optarg, pcmk_strerror(rc));
+                        exit_code = crm_errno2exit(rc);
                     }
                     lrmd_api_delete(lrmd_conn);
-                    return crm_exit(rc);
+                    crm_exit(exit_code);
 
                 } else if (safe_str_eq("list-agents", longname)) {
                     lrmd_list_t *list = NULL;
@@ -577,20 +550,17 @@ main(int argc, char **argv)
                     rc = lrmd_conn->cmds->list_agents(lrmd_conn, &list, optarg, provider);
 
                     if (rc > 0) {
-                        rc = 0;
                         for (iter = list; iter != NULL; iter = iter->next) {
                             printf("%s\n", iter->val);
-                            rc++;
                         }
                         lrmd_list_freeall(list);
-                        rc = 0;
                     } else {
                         fprintf(stderr, "No agents found for standard=%s, provider=%s\n",
                                 optarg, (provider? provider : "*"));
-                        rc = -1;
+                        exit_code = CRM_EX_NOSUCH;
                     }
                     lrmd_api_delete(lrmd_conn);
-                    return crm_exit(rc);
+                    crm_exit(exit_code);
 
                 } else {
                     crm_err("Unhandled long option: %s", longname);
@@ -602,7 +572,7 @@ main(int argc, char **argv)
                 break;
             case '$':
             case '?':
-                crm_help(flag, EX_OK);
+                crm_help(flag, CRM_EX_OK);
                 break;
             case 'x':
                 xml_file = strdup(optarg);
@@ -648,13 +618,12 @@ main(int argc, char **argv)
 
             case 'C':
             case 'R':
-            case 'P':
                 crm_log_args(argc, argv);
                 require_resource = FALSE;
                 if (cib_file == NULL) {
                     require_crmd = TRUE;
                 }
-                rsc_cmd = 'R'; // disable new behavior until 2.0
+                rsc_cmd = flag;
                 find_flags = pe_find_renamed|pe_find_anon;
                 break;
 
@@ -663,7 +632,7 @@ main(int argc, char **argv)
                 break;
 
             case 'I':
-                interval = optarg;
+                interval_spec = optarg;
                 break;
 
             case 'D':
@@ -715,10 +684,6 @@ main(int argc, char **argv)
                 find_flags = pe_find_renamed|pe_find_anon;
                 break;
 
-            case 'j':
-                print_pending = TRUE;
-                break;
-
             case 'S':
                 require_dataset = FALSE;
                 crm_log_args(argc, argv);
@@ -741,7 +706,7 @@ main(int argc, char **argv)
                 rsc_cmd = flag;
                 find_flags = pe_find_renamed|pe_find_any;
                 break;
-            case 'h':
+
             case 'H':
             case 'N':
                 crm_trace("Option %c => %s", flag, optarg);
@@ -804,7 +769,7 @@ main(int argc, char **argv)
 
     if (argerr) {
         CMD_ERR("Invalid option(s) supplied, use --help for valid usage");
-        return crm_exit(EX_USAGE);
+        crm_exit(CRM_EX_USAGE);
     }
 
     our_pid = crm_getpid_s();
@@ -824,15 +789,11 @@ main(int argc, char **argv)
         require_dataset = TRUE;
     }
 
-    /* Establish a connection to the CIB */
+    /* Establish a connection to the CIB manager */
     cib_conn = cib_new();
-    if ((cib_conn == NULL) || (cib_conn->cmds == NULL)) {
-        CMD_ERR("Could not create CIB connection: %s", pcmk_strerror(rc));
-        goto bail;
-    }
     rc = cib_conn->cmds->signon(cib_conn, crm_system_name, cib_command);
     if (rc != pcmk_ok) {
-        CMD_ERR("Error signing on to the CIB service: %s", pcmk_strerror(rc));
+        CMD_ERR("Error connecting to the CIB manager: %s", pcmk_strerror(rc));
         goto bail;
     }
 
@@ -857,8 +818,6 @@ main(int argc, char **argv)
             rc = -ENOMEM;
             goto bail;
         }
-        set_bit(data_set->flags, pe_flag_no_counts);
-        set_bit(data_set->flags, pe_flag_no_compat);
         rc = update_working_set_xml(data_set, &cib_xml_copy);
         if (rc != pcmk_ok) {
             goto bail;
@@ -877,7 +836,7 @@ main(int argc, char **argv)
         }
     }
 
-    /* Establish a connection to the CRMd if needed */
+    // Establish a connection to the controller if needed
     if (require_crmd) {
         xmlNode *xml = NULL;
         mainloop_io_t *source =
@@ -885,7 +844,7 @@ main(int argc, char **argv)
         crmd_channel = mainloop_get_ipc_client(source);
 
         if (crmd_channel == NULL) {
-            CMD_ERR("Error signing on to the CRMd service");
+            CMD_ERR("Error connecting to the controller");
             rc = -ENOTCONN;
             goto bail;
         }
@@ -1004,8 +963,7 @@ main(int argc, char **argv)
         if (host_uname) {
             dest = pe_find_node(data_set->nodes, host_uname);
             if (dest == NULL) {
-                CMD_ERR("Unknown node: %s", host_uname);
-                rc = -ENXIO;
+                rc = -pcmk_err_node_unknown;
                 goto bail;
             }
         }
@@ -1021,8 +979,7 @@ main(int argc, char **argv)
         } else if (host_uname) {
             dest = pe_find_node(data_set->nodes, host_uname);
             if (dest == NULL) {
-                CMD_ERR("Unknown node: %s", host_uname);
-                rc = -ENXIO;
+                rc = -pcmk_err_node_unknown;
                 goto bail;
             }
             rc = cli_resource_clear(rsc_id, dest->details->uname, NULL, cib_conn, TRUE);
@@ -1038,8 +995,7 @@ main(int argc, char **argv)
         node_t *dest = pe_find_node(data_set->nodes, host_uname);
 
         if (dest == NULL) {
-            CMD_ERR("Error performing operation: node '%s' is unknown", host_uname);
-            rc = -ENXIO;
+            rc = -pcmk_err_node_unknown;
             goto bail;
         }
         rc = cli_resource_ban(rsc_id, dest->details->uname, NULL, cib_conn);
@@ -1048,13 +1004,12 @@ main(int argc, char **argv)
         pe_node_t *current = NULL;
         unsigned int nactive = 0;
 
-        rc = -EINVAL;
         current = pe__find_active_requires(rsc, &nactive);
 
         if (nactive == 1) {
             rc = cli_resource_ban(rsc_id, current->details->uname, NULL, cib_conn);
 
-        } else if(rsc->variant == pe_master) {
+        } else if (is_set(rsc->flags, pe_rsc_promotable)) {
             int count = 0;
             GListPtr iter = NULL;
 
@@ -1073,6 +1028,8 @@ main(int argc, char **argv)
                 rc = cli_resource_ban(rsc_id, current->details->uname, NULL, cib_conn);
 
             } else {
+                rc = -EINVAL;
+                exit_code = CRM_EX_USAGE;
                 CMD_ERR("Resource '%s' not moved: active in %d locations (promoted in %d).",
                         rsc_id, nactive, count);
                 CMD_ERR("To prevent '%s' from running on a specific location, "
@@ -1083,6 +1040,8 @@ main(int argc, char **argv)
             }
 
         } else {
+            rc = -EINVAL;
+            exit_code = CRM_EX_USAGE;
             CMD_ERR("Resource '%s' not moved: active in %d locations.", rsc_id, nactive);
             CMD_ERR("To prevent '%s' from running on a specific location, "
                     "specify a node.", rsc_id);
@@ -1143,7 +1102,7 @@ main(int argc, char **argv)
         crm_debug("Erasing failures of %s (%s requested) on %s",
                   rsc->id, rsc_id, (host_uname? host_uname: "all nodes"));
         rc = cli_resource_delete(crmd_channel, host_uname, rsc,
-                                 operation, interval, TRUE, data_set);
+                                 operation, interval_spec, TRUE, data_set);
 
         if ((rc == pcmk_ok) && !BE_QUIET) {
             // Show any reasons why resource might stay stopped
@@ -1155,34 +1114,33 @@ main(int argc, char **argv)
         }
 
     } else if (rsc_cmd == 'C') {
-        rc = cli_cleanup_all(crmd_channel, host_uname, operation, interval,
+        rc = cli_cleanup_all(crmd_channel, host_uname, operation, interval_spec,
                              data_set);
         if (rc == pcmk_ok) {
             start_mainloop();
         }
 
-     } else if ((rsc_cmd == 'R') && rsc) {
-         if (do_force == FALSE) {
-             rsc = uber_parent(rsc);
-         }
-         crmd_replies_needed = 0;
+    } else if ((rsc_cmd == 'R') && rsc) {
+        if (do_force == FALSE) {
+            rsc = uber_parent(rsc);
+        }
+        crmd_replies_needed = 0;
 
-         crm_debug("Re-checking the state of %s (%s requested) on %s",
-                   rsc->id, rsc_id, (host_uname? host_uname: "all nodes"));
-         rc = cli_resource_delete(crmd_channel, host_uname, rsc,
-                                  NULL, 0, FALSE, data_set);
+        crm_debug("Re-checking the state of %s (%s requested) on %s",
+                  rsc->id, rsc_id, (host_uname? host_uname: "all nodes"));
+        rc = cli_resource_delete(crmd_channel, host_uname, rsc,
+                                 NULL, 0, FALSE, data_set);
 
-         if ((rc == pcmk_ok) && !BE_QUIET) {
-             // Show any reasons why resource might stay stopped
-             cli_resource_check(cib_conn, rsc);
-         }
+        if ((rc == pcmk_ok) && !BE_QUIET) {
+            // Show any reasons why resource might stay stopped
+            cli_resource_check(cib_conn, rsc);
+        }
 
-         if (rc == pcmk_ok) {
-             start_mainloop();
-         }
+        if (rc == pcmk_ok) {
+            start_mainloop();
+        }
 
     } else if (rsc_cmd == 'R') {
-#if HAVE_ATOMIC_ATTRD
         const char *router_node = host_uname;
         xmlNode *msg_data = NULL;
         xmlNode *cmd = NULL;
@@ -1194,7 +1152,8 @@ main(int argc, char **argv)
             if (node && is_remote_node(node)) {
                 node = pe__current_node(node->details->remote_rsc);
                 if (node == NULL) {
-                    CMD_ERR("No lrmd connection detected to remote node %s", host_uname);
+                    CMD_ERR("No cluster connection to Pacemaker Remote node %s detected",
+                            host_uname);
                     rc = -ENXIO;
                     goto bail;
                 }
@@ -1230,18 +1189,6 @@ main(int argc, char **argv)
         }
 
         free_xml(cmd);
-#else
-        GListPtr rIter = NULL;
-
-        crmd_replies_needed = 0;
-        for (rIter = data_set->resources; rIter; rIter = rIter->next) {
-            rsc = rIter->data;
-            cli_resource_delete(crmd_channel, host_uname, rsc, NULL, 0,
-                                FALSE, data_set);
-        }
-
-        start_mainloop();
-#endif
 
     } else if (rsc_cmd == 'D') {
         xmlNode *msg_data = NULL;
@@ -1255,7 +1202,7 @@ main(int argc, char **argv)
         msg_data = create_xml_node(NULL, rsc_type);
         crm_xml_add(msg_data, XML_ATTR_ID, rsc_id);
 
-        rc = cib_conn->cmds->delete(cib_conn, XML_CIB_TAG_RESOURCES, msg_data, cib_options);
+        rc = cib_conn->cmds->remove(cib_conn, XML_CIB_TAG_RESOURCES, msg_data, cib_options);
         free_xml(msg_data);
 
     } else {
@@ -1271,13 +1218,18 @@ main(int argc, char **argv)
         cib_delete(cib_conn);
     }
 
-    if (rc == -pcmk_err_no_quorum) {
-        CMD_ERR("Error performing operation: %s", pcmk_strerror(rc));
-        CMD_ERR("To ignore quorum, use the force option");
+    if (is_ocf_rc) {
+        exit_code = rc;
 
-    } else if (rc != pcmk_ok && !is_ocf_rc) {
+    } else if (rc != pcmk_ok) {
         CMD_ERR("Error performing operation: %s", pcmk_strerror(rc));
+        if (rc == -pcmk_err_no_quorum) {
+            CMD_ERR("To ignore quorum, use the force option");
+        }
+        if (exit_code == CRM_EX_OK) {
+            exit_code = crm_errno2exit(rc);
+        }
     }
 
-    return crm_exit(rc);
+    return crm_exit(exit_code);
 }
