@@ -1,20 +1,8 @@
-
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_resource.h>
@@ -182,9 +170,8 @@ find_matching_attr_resource(resource_t * rsc, const char * rsc_id, const char *
                     printf("Performing %s of '%s' for '%s' will not apply to its peers in '%s'\n", cmd, attr_name, rsc_id, rsc->parent->id);
                 }
                 break;
-            case pe_master:
-            case pe_clone:
 
+            case pe_clone:
                 rc = find_resource_attr(cib, XML_ATTR_ID, rsc_id, attr_set_type, attr_set, attr_id, attr_name, &local_attr_id);
                 free(local_attr_id);
 
@@ -241,12 +228,10 @@ cli_resource_update_attribute(resource_t *rsc, const char *requested_name,
     xmlNode *xml_top = NULL;
     xmlNode *xml_obj = NULL;
 
-    bool use_attributes_tag = FALSE;
-
     if(attr_id == NULL
        && do_force == FALSE
-       && pcmk_ok != find_resource_attr(
-           cib, XML_ATTR_ID, uber_parent(rsc)->id, NULL, NULL, NULL, attr_name, NULL)) {
+       && find_resource_attr(
+           cib, XML_ATTR_ID, uber_parent(rsc)->id, NULL, NULL, NULL, attr_name, NULL) == -EINVAL) {
         printf("\n");
     }
 
@@ -286,24 +271,8 @@ cli_resource_update_attribute(resource_t *rsc, const char *requested_name,
         return rc;
 
     } else {
-        const char *value = NULL;
-        xmlNode *cib_top = NULL;
         const char *tag = crm_element_name(rsc->xml);
 
-        cib->cmds->query(cib, "/cib", &cib_top,
-                              cib_sync_call | cib_scope_local | cib_xpath | cib_no_children);
-        value = crm_element_value(cib_top, "ignore_dtd");
-        if (value != NULL) {
-            use_attributes_tag = TRUE;
-
-        } else {
-            value = crm_element_value(cib_top, XML_ATTR_VALIDATION);
-            if (crm_ends_with_ext(value, "-0.6")) {
-                use_attributes_tag = TRUE;
-            }
-        }
-        free_xml(cib_top);
-
         if (attr_set == NULL) {
             local_attr_set = crm_concat(lookup_id, attr_set_type, '-');
             attr_set = local_attr_set;
@@ -313,19 +282,11 @@ cli_resource_update_attribute(resource_t *rsc, const char *requested_name,
             attr_id = local_attr_id;
         }
 
-        if (use_attributes_tag && safe_str_eq(tag, XML_CIB_TAG_MASTER)) {
-            tag = "master_slave";       /* use the old name */
-        }
-
         xml_top = create_xml_node(NULL, tag);
         crm_xml_add(xml_top, XML_ATTR_ID, lookup_id);
 
         xml_obj = create_xml_node(xml_top, attr_set_type);
         crm_xml_add(xml_obj, XML_ATTR_ID, attr_set);
-
-        if (use_attributes_tag) {
-            xml_obj = create_xml_node(xml_obj, XML_TAG_ATTRS);
-        }
     }
 
     xml_obj = crm_create_nvpair_xml(xml_obj, attr_id, attr_name, attr_value);
@@ -337,7 +298,7 @@ cli_resource_update_attribute(resource_t *rsc, const char *requested_name,
 
     rc = cib->cmds->modify(cib, XML_CIB_TAG_RESOURCES, xml_top, cib_options);
     if (rc == pcmk_ok && BE_QUIET == FALSE) {
-        printf("Set '%s' option: id=%s%s%s%s%s=%s\n", lookup_id, local_attr_id,
+        printf("Set '%s' option: id=%s%s%s%s%s value=%s\n", lookup_id, local_attr_id,
                attr_set ? " set=" : "", attr_set ? attr_set : "",
                attr_name ? " name=" : "", attr_name ? attr_name : "", attr_value);
     }
@@ -399,7 +360,7 @@ cli_resource_delete_attribute(resource_t *rsc, const char *requested_name,
     if(attr_id == NULL
        && do_force == FALSE
        && find_resource_attr(
-           cib, XML_ATTR_ID, uber_parent(rsc)->id, NULL, NULL, NULL, attr_name, NULL) != pcmk_ok) {
+           cib, XML_ATTR_ID, uber_parent(rsc)->id, NULL, NULL, NULL, attr_name, NULL) == -EINVAL) {
         printf("\n");
     }
 
@@ -429,7 +390,7 @@ cli_resource_delete_attribute(resource_t *rsc, const char *requested_name,
     crm_log_xml_debug(xml_obj, "Delete");
 
     CRM_ASSERT(cib);
-    rc = cib->cmds->delete(cib, XML_CIB_TAG_RESOURCES, xml_obj, cib_options);
+    rc = cib->cmds->remove(cib, XML_CIB_TAG_RESOURCES, xml_obj, cib_options);
 
     if (rc == pcmk_ok && BE_QUIET == FALSE) {
         printf("Deleted '%s' option: id=%s%s%s%s%s\n", lookup_id, local_attr_id,
@@ -453,9 +414,8 @@ send_lrm_rsc_op(crm_ipc_t * crmd_channel, const char *op,
     int rc = -ECOMM;
     xmlNode *cmd = NULL;
     xmlNode *xml_rsc = NULL;
+    const char *value = NULL;
     const char *router_node = host_uname;
-    const char *rsc_class = NULL;
-    const char *rsc_type = NULL;
     xmlNode *params = NULL;
     xmlNode *msg_data = NULL;
     resource_t *rsc = pe_find_resource(data_set->resources, rsc_id);
@@ -467,48 +427,27 @@ send_lrm_rsc_op(crm_ipc_t * crmd_channel, const char *op,
     } else if (rsc->variant != pe_native) {
         CMD_ERR("We can only process primitive resources, not %s", rsc_id);
         return -EINVAL;
-    }
 
-    rsc_class = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
-    rsc_type = crm_element_value(rsc->xml, XML_ATTR_TYPE);
-    if ((rsc_class == NULL) || (rsc_type == NULL)) {
-        CMD_ERR("Resource %s does not have a class and type", rsc_id);
-        return -EINVAL;
-    }
-
-    if (host_uname == NULL) {
+    } else if (host_uname == NULL) {
         CMD_ERR("Please specify a node name");
         return -EINVAL;
-
     } else {
-        pe_node_t *node = pe_find_node(data_set->nodes, host_uname);
+        node_t *node = pe_find_node(data_set->nodes, host_uname);
 
-        if (node == NULL) {
-            CMD_ERR("Node %s not found", host_uname);
-            return -pcmk_err_node_unknown;
-        }
-
-        if (!(node->details->online)) {
-            CMD_ERR("Node %s is not online", host_uname);
-            return -ENOTCONN;
-        }
         if (node && is_remote_node(node)) {
             node = pe__current_node(node->details->remote_rsc);
             if (node == NULL) {
-                CMD_ERR("No lrmd connection detected to remote node %s", host_uname);
-                return -ENOTCONN;
+                CMD_ERR("No cluster connection to Pacemaker Remote node %s detected",
+                        host_uname);
+                return -ENXIO;
             }
             router_node = node->details->uname;
         }
     }
 
-    msg_data = create_xml_node(NULL, XML_GRAPH_TAG_RSC_OP);
+    key = generate_transition_key(0, getpid(), 0, "xxxxxxxx-xrsc-opxx-xcrm-resourcexxxx");
 
-    /* The controller logs the transition key from requests, so we need to have
-     * *something* for it.
-     */
-    key = generate_transition_key(0, getpid(), 0,
-                                  "xxxxxxxx-xrsc-opxx-xcrm-resourcexxxx");
+    msg_data = create_xml_node(NULL, XML_GRAPH_TAG_RSC_OP);
     crm_xml_add(msg_data, XML_ATTR_TRANSITION_KEY, key);
     free(key);
 
@@ -526,27 +465,38 @@ send_lrm_rsc_op(crm_ipc_t * crmd_channel, const char *op,
         crm_xml_add(xml_rsc, XML_ATTR_ID, rsc->id);
     }
 
-    crm_xml_add(xml_rsc, XML_AGENT_ATTR_CLASS, rsc_class);
+    value = crm_copy_xml_element(rsc->xml, xml_rsc, XML_ATTR_TYPE);
+    if (value == NULL) {
+        CMD_ERR("%s has no type!  Aborting...", rsc_id);
+        return -ENXIO;
+    }
+
+    value = crm_copy_xml_element(rsc->xml, xml_rsc, XML_AGENT_ATTR_CLASS);
+    if (value == NULL) {
+        CMD_ERR("%s has no class!  Aborting...", rsc_id);
+        return -ENXIO;
+    }
+
     crm_copy_xml_element(rsc->xml, xml_rsc, XML_AGENT_ATTR_PROVIDER);
-    crm_xml_add(xml_rsc, XML_ATTR_TYPE, rsc_type);
 
     params = create_xml_node(msg_data, XML_TAG_ATTRS);
     crm_xml_add(params, XML_ATTR_CRM_VERSION, CRM_FEATURE_SET);
 
-    // The controller parses the timeout from the request
-    key = crm_meta_name(XML_ATTR_TIMEOUT);
+    key = crm_meta_name(XML_LRM_ATTR_INTERVAL_MS);
     crm_xml_add(params, key, "60000");  /* 1 minute */
     free(key);
 
     our_pid = crm_getpid_s();
     cmd = create_request(op, msg_data, router_node, CRM_SYSTEM_CRMD, crm_system_name, our_pid);
+
+/* 	crm_log_xml_warn(cmd, "send_lrm_rsc_op"); */
     free_xml(msg_data);
 
     if (crm_ipc_send(crmd_channel, cmd, 0, 0, NULL) > 0) {
         rc = 0;
 
     } else {
-        crm_debug("Could not send %s op to the crmd", op);
+        crm_debug("Could not send %s op to the controller", op);
         rc = -ENOTCONN;
     }
 
@@ -604,12 +554,12 @@ clear_rsc_history(crm_ipc_t *crmd_channel, const char *host_uname,
 static int
 clear_rsc_failures(crm_ipc_t *crmd_channel, const char *node_name,
                    const char *rsc_id, const char *operation,
-                   const char *interval, pe_working_set_t *data_set)
+                   const char *interval_spec, pe_working_set_t *data_set)
 {
     int rc = pcmk_ok;
     const char *failed_value = NULL;
     const char *failed_id = NULL;
-    const char *interval_ms_str = NULL;
+    const char *interval_ms_s = NULL;
     GHashTable *rscs = NULL;
     GHashTableIter iter;
 
@@ -621,7 +571,8 @@ clear_rsc_failures(crm_ipc_t *crmd_channel, const char *node_name,
 
     // Normalize interval to milliseconds for comparison to history entry
     if (operation) {
-        interval_ms_str = crm_strdup_printf("%llu", crm_get_interval(interval));
+        interval_ms_s = crm_strdup_printf("%u",
+                                          crm_parse_interval_spec(interval_spec));
     }
 
     for (xmlNode *xml_op = __xml_first_child(data_set->failed); xml_op != NULL;
@@ -658,8 +609,8 @@ clear_rsc_failures(crm_ipc_t *crmd_channel, const char *node_name,
             }
 
             // Interval (if operation was specified) defaults to 0 (not all)
-            failed_value = crm_element_value(xml_op, XML_LRM_ATTR_INTERVAL);
-            if (safe_str_neq(interval_ms_str, failed_value)) {
+            failed_value = crm_element_value(xml_op, XML_LRM_ATTR_INTERVAL_MS);
+            if (safe_str_neq(interval_ms_s, failed_value)) {
                 continue;
             }
         }
@@ -684,7 +635,7 @@ clear_rsc_failures(crm_ipc_t *crmd_channel, const char *node_name,
 
 static int
 clear_rsc_fail_attrs(resource_t *rsc, const char *operation,
-                     const char *interval, node_t *node)
+                     const char *interval_spec, node_t *node)
 {
     int rc = pcmk_ok;
     int attr_options = attrd_opt_none;
@@ -694,7 +645,7 @@ clear_rsc_fail_attrs(resource_t *rsc, const char *operation,
         attr_options |= attrd_opt_remote;
     }
     rc = attrd_clear_delegate(NULL, node->details->uname, rsc_name, operation,
-                              interval, NULL, attr_options);
+                              interval_spec, NULL, attr_options);
     free(rsc_name);
     return rc;
 }
@@ -702,7 +653,7 @@ clear_rsc_fail_attrs(resource_t *rsc, const char *operation,
 int
 cli_resource_delete(crm_ipc_t *crmd_channel, const char *host_uname,
                     resource_t *rsc, const char *operation,
-                    const char *interval, bool just_failures,
+                    const char *interval_spec, bool just_failures,
                     pe_working_set_t *data_set)
 {
     int rc = pcmk_ok;
@@ -718,7 +669,7 @@ cli_resource_delete(crm_ipc_t *crmd_channel, const char *host_uname,
             resource_t *child = (resource_t *) lpc->data;
 
             rc = cli_resource_delete(crmd_channel, host_uname, child, operation,
-                                     interval, just_failures, data_set);
+                                     interval_spec, just_failures, data_set);
             if (rc != pcmk_ok) {
                 return rc;
             }
@@ -752,7 +703,7 @@ cli_resource_delete(crm_ipc_t *crmd_channel, const char *host_uname,
 
             if (node->details->online) {
                 rc = cli_resource_delete(crmd_channel, node->details->uname,
-                                         rsc, operation, interval,
+                                         rsc, operation, interval_spec,
                                          just_failures, data_set);
             }
             if (rc != pcmk_ok) {
@@ -785,7 +736,7 @@ cli_resource_delete(crm_ipc_t *crmd_channel, const char *host_uname,
         return pcmk_ok;
     }
 
-    rc = clear_rsc_fail_attrs(rsc, operation, interval, node);
+    rc = clear_rsc_fail_attrs(rsc, operation, interval_spec, node);
     if (rc != pcmk_ok) {
         printf("Unable to clean up %s failures on %s: %s\n",
                 rsc->id, host_uname, pcmk_strerror(rc));
@@ -794,7 +745,7 @@ cli_resource_delete(crm_ipc_t *crmd_channel, const char *host_uname,
 
     if (just_failures) {
         rc = clear_rsc_failures(crmd_channel, host_uname, rsc->id, operation,
-                                interval, data_set);
+                                interval_spec, data_set);
     } else {
         rc = clear_rsc_history(crmd_channel, host_uname, rsc->id, data_set);
     }
@@ -809,7 +760,7 @@ cli_resource_delete(crm_ipc_t *crmd_channel, const char *host_uname,
 
 int
 cli_cleanup_all(crm_ipc_t *crmd_channel, const char *node_name,
-                const char *operation, const char *interval,
+                const char *operation, const char *interval_spec,
                 pe_working_set_t *data_set)
 {
     int rc = pcmk_ok;
@@ -835,7 +786,7 @@ cli_cleanup_all(crm_ipc_t *crmd_channel, const char *node_name,
         }
     }
 
-    rc = attrd_clear_delegate(NULL, node_name, NULL, operation, interval,
+    rc = attrd_clear_delegate(NULL, node_name, NULL, operation, interval_spec,
                               NULL, attr_options);
     if (rc != pcmk_ok) {
         printf("Unable to clean up all failures on %s: %s\n",
@@ -845,7 +796,7 @@ cli_cleanup_all(crm_ipc_t *crmd_channel, const char *node_name,
 
     if (node_name) {
         rc = clear_rsc_failures(crmd_channel, node_name, NULL,
-                                operation, interval, data_set);
+                                operation, interval_spec, data_set);
         if (rc != pcmk_ok) {
             printf("Cleaned all resource failures on %s, but unable to clean history: %s\n",
                    node_name, pcmk_strerror(rc));
@@ -856,7 +807,7 @@ cli_cleanup_all(crm_ipc_t *crmd_channel, const char *node_name,
             pe_node_t *node = (pe_node_t *) iter->data;
 
             rc = clear_rsc_failures(crmd_channel, node->details->uname, NULL,
-                                    operation, interval, data_set);
+                                    operation, interval_spec, data_set);
             if (rc != pcmk_ok) {
                 printf("Cleaned all resource failures on all nodes, but unable to clean history: %s\n",
                        pcmk_strerror(rc));
@@ -872,7 +823,7 @@ cli_cleanup_all(crm_ipc_t *crmd_channel, const char *node_name,
 void
 cli_resource_check(cib_t * cib_conn, resource_t *rsc)
 {
-    bool printed = false;
+    int need_nl = 0;
     char *role_s = NULL;
     char *managed = NULL;
     resource_t *parent = uber_parent(rsc);
@@ -885,29 +836,29 @@ cli_resource_check(cib_t * cib_conn, resource_t *rsc)
 
     if(role_s) {
         enum rsc_role_e role = text2role(role_s);
+
+        free(role_s);
         if(role == RSC_ROLE_UNKNOWN) {
             // Treated as if unset
 
         } else if(role == RSC_ROLE_STOPPED) {
-            printf("\n  * Configuration specifies '%s' should remain stopped\n",
-                   parent->id);
-            printed = true;
+            printf("\n  * The configuration specifies that '%s' should remain stopped\n", parent->id);
+            need_nl++;
 
-        } else if(parent->variant == pe_master && role == RSC_ROLE_SLAVE) {
-            printf("\n  * Configuration specifies '%s' should not be promoted\n",
-                   parent->id);
-            printed = true;
+        } else if (is_set(parent->flags, pe_rsc_promotable)
+                   && (role == RSC_ROLE_SLAVE)) {
+            printf("\n  * The configuration specifies that '%s' should not be promoted\n", parent->id);
+            need_nl++;
         }
     }
 
-    if (managed && !crm_is_true(managed)) {
-        printf("%s  * Configuration prevents cluster from stopping or starting unmanaged '%s'\n",
-               (printed? "" : "\n"), parent->id);
-        printed = true;
+    if(managed && crm_is_true(managed) == FALSE) {
+        printf("%s  * The configuration prevents the cluster from stopping or starting '%s' (unmanaged)\n", need_nl == 0?"\n":"", parent->id);
+        need_nl++;
     }
     free(managed);
 
-    if (printed) {
+    if(need_nl) {
         printf("\n");
     }
 }
@@ -1046,7 +997,8 @@ subtract_lists(GList *from, GList *items)
     for (item = items; item != NULL; item = item->next) {
         GList *candidate = NULL;
         for (candidate = from; candidate != NULL; candidate = candidate->next) {
-            crm_info("Comparing %s with %s", candidate->data, item->data);
+            crm_info("Comparing %s with %s", (const char *) candidate->data,
+                     (const char *) item->data);
             if(strcmp(candidate->data, item->data) == 0) {
                 result = g_list_remove(result, candidate->data);
                 break;
@@ -1109,7 +1061,7 @@ update_working_set_xml(pe_working_set_t *data_set, xmlNode **xml)
  * \brief Update a working set's XML input based on a CIB query
  *
  * \param[in] data_set   Data set instance to initialize
- * \param[in] cib        Connection to the CIB
+ * \param[in] cib        Connection to the CIB manager
  *
  * \return pcmk_ok on success, -errno on failure
  * \note On success, caller is responsible for freeing memory allocated for
@@ -1262,7 +1214,7 @@ max_delay_in(pe_working_set_t * data_set, GList *resources)
  *                       (specified in milliseconds, but a two-second
  *                       granularity is actually used; if 0, a timeout will be
  *                       calculated based on the resource timeout)
- * \param[in] cib        Connection to the CIB for modifying/checking resource
+ * \param[in] cib        Connection to the CIB manager
  *
  * \return pcmk_ok on success, -errno on failure (exits on certain failures)
  */
@@ -1330,8 +1282,6 @@ cli_resource_restart(pe_resource_t *rsc, const char *host, int timeout_ms,
         rc = -ENOMEM;
         goto done;
     }
-    set_bit(data_set->flags, pe_flag_no_counts);
-    set_bit(data_set->flags, pe_flag_no_compat);
     rc = update_dataset(cib, data_set, FALSE);
     if(rc != pcmk_ok) {
         fprintf(stdout, "Could not get new resource list: %s (%d)\n", pcmk_strerror(rc), rc);
@@ -1619,7 +1569,7 @@ print_pending_actions(GListPtr actions)
  * \param[in] timeout_ms Consider failed if actions do not complete in this time
  *                       (specified in milliseconds, but one-second granularity
  *                       is actually used; if 0, a default will be used)
- * \param[in] cib        Connection to the CIB
+ * \param[in] cib        Connection to the CIB manager
  *
  * \return pcmk_ok on success, -errno on failure
  */
@@ -1637,15 +1587,13 @@ wait_till_stable(int timeout_ms, cib_t * cib)
     if (data_set == NULL) {
         return -ENOMEM;
     }
-    set_bit(data_set->flags, pe_flag_no_counts);
-    set_bit(data_set->flags, pe_flag_no_compat);
 
     do {
 
         /* Abort if timeout is reached */
         time_diff = expire_time - time(NULL);
         if (time_diff > 0) {
-            crm_info("Waiting up to %d seconds for cluster actions to complete", time_diff);
+            crm_info("Waiting up to %ld seconds for cluster actions to complete", time_diff);
         } else {
             print_pending_actions(data_set->actions);
             pe_free_working_set(data_set);
@@ -1670,8 +1618,8 @@ wait_till_stable(int timeout_ms, cib_t * cib)
              * done. Warn the user in this case.
              *
              * @TODO A possible long-term solution would be to reimplement the
-             * wait as a new crmd operation that would be forwarded to the DC.
-             * However, that would have potential problems of its own.
+             * wait as a new controller operation that would be forwarded to the
+             * DC. However, that would have potential problems of its own.
              */
             const char *dc_version = g_hash_table_lookup(data_set->config_hash,
                                                          "dc-version");
@@ -1724,7 +1672,7 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
                         action, rsc->id);
                 CMD_ERR("Try setting target-role=Stopped first or specifying "
                         "the force option");
-                crm_exit(EPERM);
+                crm_exit(CRM_EX_UNSAFE);
             }
         }
     }
@@ -1737,7 +1685,7 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
     if(rsc->variant == pe_group) {
         CMD_ERR("Sorry, the %s option doesn't support group resources",
                 rsc_action);
-        crm_exit(EOPNOTSUPP);
+        crm_exit(CRM_EX_UNIMPLEMENT_FEATURE);
     }
 
     rclass = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
@@ -1747,7 +1695,7 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
     if (safe_str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH)) {
         CMD_ERR("Sorry, the %s option doesn't support %s resources yet",
                 rsc_action, rclass);
-        crm_exit(EOPNOTSUPP);
+        crm_exit(CRM_EX_UNIMPLEMENT_FEATURE);
     }
 
     params = generate_resource_params(rsc, data_set);
@@ -1775,7 +1723,7 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
         /* We know op will be NULL, but this makes static analysis happy */
         services_action_free(op);
 
-        return crm_exit(EINVAL);
+        return crm_exit(CRM_EX_DATAERR);
     }
 
 
@@ -1860,34 +1808,33 @@ int
 cli_resource_move(resource_t *rsc, const char *rsc_id, const char *host_name,
                   cib_t *cib, pe_working_set_t *data_set)
 {
-    int rc = -EINVAL;
+    int rc = pcmk_ok;
     unsigned int count = 0;
     node_t *current = NULL;
     node_t *dest = pe_find_node(data_set->nodes, host_name);
     bool cur_is_dest = FALSE;
 
     if (dest == NULL) {
-        CMD_ERR("Error performing operation: node '%s' is unknown", host_name);
-        return -ENXIO;
+        return -pcmk_err_node_unknown;
     }
 
-    if (scope_master && rsc->variant != pe_master) {
+    if (scope_master && is_not_set(rsc->flags, pe_rsc_promotable)) {
         resource_t *p = uber_parent(rsc);
-        if(p->variant == pe_master) {
+
+        if (is_set(p->flags, pe_rsc_promotable)) {
             CMD_ERR("Using parent '%s' for move instead of '%s'.", rsc->id, rsc_id);
             rsc_id = p->id;
             rsc = p;
 
         } else {
-            CMD_ERR("Ignoring master option: not valid for %s resources.",
-                    get_resource_typename(rsc->variant));
+            CMD_ERR("Ignoring master option: %s is not promotable", rsc_id);
             scope_master = FALSE;
         }
     }
 
     current = pe__find_active_requires(rsc, &count);
 
-    if(rsc->variant == pe_master) {
+    if (is_set(rsc->flags, pe_rsc_promotable)) {
         GListPtr iter = NULL;
         unsigned int master_count = 0;
         pe_node_t *master_node = NULL;
@@ -1913,8 +1860,7 @@ cli_resource_move(resource_t *rsc, const char *rsc_id, const char *host_name,
         if (pe_rsc_is_clone(rsc)) {
             current = NULL;
         } else {
-            CMD_ERR("Resource '%s' not moved: active on multiple nodes", rsc_id);
-            return rc;
+            return -pcmk_err_multiple;
         }
     }
 
@@ -1924,9 +1870,7 @@ cli_resource_move(resource_t *rsc, const char *rsc_id, const char *host_name,
             crm_info("%s is already %s on %s, reinforcing placement with location constraint.",
                      rsc_id, scope_master?"promoted":"active", dest->details->uname);
         } else {
-            CMD_ERR("Error performing operation: %s is already %s on %s",
-                    rsc_id, scope_master?"promoted":"active", dest->details->uname);
-            return rc;
+            return -pcmk_err_already;
         }
     }
 
