@@ -1,19 +1,8 @@
-/* 
+/*
  * Copyright 2005-2019 Andrew Beekhof <andrew@beekhof.net>
- * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- * 
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -34,10 +23,15 @@ static struct crm_option long_options[] = {
     {"-spacer-",    0, 0, '-', "\nCommands:"},
     {"now",      0, 0, 'n', "\tDisplay the current date/time"},
     {"date",     1, 0, 'd', "Parse an ISO8601 date/time.  Eg. '2005-01-20 00:30:00 +01:00' or '2005-040'"},
-    {"period",   1, 0, 'p', "Parse an ISO8601 date/time with interval/period (wth start time).  Eg. '2005-040/2005-043'"},
-    {"duration", 1, 0, 'D', "Parse an ISO8601 date/time with duration (wth start time). Eg. '2005-040/P1M'"},
-    {"expected", 1, 0, 'E', "Parse an ISO8601 date/time with duration (wth start time). Eg. '2005-040/P1M'"},
-
+    { "period",   1, 0, 'p',
+      "Parse an ISO8601 period (interval) with start time (for example, '2005-040/2005-043')"
+    },
+    { "duration", 1, 0, 'D',
+      "Parse an ISO8601 duration with start time (for example, '2005-040/P1M')"
+    },
+    { "expected", 1, 0, 'E',
+      "Parse an ISO8601 duration with start time (for example, '2005-040/P1M')"
+    },
     {"-spacer-",0, 0, '-', "\nOutput Modifiers:"},
     {"seconds", 0, 0, 's', "\tShow result as a seconds since 0000-001 00:00:00Z"},
     {"epoch", 0, 0, 'S', "\tShow result as a seconds since EPOCH (1970-001 00:00:00Z)"},
@@ -69,14 +63,14 @@ log_time_period(int log_level, crm_time_period_t * dtp, int flags)
 int
 main(int argc, char **argv)
 {
-    int rc = 0;
+    crm_exit_t exit_code = CRM_EX_OK;
     int argerr = 0;
     int flag;
     int index = 0;
     int print_options = 0;
     crm_time_t *duration = NULL;
     crm_time_t *date_time = NULL;
-    crm_time_period_t *interval = NULL;
+    crm_time_period_t *period = NULL;
 
     const char *period_s = NULL;
     const char *duration_s = NULL;
@@ -102,7 +96,7 @@ main(int argc, char **argv)
                 break;
             case '?':
             case '$':
-                return crm_help(flag, 0);
+                return crm_help(flag, CRM_EX_OK);
                 break;
             case 'n':
                 date_time_s = "now";
@@ -143,7 +137,7 @@ main(int argc, char **argv)
 
         if (date_time == NULL) {
             fprintf(stderr, "Internal error: couldn't determine 'now'!\n");
-            return crm_help('?', 1);
+            crm_exit(CRM_EX_SOFTWARE);
         }
         crm_time_log(LOG_TRACE, "Current date/time", date_time,
                      crm_time_ordinal | crm_time_log_date | crm_time_log_timeofday);
@@ -155,7 +149,7 @@ main(int argc, char **argv)
 
         if (date_time == NULL) {
             fprintf(stderr, "Invalid date/time specified: %s\n", optarg);
-            return crm_help('?', 1);
+            return crm_help('?', CRM_EX_USAGE);
         }
         crm_time_log(LOG_TRACE, "Date", date_time,
                      crm_time_ordinal | crm_time_log_date | crm_time_log_timeofday);
@@ -168,22 +162,23 @@ main(int argc, char **argv)
 
         if (duration == NULL) {
             fprintf(stderr, "Invalid duration specified: %s\n", duration_s);
-            return crm_help('?', 1);
+            return crm_help('?', CRM_EX_USAGE);
         }
         crm_time_log(LOG_TRACE, "Duration", duration, crm_time_log_duration);
         crm_time_log(-1, "Duration", duration, print_options | crm_time_log_duration);
     }
 
     if (period_s) {
-        interval = crm_time_parse_period(period_s);
+        period = crm_time_parse_period(period_s);
 
-        if (interval == NULL) {
+        if (period == NULL) {
             fprintf(stderr, "Invalid interval specified: %s\n", optarg);
-            return crm_help('?', 1);
+            return crm_help('?', CRM_EX_USAGE);
         }
-        log_time_period(LOG_TRACE, interval,
+        log_time_period(LOG_TRACE, period,
+                        print_options | crm_time_log_date | crm_time_log_timeofday);
+        log_time_period(-1, period,
                         print_options | crm_time_log_date | crm_time_log_timeofday);
-        log_time_period(-1, interval, print_options | crm_time_log_date | crm_time_log_timeofday);
     }
 
     if (date_time && duration) {
@@ -199,7 +194,7 @@ main(int argc, char **argv)
                                             print_options | crm_time_log_date |
                                             crm_time_log_timeofday);
             if (safe_str_neq(expected_s, dt_s)) {
-                rc = 1;
+                exit_code = CRM_EX_ERROR;
             }
             free(dt_s);
         }
@@ -210,22 +205,19 @@ main(int argc, char **argv)
                                         print_options | crm_time_log_date | crm_time_log_timeofday);
 
         if (safe_str_neq(expected_s, dt_s)) {
-            rc = 1;
+            exit_code = CRM_EX_ERROR;
         }
         free(dt_s);
     }
 
-    /* if(date_time && interval) { */
-    /* } */
-
     crm_time_free(date_time);
     crm_time_free(duration);
-    if (interval) {
-        crm_time_free(interval->start);
-        crm_time_free(interval->end);
-        crm_time_free(interval->diff);
-        free(interval);
+    if (period) {
+        crm_time_free(period->start);
+        crm_time_free(period->end);
+        crm_time_free(period->diff);
+        free(period);
     }
 
-    return crm_exit(rc);
+    return crm_exit(exit_code);
 }
